% Load-balancing scheme 3a
%  'W-cycle' step size in theta
%  Nlevels direction decided from loadbalance



function [at nlevels theta] = at3a(at, newtime, iter, loadbalance)
  at.smoothcount = at.smoothcount + 1;
  
  at.shorthist(at.smoothcount) = newtime;  
  at.athist(at.smoothcount) = loadbalance;

  nlevels = at.nlevels;
  theta = at.theta;

  at.levwait = at.levwait-1;
  at.thetwait = at.thetwait-1;

  if(at.smoothcount == at.smoothiters)
    at.smoothcount = 0;
    smoothtime = min(at.shorthist);
    [loadbalance li] = min(abs(at.athist)); 
    loadbalance = sign(at.athist(li))*loadbalance;               
    
    % If I came from a better place in nlevels, go back
    if(at.levprev < smoothtime)
       at.levprev = inf;     
       at.levmove = -1 * at.levmove;
       nlevels = nlevels + at.levmove;
    else  % new place is better
       at.levprev = inf; 
       if at.levwait < 1 % new move
          at.levwait = at.levinterval;
          % Use loadbalance to set the direction of changes in nlevels
          if(loadbalance > 0) % gpu does more work
            at.levmove = 1; % Grow fmm-tree, give cpu more work
          elseif(loadbalance < 0)
            at.levmove = -1; % Shrink tree, give gpu more work
          else % can't say anything from loadbalance
            at.levmove = at.levmove; % continue in this direction
          end       
          nlevels = nlevels + at.levmove;
          at.levprev = smoothtime;
       end
    end 
    % If I came from a better place in theta, go back
    if(at.thetprev < smoothtime)
       at.thetprev = inf;     
       at.thetmove = -1 * at.thetmove;
       theta = theta + at.thetstep*at.thetunit*at.thetmove;
       at.thetkeep = 0;
       if(at.fibnum < at.fibiters)
          at.thetstep = at.f1+at.f2;
          at.f1 = at.f2;
          at.f2 = at.thetstep;
          at.fibnum = at.fibnum+1;
       else
          at.fibiters = at.fibiters + 1;
          at.f1 = 0;
          at.f2 = 1;
          at.fibnum = 0;
          at.thetstep = 1;
       end
    else  % new place is better
       at.thetprev = inf;
       if at.thetwait < 1
          at.thetwait = at.thetinterval;
          if at.thetkeep == 1      % I went the right way last time
             at.fibiters = 1;
             at.f1 = 0;
             at.f2 = 1;
             at.fibnum = 0;
             at.thetstep = 1;
          end
          at.thetkeep = 1;          
          theta = theta + at.thetmove*at.thetstep*at.thetunit;
          at.thetprev = smoothtime;
       end
    end 
  end
  
  if(nlevels < 2)
     nlevels = 2;
  end
  if(theta < 0.2)
    theta = 0.2;
  elseif(theta > 0.8)
    theta = 0.8;
  end
  at.nlevels = nlevels;
  at.theta = theta;

