% Load-balancing scheme 3b
%  'W-cycle' step size in theta
%  Nlevels frequency set by cost schedules


function [at nlevels theta] = at3b(at, newtime, iter, loadbalance)
  at.smoothcount = at.smoothcount + 1;
  
  at.shorthist(at.smoothcount) = newtime; 

  nlevels = at.nlevels;
  theta = at.theta;
  
  at.levupinterval = at.levupinterval - 1;
  at.levdowninterval = at.levdowninterval - 1;
  at.thetwait = at.thetwait - 1;

  maxcost = (at.nlevprop)*0.5;

  if(at.smoothcount == at.smoothiters)
    at.smoothcount = 0;
    smoothtime = min(at.shorthist);
    % If I came from a better place in nlevels, go back
    if(at.levprev < smoothtime)
%loadbalance
%nlevels
       
       at.levprev = inf;     
       at.levmove = -1 * at.levmove;
       nlevels = nlevels + at.levmove;
       at.levkeep = 0;
       newofftime = sum(at.shorthist) - at.basetime/iter*at.smoothiters;
       if at.levmove == -1 % set penalty for levup
         at.offtimeup = at.offtimeup + newofftime; 
         maxtimenext = (at.offtimeup + newofftime - maxcost*at.basetime)/maxcost;
         at.levupinterval = maxtimenext * iter / at.basetime;
         at.levupinterval = floor(at.levupinterval/at.smoothiters)*at.smoothiters; 
         at.levupinterval = max(at.levupinterval, 11); % minimum interval 
         at.levupinterval = min(at.levupinterval, 497); % max interval
       else
         at.offtimedown = at.offtimedown + newofftime;        
         maxtimenext =(at.offtimedown + newofftime - maxcost*at.basetime)/maxcost; % safe estimate
         at.levdowninterval = maxtimenext * iter / at.basetime;
         at.levdowninterval = floor(at.levdowninterval/at.smoothiters)*at.smoothiters; 
         at.levdowninterval = max(at.levdowninterval, 11); % minimum interval 
         at.levdowninterval = min(at.levdowninterval, 497);
       end
    else  % stay in new place
       at.basetime = at.basetime + sum(at.shorthist);
       if(at.levkeep == 1)  % successful step. Update wait times, will trying to continue soon, should also try to go back soon.
          if at.levmove == -1  % keep going down soon.
            at.levdowninterval = 3*at.smoothiters;
            %at.levupinterval = 3*at.levdowninterval;
          else
            at.levupinterval = 3*at.smoothiters;
           % at.levdowninterval = 3*at.levupinterval;
          end
          at.levkeep = 0; % don't want to keep resetting these intervals.
       end
       if at.levupinterval < 1
          at.levmove = 1; % Grow fmm-tree, give cpu more work
          at.levkeep = 1;          
          nlevels = nlevels + at.levmove;
          at.levprev = smoothtime;
       end
       if at.levdowninterval < 1
          at.levmove = -1; % Shrink fmm-tree, give gpu more work          
          at.levkeep = 1;          
          nlevels = nlevels + at.levmove;
          at.levprev = smoothtime;
       end
    end 
    % If I came from a better place in theta, go back
    if(at.thetprev < smoothtime)
       at.thetprev = inf;     
       at.thetmove = -1 * at.thetmove;
       theta = theta + at.thetstep*at.thetunit*at.thetmove;
       at.thetkeep = 0;
       if(at.fibnum < at.fibiters)
          at.thetstep = at.f1+at.f2;
          at.f1 = at.f2;
          at.f2 = at.thetstep;
          at.fibnum = at.fibnum+1;
       else
          at.fibiters = at.fibiters + 1;
          at.f1 = 0;
          at.f2 = 1;
          at.fibnum = 0;
          at.thetstep = 1;
       end
    else  % new place is better
       at.thetprev = inf;
       if at.thetwait < 1
          at.thetwait = at.thetinterval;
          if at.thetkeep == 1      % I went the right way last time
             at.fibiters = 1;
             at.f1 = 0;
             at.f2 = 1;
             at.fibnum = 0;
             at.thetstep = 1;
          end
          at.thetkeep = 1;          
          theta = theta + at.thetmove*at.thetstep*at.thetunit;
          at.thetprev = smoothtime;
       end
    end 
  end
  
  if(nlevels < 2)
     nlevels = 2;
  end
  if(theta < 0.2)
    theta = 0.2;
  elseif(theta > 0.8)
    theta = 0.8;
  end
  at.nlevels = nlevels;
  at.theta = theta;

