#include <math.h>

#include "mex.h"
#include "matrix.h"
#include "string.h"
/* #include "mem.h"*/

struct boundarydata {
    double basexvector;
    double baseyvector;
    double xvector;
    double yvector;
    double distance;
    int majordir; /*=1: x vector larger than yvector, 0: yvector larger*/
};

struct inputdata {
    double xpos;        /*xpos,ypos - position of current vortex (in)*/
    double ypos;
    double *xvortices;  /*xvortices,yvortices - list of neighboring vortices (in/out)*/
    double *yvortices;
    int nrvortices;     /*number of elements in xvortices,yvortices (in)*/
    int alloccount;     /*allocated length of xvortices,yvortices (in)*/
    double hnu;         /*hnu - h_nu valeu (in)*/
    int order;          /*order - order of VRM (in)*/
    double *newxpos;    /*newxpos,newypos - list of positions for new vortices (in)*/
    double *newypos;
    int newcount;       /*newcount - number of elements in newxpos (in)*/
    int includeself;
    int noaddelete;     /*when deleting vortices, do not add new vortices while deleting the original*/
    int multipassdelete;  /*attempt to delete vortices first, then diffuse the rest of the vortices*/
    double newposmaxdist;/*largest distance from center that a new vortice can be added, in case of adaptive getnewpos*/
    void (*getnewpos)(struct inputdata*,double*,double*);
    int includeboundary;/*include boundary elements (reflect points)*/
    int side;           /*side of boundary (to avoid diffusing through thin walls*/
    struct boundarydata *boundary; /*the boundary object*/
    double sqrcutoff;
    /*int debug;*/
};


/* forward declaration */
void getdiffusioneq(double *f,double *c,double xpos,double ypos,double* xarray,double* yarray,double h,int n,int order,int inclself);
int gramschmidtt(double *f,double *A,int m,int n);
void gramschmidttoriginal(double *f,double *A,int m,int n);
int minlinf(double* ret,double *f,double *A,int m,int n);
void printmatrix(double *A,int m,int n);
/* void getnewpos(const double *xcoord,const double* ycoord,const double xpos,const double ypos, const double *newxpos, const double *newypos, double *newx,double* newy,int ncoord, int npos);*/
void getnewposa(struct inputdata *input, double *newx,double* newy);
void getnewpos(struct inputdata *input, double *newx,double* newy);
void addcomplexelement(mxArray *array,double xval,double yval);
void resizerealarray(mxArray *array,int size);
int solvediffusioneq(struct inputdata *input,double *fractions,int *addednr);
struct boundarydata mindistfromvector(double xpos,double ypos,double* xvect,double *yvect, double cutoff,int vectcount);
int getreflectionpoint(struct boundarydata *boundary,double xpos,double ypos,double *rxpos,double *rypos); 
#define ISDOUBLEMATRIX(A) (mxIsDouble(A) && !mxIsSparse(A) && \
			   mxGetNumberOfDimensions(A) == 2)
#define ISREALSCALAR(A) (!mxIsCell(A) && !mxIsStruct(A) && \
			 !mxIsComplex(A) && mxGetNumberOfElements(A) == 1)
/*#define includeself 1*/
#define newallocinc 1000
#define arraysize 500
#define maxlist 100 /*number of free positions in array sent to VRM solver*/
#ifndef M_SQRT3
#define M_SQRT3 1.73205080756887719000
#endif
#define CIRCTOL 1e-6
/*#define DEBUG_main*/
#define imin(x,y) ((x)<(y)?(x):(y))
#define imax(x,y) ((x)>(y)?(x):(y))
/*------------------------------------------------------------------------*/
void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
    int order,fieldnr,localarraysize,vectorlength,i,j,k,localarraycount,
        base,base2,addednr,newvectorsize,newvectorcount=0,*vectorind,vectorindpos,lefttodelete=0,
        splitpoint,*newvectorind,vectoraddpos,newlocalcount,success,pass,probe,storefractions=0,
        storefractionalloc,storefractionpoint=0,infractions,infractionlength,ifnzmax,
        nrboundarypoints=0,includereflection,side,reflectionaddpos,
        xgridcount,ygridcount,*grid,*countgrid,xgridpos,ygridpos,*gridindex,yind;
    double cutoff,innercutoff,hnu,*localxarray,*localyarray,*xvector,*yvector,
        *fractions,*newxvector,*newyvector,*gammavortin,*gammaoldvort,*gammanewvort,
        *gammavortout,xtmp,ytmp,rxtmp,rytmp,sqrtmp,sqrinnercutoff,sqrcutoff,gammacutoff,
        xminn,xminp,yminn,yminp,xyminn,xyminp,yxminn,yxminp,xpos,ypos,probelimit,*sfarray,*ifarray,
        *xboundary=NULL,*yboundary=NULL,xmin,xmax,ymin,ymax;
    mwIndex /**irs,*jcs,*/*sfirs,*sfjcs,*ifirs,*ifjcs;
    mxLogical *deletevortex,*ignorevortex;
    mxArray* tmparray;
    char inputstring[100];
    struct inputdata indata;
    struct boundarydata boundary;
    /*mwSize nzmax;*/
    int debug1,debug2,debug3,debug4;
    int xgridoffset[]={-1,0,1,-1,0,1,-1,0,1};
    int ygridoffset[]={-1,-1,-1,0,0,0,1,1,1};
    localarraysize=arraysize; /*will probably remain constant for now*/
    indata.alloccount=localarraysize;
    if (2 != nlhs && 3 != nlhs || nrhs != 3)
        mexErrMsgTxt("Expecting 2 or 3 outputs and 3 inputs.");

    vectorlength=mxGetN(prhs[1]);
    if(vectorlength!=mxGetN(prhs[2]))
        mexErrMsgTxt("Size of position vector and vortex strenght vector must be equal");
    xvector=mxGetPr(prhs[1]);
    yvector=mxGetPi(prhs[1]);
    gammavortin=mxGetPr(prhs[2]);
    /* inputs C and X */
    /*if (!ISDOUBLEMATRIX(prhs[0]) || !ISDOUBLEMATRIX(prhs[1]))*/
    /*    mexErrMsgTxt("First two arguments must be double matrices.");*/

    /*read in the structure*/
    /*if(!mxIsSparse(prhs[2]))
        mexErrMsgTxt("argument 3 must be sparse matrix");*/
    fieldnr=mxGetFieldNumber(prhs[0],"order");
    if(fieldnr==-1)
        order=2; /*default value*/
    else {
        order=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
    }
    indata.order=order;
    fieldnr=mxGetFieldNumber(prhs[0],"hnu");
    if(fieldnr==-1)
        mexErrMsgTxt("Diffusion distance not given"); /*no default*/
    else {
        hnu=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
    }
    indata.hnu=hnu;
    probelimit=-0.5/(hnu*hnu);
    fieldnr=mxGetFieldNumber(prhs[0],"cutoff");
    if(fieldnr==-1)
        mexErrMsgTxt("cutoff not given"); /*no default*/
    else {
        cutoff=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
    }
    sqrcutoff=cutoff*cutoff;
    indata.sqrcutoff=sqrcutoff;
    fieldnr=mxGetFieldNumber(prhs[0],"innercutoff");
    if(fieldnr==-1)
        innercutoff=0; /*default value*/
    else {
        innercutoff=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
    }
    fieldnr=mxGetFieldNumber(prhs[0],"gammacutoff");
    if(fieldnr==-1)
        gammacutoff=1e-16; /*default value*/
    else {
        gammacutoff=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
    }
    sqrinnercutoff=innercutoff*innercutoff;
    fieldnr=mxGetFieldNumber(prhs[0],"newpos");
    if(fieldnr==-1) {
        /*Allocate one extra element at the end, which is origo. This is used by getnewposa to have origo as a position to add new vortices in when the vortex should be deleted*/
        if(order<=2) {
            indata.newxpos=mxCalloc(13,sizeof(double));
            indata.newypos=mxCalloc(13,sizeof(double));
            indata.newcount=12;
        }
        else {
            indata.newxpos=mxCalloc(37,sizeof(double));
            indata.newypos=mxCalloc(37,sizeof(double));
            indata.newcount=36;
        }
        for(i=0;i<12;i++){
            indata.newxpos[i]=hnu*M_SQRT3*M_SQRT2*cos(i*M_PI/6);
            indata.newypos[i]=hnu*M_SQRT3*M_SQRT2*sin(i*M_PI/6);
        }
        if(order>2) {
            for(;i<24;i++){
                indata.newxpos[i]=hnu*M_SQRT3*M_SQRT2*2*cos(i*M_PI/6+M_PI/12);
                indata.newypos[i]=hnu*M_SQRT3*M_SQRT2*2*sin(i*M_PI/6+M_PI/12);
            }
            for(;i<36;i++){
                indata.newxpos[i]=hnu*M_SQRT3*2*cos(i*M_PI/6+M_PI/12);
                indata.newypos[i]=hnu*M_SQRT3*2*sin(i*M_PI/6+M_PI/12);
            }
        }
    }
    else {
        indata.newcount=mxGetN(mxGetFieldByNumber(prhs[0],0,fieldnr));
        indata.newxpos=mxCalloc(indata.newcount+1,sizeof(double));
        indata.newypos=mxCalloc(indata.newcount+1,sizeof(double));
        memcpy(indata.newxpos,mxGetPr(mxGetFieldByNumber(prhs[0],0,fieldnr)),sizeof(double)*indata.newcount);
        memcpy(indata.newypos,mxGetPi(mxGetFieldByNumber(prhs[0],0,fieldnr)),sizeof(double)*indata.newcount);
/*         indata.newxpos=mxGetPr(mxGetFieldByNumber(prhs[0],0,fieldnr));*/
/*         indata.newypos=mxGetPi(mxGetFieldByNumber(prhs[0],0,fieldnr));*/
    }
    /*mexPrintf("Vector positions are indata.newxpos=%x, indata.newypos=%x\n",indata.newxpos,indata.newypos);*/
    fieldnr=mxGetFieldNumber(prhs[0],"ignore");
    if(fieldnr==-1||mxIsLogical(mxGetFieldByNumber(prhs[0],0,fieldnr))==0) {
        tmparray=mxCreateLogicalMatrix(1,vectorlength);
        ignorevortex=mxGetLogicals(tmparray);
        if(fieldnr!=-1)
            mexWarnMsgTxt("ignore list not logical array, All vortices will be diffused");
    }
    else {
        ignorevortex=mxGetLogicals(mxDuplicateArray(mxGetFieldByNumber(prhs[0],0,fieldnr)));
        if(mxGetN(mxGetFieldByNumber(prhs[0],0,fieldnr))!=vectorlength)
            mexErrMsgTxt("Invalid length of ignore list");
    }
    fieldnr=mxGetFieldNumber(prhs[0],"delete");
    if(fieldnr==-1||mxIsLogical(mxGetFieldByNumber(prhs[0],0,fieldnr))==0) {
        tmparray=mxCreateLogicalMatrix(1,vectorlength);
        deletevortex=mxGetLogicals(tmparray);
        if(fieldnr!=-1)
            mexWarnMsgTxt("deletion list not logical array, no vortices will be deleted");
    }
    else {
        deletevortex=mxGetLogicals(mxDuplicateArray(mxGetFieldByNumber(prhs[0],0,fieldnr)));
        if(mxGetN(mxGetFieldByNumber(prhs[0],0,fieldnr))!=vectorlength)
            mexErrMsgTxt("Invalid length of deletion list");
    }
    fieldnr=mxGetFieldNumber(prhs[0],"noaddelete");
    if(fieldnr==-1)
        indata.noaddelete=0; /*default value*/
    else {
        indata.noaddelete=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
    }
    fieldnr=mxGetFieldNumber(prhs[0],"multipassdelete");
    if(fieldnr==-1) {
        if(indata.noaddelete)       /*if noaddelete is active, default is twopass, otherwise, default is onepass  */
            indata.multipassdelete=2; /*default value*/
        else
            indata.multipassdelete=0; /*default value*/
    }
    else {
        indata.multipassdelete=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
        if(indata.multipassdelete<0)
            indata.multipassdelete=0;
        if(indata.multipassdelete>2)
            indata.multipassdelete=2;
    }
    fieldnr=mxGetFieldNumber(prhs[0],"probe");
    if(fieldnr==-1) {
        if(indata.noaddelete)       /*if noaddelete is active, default is twopass, otherwise, default is onepass  */
            probe=1; /*default value*/
        else
            probe=0; /*default value*/
    }
    else {
        probe=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
    }
    fieldnr=mxGetFieldNumber(prhs[0],"newposmethod");
    if(fieldnr==-1) {
        indata.getnewpos=getnewpos;
    }
    else {
        if(mxGetString(mxGetFieldByNumber(prhs[0],0,fieldnr),inputstring,99)) {
            mexWarnMsgTxt("Invalid input data, newposmethod, using default");
            indata.getnewpos=getnewpos;
        }
        if(strcmp(inputstring,"trianglecenter")==0)
            indata.getnewpos=getnewposa;
        else if(strcmp(inputstring,"original")==0)
            indata.getnewpos=getnewpos;
        else {
            mexWarnMsgTxt("Unknown newposmethod, using default");
            indata.getnewpos=getnewpos;
        }
    }
    fieldnr=mxGetFieldNumber(prhs[0],"newposmaxdist");
    if(fieldnr==-1) {
        sqrtmp=indata.newxpos[0]*indata.newxpos[0]+indata.newypos[0]*indata.newypos[0];
        for(i=1;i<indata.newcount;i++){
            ytmp=indata.newxpos[i]*indata.newxpos[i]+indata.newypos[i]*indata.newypos[i];
/*             mexPrintf("ytmp=%f\n",ytmp);*/
            if(ytmp>sqrtmp)
                sqrtmp=ytmp;
        }
        indata.newposmaxdist=sqrt(sqrtmp)*M_SQRT2; /*default value*/
    }
    else {
        indata.newposmaxdist=mxGetScalar(mxGetFieldByNumber(prhs[0],0,fieldnr));
    }
    fieldnr=mxGetFieldNumber(prhs[0],"boundary");
    nrboundarypoints=0;
    if(fieldnr!=-1) {
        tmparray=mxGetFieldByNumber(prhs[0],0,fieldnr);
        if(mxGetNumberOfElements(tmparray)!=0) {
            if(!mxIsComplex(tmparray))
                mexWarnMsgTxt("boundary must be complex matrix");
            else if(mxIsSparse(tmparray))
                mexWarnMsgTxt("boundary must be dense matrix");
            else if(mxGetM(tmparray)!=2)
                mexWarnMsgTxt("boundary must be a 2xN matrix");
            else {
                xboundary=mxGetPr(tmparray);
                yboundary=mxGetPi(tmparray);
                nrboundarypoints=mxGetN(tmparray);
            }
        }
    }
    fieldnr=mxGetFieldNumber(prhs[0],"fractions");
    if(fieldnr==-1) {
        infractions=0; /*default value*/
    }
    else {
        if(!mxIsSparse(mxGetFieldByNumber(prhs[0],0,fieldnr))){
            mexWarnMsgTxt("Fractions must be sparse matrix, ignoring given fractions");
            infractions=0;
        }
        else if(mxGetN(mxGetFieldByNumber(prhs[0],0,fieldnr))>vectorlength||mxGetM(mxGetFieldByNumber(prhs[0],0,fieldnr))>vectorlength){
            mexWarnMsgTxt("Invalid size of fraction matrix, ignoring given fractions");
            infractions=0;
        }
        else {
            ifarray=mxGetPr(mxGetFieldByNumber(prhs[0],0,fieldnr));
            ifirs=mxGetIr(mxGetFieldByNumber(prhs[0],0,fieldnr));
            ifjcs=mxGetJc(mxGetFieldByNumber(prhs[0],0,fieldnr));
            ifnzmax=mxGetNzmax(mxGetFieldByNumber(prhs[0],0,fieldnr));
            infractions=1;
            infractionlength=mxGetN(mxGetFieldByNumber(prhs[0],0,fieldnr));
/*             mexPrintf("infractionlength=%d ifnzmax=%d\n",infractionlength,ifnzmax);*/
        }
    }
    if(nlhs==3) {
        storefractions=1;
        if(order==1) /*make approximate guesses on how large the sparse matrix will be. Will be reallocated later if necessary*/
            storefractionalloc=4*vectorlength;
        else if(order==2)
            storefractionalloc=10*vectorlength;
        else if(order==3)
            storefractionalloc=15*vectorlength;
        else
            storefractionalloc=21*vectorlength;
        if(infractions&&storefractionalloc<ifnzmax)
            storefractionalloc=ifnzmax;
        plhs[2]=mxCreateSparse(vectorlength,vectorlength,storefractionalloc,mxREAL);
        sfirs=mxGetIr(plhs[2]);
        sfjcs=mxGetJc(plhs[2]);
        sfarray=mxGetPr(plhs[2]);
        if(infractions){/*copy previous fractions*/
            memcpy(sfarray,ifarray,ifnzmax*sizeof(double));
            memcpy(sfirs,ifirs,ifnzmax*sizeof(mwIndex));
            memcpy(sfjcs,ifjcs,(infractionlength+1)*sizeof(mwIndex));
            storefractionpoint=ifnzmax;
        }
/*         mexPrintf("Creating sparse %dx%d matrix with NZmax=%d\n",vectorlength,vectorlength,storefractionalloc);*/
    }
    /*Allocate matrices to use. use quite large matrices to avoid having to reallocate*/
    /*mexPrintf("order=%d hnu=%f cutoff=%f innercutoff=%f\n",order,hnu,cutoff,innercutoff);*/
    localxarray=mxMalloc(localarraysize*sizeof(double));
    localyarray=mxMalloc(localarraysize*sizeof(double));
    fractions=mxMalloc((localarraysize+1)*sizeof(double));
    vectorind=mxMalloc((localarraysize+1)*sizeof(double));
    /*mexPrintf("Vector positions are localxarray=%x, localyarray=%x fractions=%x, vectorind=%x, size=%d\n",localxarray,localyarray,fractions,vectorind,localarraysize*sizeof(double));*/
    indata.xvortices=localxarray;
    indata.yvortices=localyarray;
    newvectorsize=newallocinc;
    newxvector=mxMalloc(newvectorsize*sizeof(double));
    newyvector=mxMalloc(newvectorsize*sizeof(double));
    newvectorind=mxMalloc((vectorlength+1)*sizeof(int));
    newvectorind[0]=0;
    gammaoldvort=mxCalloc(vectorlength,sizeof(double));
    gammanewvort=mxCalloc(newvectorsize,sizeof(double));
    #ifdef DEBUG_main
    mexPrintf("Vector positions are newxvector=%x, newyvector=%x newvectorind=%x, gammaoldvort=%x, gammanewvort=%x\n",newxvector,newyvector,newvectorind,gammaoldvort,gammanewvort);
    #endif
    /*irs=mxGetIr(prhs[2]);
    jcs=mxGetJc(prhs[2]);
    nzmax=mxGetNzmax(prhs[2]);*/
/*     for(i=0;i<indata.newcount;i++)*/
/*         mexPrintf("New vortex position %d: %f + %fi\n",i,indata.newxpos[i],indata.newypos[i]);*/
    
    /*determine connecticity information*/
    
    /*start by determining the bounding box*/
    for(i=0;i<vectorlength&&ignorevortex[i];i++);
    if(i<vectorlength) {
        xmin=xvector[i];
        xmax=xvector[i];
        ymin=yvector[i];
        ymax=yvector[i];
    }
    /*mexPrintf("i=%d, xmin=%f xmax=%f ymin=%f ymax=%f\n",i,xmin,xmax,ymin,ymax);*/
    for(;i<vectorlength;i++) {
        if(ignorevortex[i])
            continue;
        if(xvector[i]<xmin)
            xmin=xvector[i];
        if(xvector[i]>xmax)
            xmax=xvector[i];
        if(yvector[i]<ymin)
            ymin=yvector[i];
        if(yvector[i]>ymax)
            ymax=yvector[i];
    }
    /*mexPrintf("i=%d, xmin=%f xmax=%f ymin=%f ymax=%f\n",i,xmin,xmax,ymin,ymax);*/
    /*in case ignored vortices are outside bounding box, include if close enough*/
    xmin-=cutoff;
    xmax+=cutoff;
    ymin-=cutoff;
    ymax+=cutoff;
    #ifdef DEBUG_main
    mexPrintf("grid determined xmin=%f xmax=%f ymin=%f ymax=%f\n",xmin,xmax,ymin,ymax);
    #endif
    xgridcount=ceil((xmax-xmin)/cutoff);
    ygridcount=ceil((ymax-ymin)/cutoff);
    grid=(int*)mxCalloc(xgridcount*ygridcount+1,sizeof(int));
    countgrid=(int*)mxCalloc(xgridcount*ygridcount,sizeof(int));
    /*count elements*/
    for(i=0;i<vectorlength;i++) {
        xgridpos=(int)((xvector[i]-xmin)/cutoff);
        ygridpos=(int)((yvector[i]-ymin)/cutoff);
        if(xgridpos<0||xgridpos>=xgridcount||ygridpos<0||ygridpos>=ygridcount)
            continue;
        grid[ygridpos*xgridcount+xgridpos+1]++;
    }
    /*calculate cumulative sum*/
    #ifdef DEBUG_main
    mexPrintf("calculating cumulative sum\n");
    #endif
    for(i=2;i<=xgridcount*ygridcount;i++)
        grid[i]+=grid[i-1];
    gridindex=(int*)mxMalloc(grid[xgridcount*ygridcount]*sizeof(int));
    /*sort the elements to the gridindex array*/
    #ifdef DEBUG_main
    mexPrintf("distributing elements\n");
    #endif
    for(i=0;i<vectorlength;i++) {
        xgridpos=(int)((xvector[i]-xmin)/cutoff);
        ygridpos=(int)((yvector[i]-ymin)/cutoff);
        if(xgridpos<0||xgridpos>=xgridcount||ygridpos<0||ygridpos>=ygridcount)
            continue;
        /*mexPrintf("gridpos=%d ygridpos=%d xgridcount=%d ygridcount=%d\n",xgridpos,ygridpos,xgridcount,ygridcount);*/
        gridindex[grid[ygridpos*xgridcount+xgridpos]+countgrid[ygridpos*xgridcount+xgridpos]]=i;
        countgrid[ygridpos*xgridcount+xgridpos]++;
    }
    #ifdef DEBUG_main
    mexPrintf("sorting done xmin=%f xmax=%f ymin=%f ymax=%f\n",xmin,xmax,ymin,ymax);
    #endif
    /*sorting done*/
    
    if(probe) {
        debug1=debug2=debug3=debug4=0;
        for(i=0;i<vectorlength;i++){
            if(deletevortex[i]==0||ignorevortex[i]==1) /*this check only determines if it is impossible to delete a vortex or not (to avoid having to call the linprog solver). */
                continue;
            /*localarraycount=i==(vectorlength-1)?nzmax-jcs[i]:jcs[i+1]-jcs[i];*/
            localarraycount=0;
            for(yind=imax(0,ygridpos-1);yind<=imin(ygridcount-1,ygridpos+1);yind++)
                localarraycount+=grid[yind*xgridcount+imin(xgridcount,xgridpos+2)]-grid[yind*xgridcount+imax(0,xgridpos-1)];
            if(localarraycount<6) { /*to few vortices, deletion not possible*/
                deletevortex[i]=0;
                debug1++;
                continue;
            }
            /*base=jcs[i];*/
            xpos=xvector[i];
            ypos=yvector[i];
/*             xtmp=xvector[irs[base]]-xpos;*/
/*             ytmp=yvector[irs[base]]-ypos;*/
/*             xminn=xtmp;*/
/*             xminp=xtmp;*/
/*             yminn=ytmp;*/
/*             yminp=ytmp;*/
/*             xyminn=xtmp+ytmp;*/
/*             xyminp=xtmp+ytmp;*/
/*             yxminn=xtmp-ytmp;*/
/*             yxminp=xtmp-ytmp;*/
            xminn=yminn=xyminn=yxminn=-1e9*hnu; /*pick large starting values*/
            xminp=yminp=xyminp=yxminp=1e9*hnu;
            for(yind=imax(0,ygridpos-1);yind<=imin(ygridcount-1,ygridpos+1);yind++) {
                for(j=grid[yind*xgridcount+imax(0,xgridpos-1)];j<grid[yind*xgridcount+imin(xgridcount,xgridpos+2)]&&vectoraddpos<localarraysize-maxlist;j++) {
                /*for(j=1;j<localarraycount;j++) {*/
                    xtmp=xvector[/*irs[base+j]*/gridindex[j]]-xpos;
                    ytmp=yvector[/*irs[base+j]*/gridindex[j]]-ypos;
                    if(xtmp>0) {
                        if(xtmp<xminp)
                            xminp=xtmp;
                    }
                    else if(xtmp>xminn)
                        xminn=xtmp;
                    if(ytmp>0){
                        if(ytmp<yminp)
                            yminp=ytmp;
                    }
                    else if(ytmp>yminn)
                        yminn=ytmp;
                    if(xtmp+ytmp>0) {
                        if(xtmp+ytmp<xyminp)
                            xyminp=xtmp+ytmp;
                    }
                    else if(xtmp+ytmp>xyminn)
                        xyminn=xtmp+ytmp;
                    if(xtmp-ytmp>0){    
                        if(xtmp-ytmp<yxminp)
                            yxminp=xtmp-ytmp;
                    }
                    else if(xtmp-ytmp>yxminn)
                        yxminn=xtmp-ytmp;
                }
            }
            mexPrintf("Vortex %d, pos %fx%f xminn=%f, xminp=%f,yminn=%f,yminp=%f,xyminn=%f,xyminp=%f,yxminn=%f,yxminp=%f\n",i,xpos,ypos,xminn,xminp,yminn,yminp,xyminn,xyminp,yxminn,yxminp);
/*             if(xminp<=0||xminn>=0||yminp<=0||yminn>=0||xyminp<=0||xyminn>=0||yxminp<=0||yxminn>=0) {*/
/*                 deletevortex[i]=0;*/
/*                 debug2++;*/
/*                 continue;*/
/*             }*/
            mexPrintf("cond1=%f,cond2=%f,limit=%f\n",1/(xminn*xminp)+1/(yminn*yminp),1/(xyminn*xyminp)+1/(yxminn*yxminp),probelimit);
            if(1/(xminn*xminp)+1/(yminn*yminp)>probelimit){
                deletevortex[i]=0;
                debug3++;
                continue;
            }
            if(1/(xyminn*xyminp)+1/(yxminn*yxminp)>probelimit*0.5){
                deletevortex[i]=0;
                debug4++;
            }
        }
        mexPrintf("unfeasable: %d,%d,%d,%d,tot=%d\n",debug1,debug2,debug3,debug4,debug1+debug2+debug3+debug4);
    }
    #ifdef DEBUG_main
    mexPrintf("infractions=%d\n",infractions);
    #endif
    for(pass=2-indata.multipassdelete;pass<3;pass++) { /*run two times to try and delete vortices first, and diffuse the rest the next step*/
        #ifdef DEBUG_main
        mexPrintf("Pass %d\n",pass);
        #endif
        i=0;
        if(pass==2&&infractions){
            for(;i<infractionlength;i++){
                base=ifjcs[i];
                for(j=0;j<ifjcs[i+1]-base;j++){
/*                     mexPrintf("Vortex %d adding fraction %f to vortex %d base=%d\n",i,ifarray[base+j],ifirs[base+j],base);*/
                    gammaoldvort[ifirs[base+j]]+=gammavortin[i]*ifarray[base+j];
                }
                newvectorind[i+1]=newvectorcount;
            }
        }
        if(pass==1&&infractions)
            i=infractionlength;
        for(;i<vectorlength;i++) {
            /*if the circulation is too low, ignore the vortex*/
            if(((gammavortin[i]<gammacutoff&&gammavortin[i]>-gammacutoff)||ignorevortex[i])&&pass==2&&(deletevortex[i]==0||indata.multipassdelete==0||ignorevortex[i])) {
                gammaoldvort[i]+=gammavortin[i];
                newvectorind[i+1]=newvectorcount;
                #ifdef DEBUG_main
                mexPrintf("ignoring vortex %d with circulation %.16e\n",i,gammavortin[i]);
                #endif
                if(storefractions){
                    if(storefractionpoint+1>storefractionalloc){
                        storefractionalloc*=1.2;/*increase size with 20 % each time*/
                        storefractionalloc++;/*and if it is really small from the start, make sure that the new values will fit*/
                        sfarray=mxRealloc(sfarray,storefractionalloc*sizeof(double));
                        sfirs=mxRealloc(sfirs,storefractionalloc*sizeof(mwIndex));
                    }
                    sfjcs[i]=storefractionpoint;
                    sfirs[storefractionpoint]=i;
                    sfarray[storefractionpoint]=1;
                    storefractionpoint++;
                }
                continue;
            }
            if((indata.multipassdelete!=0&&((pass!=2)^(deletevortex[i]!=0)))||ignorevortex[i]){ /*in twopass, first pass, only take vortices marked for deletion, second pass, the rest*/
                #ifdef DEBUG_main
                mexPrintf("Vortex %d pass=%d deletevortex=%d\n",i,pass,deletevortex[i]);
                #endif
                continue;
            }
            /*localarraycount=i==(vectorlength-1)?nzmax-jcs[i]:jcs[i+1]-jcs[i];*/
            /*#ifdef DEBUG_main
            mexPrintf("position %d, value %f + %fi number of neighbors %d nzmax=%d\n",i,xvector[i],yvector[i],localarraycount,nzmax);
            #endif*/
            /*         if(localarraycount>localarraysize-maxlist)*/
            /*             loopmax=localarraysize-maxlist;*/
            /*         else*/
            /*             loopmax=localarraycount;*/
            indata.xpos=xvector[i];
            indata.ypos=yvector[i];
            includereflection=0;
            if(nrboundarypoints>0) {
                boundary=mindistfromvector(indata.xpos,indata.ypos,xboundary,yboundary,sqrcutoff,nrboundarypoints);
/*                 mexPrintf("minimum distance from boundary=%f, vector base=%fx%f, length=%fx%f\n",boundary.distance,boundary.basexvector,boundary.baseyvector,boundary.xvector,boundary.yvector);*/
                if(boundary.distance<cutoff) {
                    includereflection=1;
                    side=getreflectionpoint(&boundary,indata.xpos,indata.ypos,&rxtmp,&rytmp);
/*                     mexPrintf("Side=%d\n",side);*/
                }
            }
            /*base=jcs[i];*/
            vectoraddpos=0;
            if(deletevortex[i]){
                vectorindpos=0;
                if(pass==0)
                    lefttodelete++;
            }
            else {
                vectorind[0]=i;
                vectorindpos=1;
                if(includereflection==1) {
                    sqrtmp=(rxtmp-indata.xpos)*(rxtmp-indata.xpos)+(rytmp-indata.ypos)*(rytmp-indata.ypos);
                    if((sqrtmp>sqrinnercutoff||deletevortex[i])&&sqrtmp<sqrcutoff) {   /*check if within cutoff limits*/
                        localxarray[1]=rxtmp;
                        localyarray[1]=rytmp;
                        vectorind[1]=i;
                        vectorindpos=2;
                        vectoraddpos=1;
                    }
                }
            }
            /*for(j=0;j<localarraycount&&vectoraddpos<localarraysize-maxlist;j++) {*/
            xgridpos=(int)((xvector[i]-xmin)/cutoff);
            ygridpos=(int)((yvector[i]-ymin)/cutoff);
            /*mexPrintf("vortex %d build list\n",i);*/
            /*if(i==5000) {
                mexPrintf("yindstart=%d yindend=%d xgridpos=%d,ygridpos=%d,xvector=%f,yvector=%f,xmin=%f,ymin=%f,xmax=%f,ymax=%f\n",imax(0,ygridpos-1),imin(ygridcount,ygridpos+1),xgridpos, ygridpos,xvector,yvector,xmin,ymin,xmax,ymax);
            }*/
            for(yind=imax(0,ygridpos-1);yind<=imin(ygridcount-1,ygridpos+1);yind++) {
                /*if(i==5000) {
                   mexPrintf("yind=%d, jstart=%d jend=%d\n",yind,grid[yind*xgridcount+imin(0,xgridpos-1)],grid[yind*xgridcount+imax(xgridcount,xgridpos+1)]);
                }*/
                for(j=grid[yind*xgridcount+imax(0,xgridpos-1)];j<grid[yind*xgridcount+imin(xgridcount,xgridpos+2)]&&vectoraddpos<localarraysize-maxlist;j++) {
                    if(i==gridindex[j])/*if(i==irs[base+j])*/ /*do not add the vortex itself to the list*/
                        continue;
                    if(deletevortex[/*irs[base+j]]*/gridindex[j]]&&pass!=0){ /*do not add the vortex if it is marked for deletion*/
    /*                     mexPrintf("ignoring vortex %d\n",irs[base+j]);*/
                        continue;
                    }
                    xtmp=xvector[/*irs[base+j]*/gridindex[j]];
                    ytmp=yvector[/*irs[base+j]*/gridindex[j]];
                    if(includereflection) {
                        if(getreflectionpoint(&boundary,xtmp,ytmp,&rxtmp,&rytmp)!=side)
                            continue;
                        /*                         mexPrintf("rigth side, point %fx%f, reflection=%fx%f\n",xtmp,ytmp,rxtmp,rytmp);*/
                        /*                     else*/
                        /* //                         mexPrintf("wrong side, point %fx%f, reflection=%fx%f\n",xtmp,ytmp,rxtmp,rytmp);*/
                        /*                         continue;*/
                        sqrtmp=(rxtmp-xvector[i])*(rxtmp-xvector[i])+(rytmp-yvector[i])*(rytmp-yvector[i]);                     /*calculate square distance to the vortex*/
                        if((sqrtmp>sqrinnercutoff||deletevortex[i])&&sqrtmp<sqrcutoff) {   /*check if within cutoff limits*/
                            localxarray[vectoraddpos]=rxtmp;
                            localyarray[vectoraddpos]=rytmp;
                            vectorind[vectorindpos]=gridindex[j];/*irs[base+j];*/
                            /*                     mexPrintf("neighbor %d, value %f + %fi\n",vectoraddpos,localxarray[vectoraddpos],localyarray[vectoraddpos]);*/
                            vectorindpos++;
                            vectoraddpos++;
                        }
                    }
                    sqrtmp=(xtmp-xvector[i])*(xtmp-xvector[i])+(ytmp-yvector[i])*(ytmp-yvector[i]);                     /*calculate square distance to the vortex*/
                    /*if(i==5000) {
                        mexPrintf("sqrtmp=%f sqrcutoff=%f deletevortex=%d\n",sqrtmp,sqrcutoff,deletevortex[i]);
                    }*/
                    if((sqrtmp>sqrinnercutoff||deletevortex[i])&&sqrtmp<sqrcutoff) {   /*check if within cutoff limits*/
                        localxarray[vectoraddpos]=xtmp;
                        localyarray[vectoraddpos]=ytmp;
                        vectorind[vectorindpos]=gridindex[j]/*irs[base+j]*/;
    /*                     mexPrintf("neighbor %d, value %f + %fi\n",vectoraddpos,localxarray[vectoraddpos],localyarray[vectoraddpos]);*/
                        vectorindpos++;
                        vectoraddpos++;
                    }
    /*                 else*/
    /*                    mexPrintf("Ignored neighbor %d value  %f + %fi distance %f\n",irs[base+j],xtmp,ytmp,sqrt(sqrtmp));*/
                }
            }
            /*mexPrintf("vortex %d build list done %d\n",i,vectoraddpos);*/
            splitpoint=vectorindpos; /*position where new vortices start*/
            /*check if the neighbors has added any new vortices*/
            /*#ifdef DEBUG_main
            mexPrintf("creating array vectorindpos=%d base=%d localarraycount=%d\n",vectorindpos,base,localarraycount);
            #endif*/
            if(pass==2) { /*for now, only check the new vortices in the non-deleting pass*/
                /*for(j=0;j<localarraycount;j++) {*/
                /*mexPrintf("vortex %d build additional list %d\n",i,vectoraddpos);*/
                for(yind=imax(0,ygridpos-1);yind<=imin(ygridcount-1,ygridpos+1);yind++) {
                    for(j=grid[yind*xgridcount+imax(0,xgridpos-1)];j<grid[yind*xgridcount+imin(xgridcount,xgridpos+2)];j++) {
                        if(vectorindpos>localarraysize-maxlist){ /*list full (should not happen for good simulations)*/
                            #ifdef DEBUG_main
                            mexPrintf("List full, vectorindpos=%d,localarraysize=%d maxlist=%d\n",vectorindpos,localarraysize,maxlist);
                            #endif
                            break;
                        }
                        if(/*irs[base+j]*/gridindex[j]>=i) /*no new vortices have been added if the vortex have a higher index than current*/
                            continue;
                        base2=gridindex[j];/*irs[base+j];*/
                        newlocalcount=newvectorind[base2+1]-newvectorind[base2]; 
                         /*mexPrintf("Starting inner loop newlocalcount=%d base2=%d newvectorind+1=%d nevectorind=%d\n",newlocalcount,base2,newvectorind[base2+1],newvectorind[base2]);*/
                        for(k=0;k<newlocalcount;k++) {
                            if(vectorindpos>localarraysize-maxlist){ /*list full (should not happen for good simulations)*/
                                #ifdef DEBUG_main
                                mexPrintf("List full,inner loop, vectorindpos=%d,localarraysize=%d maxlist=%d\n",vectorindpos,localarraysize,maxlist);
                                #endif
                                break;
                            }
                            /*mexPrintf("inner loop %d\n",k);*/
                            xtmp=newxvector[newvectorind[base2]+k];
                            ytmp=newyvector[newvectorind[base2]+k];
                            if(includereflection) {
                                if(getreflectionpoint(&boundary,xtmp,ytmp,&rxtmp,&rytmp)!=side)
                                    continue;
                                sqrtmp=(rxtmp-xvector[i])*(rxtmp-xvector[i])+(rytmp-yvector[i])*(rytmp-yvector[i]);                     /*calculate square distance to the vortex*/
                                if((sqrtmp>sqrinnercutoff||deletevortex[i])&&sqrtmp<sqrcutoff) {   /*check if within cutoff limits*/
                                    localxarray[vectoraddpos]=rxtmp;
                                    localyarray[vectoraddpos]=rytmp;
                                    vectorind[vectorindpos]=newvectorind[base2]+k;
                                                         /*mexPrintf("neighbor %d, value %f + %fi\n",vectoraddpos,localxarray[vectoraddpos],localyarray[vectoraddpos]);*/
                                    vectorindpos++;
                                    vectoraddpos++;
                                }
                            }
                            sqrtmp=(xtmp-xvector[i])*(xtmp-xvector[i])+(ytmp-yvector[i])*(ytmp-yvector[i]);
                            if((sqrtmp>sqrinnercutoff||deletevortex[i])&&sqrtmp<sqrcutoff) {
                                /* mexPrintf("Adding inner element\n");*/
                                localxarray[vectoraddpos]=xtmp;
                                localyarray[vectoraddpos]=ytmp;
                                vectorind[vectorindpos]=newvectorind[base2]+k;
                                vectorindpos++;
                                vectoraddpos++;
                                /* mexPrintf("adding complete\n");*/
                            }
                            /*if(i==3){*/
                            /*mexPrintf("new neighbor %d, value %f + %fi\n",vectoraddpos-1,localxarray[vectoraddpos-1],localyarray[vectoraddpos-1]);*/
                            /*}*/
                        }
                    }
                }
                /*mexPrintf("vortex %d build additional list done %d\n",i,vectoraddpos);*/
            }
            #ifdef DEBUG_main
            mexPrintf("Array created\n");
            #endif
            indata.nrvortices=vectoraddpos;
            indata.includeself=!deletevortex[i];
            indata.includeboundary=includereflection;
            indata.side=side;
            indata.boundary=&boundary;
            /*indata.debug=0;*/
            #ifdef DEBUG_main
            mexPrintf("solving diffusion equation, nr of vortices = %d\n",vectoraddpos);
            #endif
            /*mexPrintf("solving diffusion equation %d\n",i);*/
            success=solvediffusioneq(&indata,fractions,&addednr);
            /*mexPrintf("solving diffusion equation done %d\n",i);*/
        /*if it failed and the vortex itself was not included, try again including the vortex*/
            if(!success&&!indata.includeself) {
              /*mexPrintf("adding self %d\n",i);*/
                deletevortex[i]=0;
                lefttodelete--;
                #ifdef DEBUG_main
                mexPrintf("deletion failed\n");
                #endif
                if(indata.multipassdelete!=0) /*if twopass, move on to the next vortex, this will only happen during the first pass*/
                    continue;
                indata.nrvortices=vectoraddpos;
                indata.includeself=1;
                for(j=vectorindpos;j>0;j--)
                    vectorind[j]=vectorind[j-1];
                vectorind[0]=i;
                vectorindpos++;
                splitpoint++;
                success=solvediffusioneq(&indata,fractions,&addednr);
                #ifdef DEBUG_main
                mexPrintf("Second attempt\n");
                #endif
                        /*mexPrintf("adding self done %d\n",i);*/
            }
            if(pass==0)
                continue;
            if(success) { /*if successful, split the vorticity*/
/*                 if(pass==1)*/
/*                     mexPrintf("Deleted vortex nr %d deletevortex[%d]=%d gammanewvort[i]=%e\n",i,i,deletevortex[i],gammanewvort[i]);*/
                #ifdef DEBUG_main
                mexPrintf("solving diffusion equation complete addednr=%d newvectorcount=%d newvectorsize=%d\n",addednr,newvectorcount,newvectorsize);
                #endif
                while(addednr+newvectorcount>=newvectorsize) {
                    gammanewvort=mxRealloc(gammanewvort,(newvectorsize+newallocinc)*sizeof(double));
                    for(j=newvectorsize;j<newvectorsize+newallocinc;j++)
                        gammanewvort[j]=0.0;
                    newvectorsize+=newallocinc;
                    newxvector=mxRealloc(newxvector,newvectorsize*sizeof(double));
                    newyvector=mxRealloc(newyvector,newvectorsize*sizeof(double));
                }
                #ifdef DEBUG_main
                mexPrintf("Adding new vorticies\n");
                #endif
                        /*mexPrintf("Adding new vorticies\n");*/
                if(includereflection){
                    for(j=0,reflectionaddpos=0;j<addednr;j++) {
                        if(getreflectionpoint(&boundary,localxarray[j+vectoraddpos],localyarray[j+vectoraddpos],&rxtmp,&rytmp)==side){
                            newxvector[newvectorcount+reflectionaddpos]=localxarray[j+vectoraddpos];
                            newyvector[newvectorcount+reflectionaddpos]=localyarray[j+vectoraddpos];
                            vectorind[vectorindpos+j]=newvectorcount+reflectionaddpos;
                            reflectionaddpos++;
/*                             mexPrintf("Element %fx%f on right side\n",localxarray[j+vectoraddpos],localyarray[j+vectoraddpos]);*/
                        }
                        else
                            vectorind[vectorindpos+j]=newvectorcount+reflectionaddpos;
                    }
                    newvectorcount+=reflectionaddpos;
                }
                else {
                    for(j=0;j<addednr;j++) {
                        newxvector[newvectorcount+j]=localxarray[j+vectoraddpos];
                        newyvector[newvectorcount+j]=localyarray[j+vectoraddpos];
                        vectorind[vectorindpos+j]=newvectorcount+j;
                    }
                    newvectorcount+=addednr;
                }
                /*mexPrintf("Adding new vorticies done\n");*/
                newvectorind[i+1]=newvectorcount;
                #ifdef DEBUG_main
                mexPrintf("Added %d to index %d of newvectorind\n",newvectorcount,i+1);
                #endif
                /*             if(i==100||i==0) {*/
                /*                     mexPrintf("Vortex distribution for vortex %d\n",i);*/
                /*                     mexPrintf("Current vortices are: ");*/
                /*                 for(j=0;j<vectorindpos+addednr-includeself;j++) {*/
                /*                     if(localyarray[j]>0)*/
                /*                         mexPrintf("%.16f+%.16fi ",localxarray[j],localyarray[j]);*/
                /*                     else*/
                /*                         mexPrintf("%.16f%.16fi ",localxarray[j],localyarray[j]);*/
                /*                 }*/
                /*                     mexPrintf("\n");*/
                /*             }*/
                #ifdef DEBUG_main
                mexPrintf("Distributing vorticity\n");
                #endif
                for(j=0;j<splitpoint;j++) { /*for the old vortices*/
                    gammaoldvort[vectorind[j]]+=gammavortin[i]*fractions[j];
                }
                #ifdef DEBUG_main
                mexPrintf("Splitpoint=%d vectoraddpos=%d addednr=%d j=%d\n",splitpoint,vectoraddpos,addednr,j);
                #endif
                for(;j<vectorindpos+addednr;j++) { /*for the new vortices*/
                    gammanewvort[vectorind[j]]+=gammavortin[i]*fractions[j];
                }
/*                 for(;j<vectorindpos+addednr;j++) {*/
/*                     gammanewvort[newvectorcount+j-vectorindpos-addednr]+=gammavortin[i]*fractions[j];*/
/*                 }*/
                /*             mexPrintf("Added %d ",addednr);*/
                /*             for(j=0;j<addednr;j++)*/
                /*                 mexPrintf("%f +%fi, ",localxarray[j+vectoraddpos],localyarray[j+vectoraddpos]);*/
                /*             //if(i==0) {*/
                /*             mexPrintf("\nFractions ");*/
                /*             for(j=0;j<vectoraddpos+addednr+includeself;j++) {*/
                /*                 mexPrintf("%f ",fractions[j]);*/
                /*             }*/
                /*             mexPrintf("\n");*/
                /*             mexPrintf("gammaoldvort ");*/
                /*             for(j=0;j<vectorlength;j++)*/
                /*                 mexPrintf("%f ",gammaoldvort[j]);*/
                /*             mexPrintf("\n");*/
                /*             mexPrintf("gammanewvort ");*/
                /*             for(j=0;j<newvectorcount;j++)*/
                /*                 mexPrintf("%f ",gammanewvort[j]);*/
                /*             mexPrintf("\n");*/
                /*}*/
                xtmp=0;
                for(j=0;j<vectorindpos+addednr;j++)
                    xtmp+=fractions[j];
                if(xtmp<0.999||xtmp>1.001) {
                    mexPrintf("sum of %d fractions is %.16f for vortex %d (%.16f+%.16fi)(vectorindpos = %d, added = %d)\n",vectorindpos+addednr,xtmp,i,xvector[i],yvector[i],vectorindpos,addednr);
                    mexPrintf("Current vortices are: ");
                    for(j=0;j<vectorindpos+addednr-indata.includeself;j++) {
                        if(localyarray[j]>0)
                            mexPrintf("%.16f+%.16fi ",localxarray[j],localyarray[j]);
                        else
                            mexPrintf("%.16f%.16fi ",localxarray[j],localyarray[j]);
                    }
                    mexPrintf("\nCurrent fractions are: ");
                    for(j=0;j<vectorindpos+addednr;j++)
                        mexPrintf("%.16f ",fractions[j]);
                    mexPrintf("\n");
/*                     indata.nrvortices+=addednr; //??? Why????*/
/*                     vectorindpos+=addednr;*/
/*                     xtmp=0;*/
                    /*                 indata.debug=1;*/
                    /*                 if(solvediffusioneq(&indata,fractions,&addednr)){*/
                    /*                     for(j=0;j<vectorindpos+addednr;j++)*/
                    /*                         xtmp+=fractions[j];*/
                    /*                     mexPrintf("sum of %d fractions is %.16f for vortex %d (%.16f+%.16fi)(vectorindpos = %d, added = %d)\n",vectorindpos+addednr,xtmp,i,xvector[i],yvector[i],vectorindpos,addednr);*/
                    /*                     mexPrintf("Current vortices are: ");*/
                    /*                     for(j=0;j<vectorindpos+addednr-includeself;j++) {*/
                    /*                         if(localyarray[j]>0)*/
                    /*                             mexPrintf("%.16f+%.16fi ",localxarray[j],localyarray[j]);*/
                    /*                         else*/
                    /*                             mexPrintf("%.16f%.16fi ",localxarray[j],localyarray[j]);*/
                    /*                     }*/
                    /*                     mexPrintf("\nCurrent fractions are: ");*/
                    /*                     for(j=0;j<vectorindpos+addednr;j++)*/
                    /*                         mexPrintf("%.16f ",fractions[j]);*/
                    /*                     mexPrintf("\n");*/
                    /*                 }*/
                }
                if(storefractions){
                    if(storefractionpoint+vectorindpos+addednr>storefractionalloc){
                        storefractionalloc*=1.2;/*increase size with 20 % each time*/
                        storefractionalloc+=vectorindpos+addednr;/*and if it is really small from the start, make sure that the new values will fit*/
                        sfarray=mxRealloc(sfarray,storefractionalloc*sizeof(double));
                        sfirs=mxRealloc(sfirs,storefractionalloc*sizeof(mwIndex));
/*                         mexPrintf("Reallocating sparce matrices\n");*/
                    }
                    sfjcs[i]=storefractionpoint;
                    for(j=1;j<splitpoint&&vectorind[j]<vectorind[0];j++) { /*for the new vortices*/
                        sfirs[storefractionpoint+j-1]=vectorind[j];
                        sfarray[storefractionpoint+j-1]=fractions[j];
/*                         mexPrintf("1: Adding value %f to position %d j=%d\n",fractions[j],vectorind[j],j);*/
                    }
                    sfirs[storefractionpoint+j-1]=vectorind[0];
                    sfarray[storefractionpoint+j-1]=fractions[0];
/*                     mexPrintf("2: Adding value %f to position %d j=%d\n",fractions[0],vectorind[0],j);*/
                    for(;j<splitpoint;j++) { /*for the new vortices*/
                        sfirs[storefractionpoint+j]=vectorind[j];
                        sfarray[storefractionpoint+j]=fractions[j];
/*                         mexPrintf("3: Adding value %f to position %d j=%d\n",fractions[j],vectorind[j],j);*/
                    }
                    for(;j<vectorindpos+addednr;j++) { /*for the new vortices*/
                        sfirs[storefractionpoint+j]=vectorind[j]+vectorlength;
                        sfarray[storefractionpoint+j]=fractions[j];
/*                         mexPrintf("4: Adding value %f to position %d j=%d\n",fractions[j],vectorind[j],j);*/
                    }
/*                     for(;j<vectorindpos+addednr;j++) {*/
/*                         sfirs[storefractionpoint+j]=vectorlength+newvectorcount+j-vectorindpos-addednr;*/
/*                         sfarray[storefractionpoint+j]=fractions[j];*/
/* //                         mexPrintf("5: Adding value %f to position %d j=%d\n",fractions[j],vectorlength+newvectorcount+j-vectorindpos-addednr,j);*/
/*                     }*/
                    storefractionpoint+=vectorindpos+addednr;
/*                     mexPrintf("storefractionpoint=%d\n",storefractionpoint);*/
                }
            }
            else {
                mexPrintf("solution failed for vortex %d\n",i);
                gammaoldvort[i]+=gammavortin[i]; /*if it failed, use old value*/
                newvectorind[i+1]=newvectorcount;
                if(storefractions){
                    if(storefractionpoint+1>storefractionalloc){
                        storefractionalloc*=1.2;/*increase size with 20 % each time*/
                        storefractionalloc++;/*and if it is really small from the start, make sure that the new values will fit*/
                        sfarray=mxRealloc(sfarray,storefractionalloc*sizeof(double));
                        sfirs=mxRealloc(sfirs,storefractionalloc*sizeof(mwIndex));
                    }
                    sfjcs[i]=storefractionpoint;
                    sfirs[storefractionpoint]=i;
                    sfarray[storefractionpoint]=1;
                    storefractionpoint++;
                }
            }
        }
 /*       mexPrintf("Pass %d left to delete=%d\n",pass,lefttodelete);*/
/*         if(vectorlength>290)*/
/*             mexPrintf("45: %e, 98: %e, 129: %e, 183: %e, 184: %e\n",gammaoldvort[45],gammaoldvort[98],gammaoldvort[129],gammaoldvort[183],gammaoldvort[184]);*/
    }
    if(storefractions) {
        sfjcs[vectorlength]=storefractionpoint;
        sfarray=mxRealloc(sfarray,storefractionpoint*sizeof(double));
        sfirs=mxRealloc(sfirs,storefractionpoint*sizeof(mwIndex));
        mxSetPr(plhs[2],sfarray);
        mxSetIr(plhs[2],sfirs);
        mxSetNzmax(plhs[2],storefractionpoint);
        mxSetM(plhs[2],vectorlength+newvectorcount);
/*         mexPrintf("Completing sparse matrix, storefractionpoint=%d\n",storefractionpoint);*/
    }
    /*     mexPrintf("New %d values\n",newvectorcount);*/
/*     for(j=0;j<newvectorcount;j++) {*/
/*         mexPrintf("%f + %fi, ",newxvector[j],newyvector[j]);*/
/*     }*/
/*     mexPrintf("\n");*/
     /* create output */
    /* mexPrintf("check1\n");*/
    plhs[0]=mxCreateDoubleMatrix(1,0,mxCOMPLEX);
    /* mexPrintf("check2\n");*/
    mxSetPr(plhs[0],newxvector);
   /*  mexPrintf("check3\n");*/
    mxSetPi(plhs[0],newyvector);
   /*  mexPrintf("check4\n");*/
    mxSetN(plhs[0],newvectorcount);
   /*  mexPrintf("check5\n");*/
    plhs[1]=mxCreateDoubleMatrix(1,vectorlength+newvectorcount,mxCOMPLEX);
   /*  mexPrintf("check6\n");*/
    gammavortout=mxGetPr(plhs[1]);
   /*  mexPrintf("check7\n");*/
    for(j=0;j<vectorlength;j++) {
        gammavortout[j]=gammaoldvort[j];
    }
   /*   mexPrintf("check8\n");*/
    for(i=0;i<newvectorcount;i++,j++) {
        gammavortout[j]=gammanewvort[i];
    }
    mxFree(localxarray);
    mxFree(localyarray);
    mxFree(fractions);
    mxFree(vectorind);
    mxFree(newvectorind);
    mxFree(gammaoldvort);
    mxFree(gammanewvort);
    mxFree(grid);
    mxFree(countgrid);
    mxFree(gridindex);
    /*mexPrintf("End of VRM\n");*/
     /* evaluate the result */
/*     if(vectorlength>290)*/
/*         mexPrintf("45: %e, 98: %e, 129: %e, 183: %e, 184: %e\n",gammaoldvort[45],gammaoldvort[98],gammaoldvort[129],gammaoldvort[183],gammaoldvort[184]);*/

}
/*------------------------------------------------------------------------*/
void getdiffusioneq(double *f,double *c,double xpos,double ypos,double* xarray,double* yarray,double h,int n,int order,int inclself)
/*the algorithm is intentionally creating the diffusion matrix with the wrong sign, since it will be easier for the*/
/*minimum L infinity solver, which otherwise will change the sign of the entire matrix due to the fact that the first*/
/*column is strictly positive. The value of f has the original sign*/
/*one extra line is also added which is zero. This will be used by the minimum L infinity solver and hence the same matrix*/
/*can be used.*/
{
    double *xdist,*ydist,*tmppos,*tmppos2,*tmppos3,*tmppos4,*tmppos5,x,y;
    int j,loopmax;
    h=1/h;
    /*step 1, constants*/
    for(j=0;j<n-1;j++)
        c[j]=-1;
    if(inclself) {
        loopmax=n-2;
        c++;
    }
    else {
        loopmax=n-1;
    }
    /*f[0]=1.5-n*0.5;*/
    f[0]=1.5-n*0.5; /*n is one more than the number of points*/
    if(order<1)
        return;
    
    /*step 2 xdistances*/
    xdist=c+n;
    for(j=0;j<loopmax;j++) {
        xdist[j]=-(xarray[j]-xpos)*h;
        f[1]+=xdist[j]*0.5;
    }
    
    /*step 3 ydistances*/
    ydist=c+n+n;
    for(j=0;j<loopmax;j++) {
        ydist[j]=-(yarray[j]-ypos)*h;
        f[2]+=ydist[j]*0.5;
    }
    if(order<2)
        return;
        
    /*step 4 square distances*/
    tmppos=c+3*n;
    tmppos2=c+4*n;
    tmppos3=c+5*n;
    f[3]=2;
    f[4]=2;
    for(j=0;j<loopmax;j++) {
        x=xdist[j];
        y=ydist[j];
        f[3]+=(tmppos[j]=-x*x)*0.5;
        f[4]+=(tmppos2[j]=-y*y)*0.5;
        f[5]+=(tmppos3[j]=-x*y)*0.5;
    }
    if(order<3)
        return;
    
    /*step 5 3/2 terms*/
    tmppos=c+6*n;
    tmppos2=c+7*n;
    tmppos3=c+8*n;
    tmppos4=c+9*n;
    for(j=0;j<loopmax;j++) {
        x=xdist[j];
        y=ydist[j];
        f[6]+=(tmppos[j]=x*x*x)*0.5;
        f[7]+=(tmppos2[j]=x*x*y)*0.5;
        f[8]+=(tmppos3[j]=x*y*y)*0.5;
        f[9]+=(tmppos4[j]=y*y*y)*0.5;
    }
    if(order<4)
        return;
    
    /*step 6*/
    tmppos=c+10*n;
    tmppos2=c+11*n;
    tmppos3=c+12*n;
    tmppos4=c+13*n;
    tmppos5=c+14*n;
    f[10]=12;
    f[12]=4;
    f[14]=12;
    for(j=0;j<loopmax;j++) {
        x=xdist[j];
        y=ydist[j];
        f[10]+=(tmppos[j]=-x*x*x*x)*0.5;
        f[11]+=(tmppos2[j]=-x*x*x*y)*0.5;
        f[12]+=(tmppos3[j]=-x*x*y*y)*0.5;
        f[13]+=(tmppos4[j]=-x*y*y*y)*0.5;
        f[14]+=(tmppos5[j]=-y*y*y*y)*0.5;
    }
}
/*------------------------------------------------------------------------*/
int gramschmidtt(double *f,double *A,int m,int n)
{
    int row,j,k,nm1=n-1,ret=1;
    double sum,*tmp1,*tmp2;
    for(row=0;row<m;row++) {
        tmp2=A+row*n;
        for(j=0;j<row;j++) {
            sum=0;
            tmp1=A+j*n;
            for(k=0;k<nm1;k++) 
                sum+=tmp1[k]*tmp2[k];
            for(k=0;k<nm1;k++)
                tmp2[k]-=sum*tmp1[k];
            f[row]-=sum*f[j];
        }
        sum=0;
        for(k=0;k<nm1;k++)
            sum+=tmp2[k]*tmp2[k];
        if(sum<1e-6){ 
            /*mexWarnMsgTxt("bad conditioned matrix");*/
            ret=0;
        }
        sum=1/sqrt(sum);
        for(k=0;k<nm1;k++)
            tmp2[k]*=sum;
        f[row]*=sum;
    }
    return ret;
}
/*------------------------------------------------------------------------*/
int minlinf(double* ret,double *f,double *A,int m,int n)
{
    int j,k,iteration,maxrowpos,maxcolpos,*basis,itmp,itmp2,margcostmaxind,l,iret;
    double maxrowvalue,tmp,tmp2,minrowvalue,*margcost,z,margcostmax,absmaxrowvalue,abstmp;
/*     for(j=0;j<n;j++)*/
/*          ret[j]=0;*/
    basis=mxCalloc(m+n-1,sizeof(int));
    margcost=mxCalloc(2*(n-1),sizeof(double));
/*     for(j=0;j<n+m-2;j++)*/
/*         basis[j]=j+1;*/
    
    /*The sign of the matrix has already been changed when it was created*/
    /*all elements of the first column is now negative, and hence the corresponding*/
    /*columns will be in the basis*/
    for(j=0;j<n-1;j++)
        basis[j]=j+1;
    for(;j<n+m-2;j++)
        basis[j]=j+n+m;
    basis[j]=0;
    /*create starting matrix and basis vector,*/
/*     for(j=0;j<n-1;j++) {*/
/*         if(A[j*m]>0) {*/
/*             for(k=0;k<m;k++)*/
/*                 A[k*m+j]=-A[k*m+j];*/
/*             basis[m+j-1]+=n+m-1;*/
/*         }*/
/*     }*/
/*     printmatrix(A,m,n);*/
    /*sum the vectors and add this value to the last row (eliminates the ones in the z row (which is excluded from the matrix))*/
    for(k=0;k<n;k++) {
        tmp=A[k*m];
        for(j=1;j<m-1;j++)
            tmp+=A[k*m+j];
        A[k*m+j]=tmp;
    }
    /*divide by the last row in the first column*/
    tmp=1/A[m-1];
    A[m-1]=-tmp;
    for(k=2;k<=n;k++) /*last row in every column exept first is divided by the value from the first*/
        A[k*m-1]*=tmp;
    for(j=1;j<n;j++) { /*perform gaussian elimination using the last row first column as pivot*/
        tmp2=A[(j+1)*m-1];
        for(k=0;k<m-1;k++)
            A[j*m+k]-=A[k]*tmp2;
    }
    /*update the first column*/
    for(k=0;k<m-1;k++)
        A[k]*=tmp;
    /*if z is negative, use the corresponding matrix instead*/
    if(f[0]<0) {
        for(k=0;k<n;k++) 
            f[k]=-f[k];
        for(k=0;k<n+m-1;k++)
            basis[k]=(basis[k]+n+m-1)%(2*(n+m-1));
    }
    /*step 1 completed. Now, start the simplex method. Note that the first column*/
    /*in A actually are the b-vector. Therefore, basis[0] corresponds to the second*/
    /*column in A*/
   
    for(iteration=0;iteration<10*(n+m);iteration++) { /*put a maximum of iterations for safefy*/
        /*start by calculating z and marginal costs*/
        z=0;
        for(k=0;k<n+n-2;k++)
            margcost[k]=0;
        for(j=n-1;j<n+m-1;j++) {
            itmp=basis[j]%(n+m-1);
            if(itmp<n) {
                z+=f[itmp]*A[j-n+1];
                for(k=1;k<n;k++) {
                    margcost[(k-1)*2]+=f[itmp]*A[k*m+j-n+1];
                }
            }
        }
        
        /*calculate corresponding marginal costs and find the most negative marginal cost to use as pivot*/
        margcostmax=1;
        for(k=0;k<n-1;k++) {
            itmp2=basis[k]%(n+m-1);
            if(itmp2<n) {
                /*this is the case if the column belongs to an original equation, hence f has a value*/
                margcost[k*2]-=f[itmp2]; /*take f into account*/
                margcost[k*2+1]=-margcost[k*2]; /*corresponding cost just opposite*/
                if(margcost[k*2]<0) { /*determine if it is the original or correponding column that is negative and pick that one*/
                    if(margcost[k*2]<margcostmax) {
                        margcostmax=margcost[k*2];
                        margcostmaxind=k*2;
                    }
                }
                else {
                    if(-margcost[k*2]<margcostmax) {
                        margcostmax=-margcost[k*2];
                        margcostmaxind=k*2+1;
                    }
                }
            }
            else {
                /*if the column belongs to the original basis, no f value but different expression for the corresponding marginal cost*/
                margcost[k*2+1]=2*z-margcost[k*2];
                if(margcost[k*2]<margcostmax) { 
                    margcostmax=margcost[k*2];
                    margcostmaxind=k*2;
                }
                if(margcost[k*2+1]<margcostmax) { /*since both can have the same sign, check both.*/
                    margcostmax=margcost[k*2+1];
                    margcostmaxind=k*2+1;
                }
            }
        }
        maxcolpos=margcostmaxind/2+1; /*since there are two positions in margcost for every in A, and since the first column in A contains b*/
        itmp=maxcolpos*m; /*since this value is calculated many times*/
        if(margcostmax>-1e-9) /*ignore negative marginal costs too small*/
            break;
        if(margcostmaxind%2==0) { /*the original vector, find pivot row*/
            maxrowpos=0;
            minrowvalue=A[0]/A[itmp];
            absmaxrowvalue=maxrowvalue=A[itmp];
            if(absmaxrowvalue<0)
                absmaxrowvalue=-absmaxrowvalue;
            for(k=1;k<m;k++) {
                tmp=A[k]/A[itmp+k];
                abstmp=A[itmp+k];
                if(abstmp<0)
                    abstmp=-abstmp;
                /*if the current value of maxrowvalue is too small, do not use this as pivot due to numerical errors*/
                if((tmp<minrowvalue||minrowvalue<0||absmaxrowvalue<abstmp*1e-6||maxrowvalue<0)&&tmp>0&&absmaxrowvalue*1e-6<abstmp) {
                    minrowvalue=tmp;
                    absmaxrowvalue=maxrowvalue=A[itmp+k];
                    if(absmaxrowvalue<0)
                        absmaxrowvalue=-absmaxrowvalue;
                    maxrowpos=k;
                }
            }
        }
        else { /*corresponding vector*/
            if(basis[maxcolpos-1]%(n+m-1)<n) { /*different cases depending on if the vector belongs to an equation or the original basis, case of equation*/
                maxrowpos=0;
                minrowvalue=-A[0]/A[itmp];
                absmaxrowvalue=maxrowvalue=A[itmp]=-A[itmp]; /*since it is the corresponding column, change sign on A'*/
                if(absmaxrowvalue<0)
                    absmaxrowvalue=-absmaxrowvalue;
                for(k=1;k<m;k++) { /*scan for minimum row*/
                    abstmp=tmp2=A[itmp+k]=-A[itmp+k];
                    if(abstmp<0)
                        abstmp=-abstmp;
                    tmp=A[k]/tmp2;
                    if((tmp<minrowvalue||minrowvalue<0||absmaxrowvalue<abstmp*1e-6||maxrowvalue<0)&&tmp>0&&absmaxrowvalue*1e-6<abstmp) {
                        minrowvalue=tmp;
                        absmaxrowvalue=maxrowvalue=tmp2;
                        if(absmaxrowvalue<0)
                            absmaxrowvalue=-absmaxrowvalue;
                        maxrowpos=k;
                    }
                }
                itmp2=basis[maxcolpos-1]%(n+m-1);
                f[itmp2]=-f[itmp2]; /*change f to negative value as well*/
            }
            else { /*the vector belonged to the original basis*/
                maxrowpos=0;
                absmaxrowvalue=maxrowvalue=A[itmp]=2*A[0]-A[itmp]; /*different formula for calculating the corresponding value*/
                if(absmaxrowvalue<0)
                    absmaxrowvalue=-absmaxrowvalue;
                minrowvalue=A[0]/maxrowvalue;
                for(k=1;k<m;k++) {
                    abstmp=tmp2=A[itmp+k]=2*A[k]-A[itmp+k]; /*calculate the correponding value at the same time as searching for minimum*/
                    if(abstmp<0)
                        abstmp=-abstmp;
                    tmp=A[k]/tmp2;
                    if((tmp<minrowvalue||minrowvalue<0||absmaxrowvalue<abstmp*1e-6||maxrowvalue<0)&&tmp>0&&absmaxrowvalue*1e-6<abstmp) {
                        minrowvalue=tmp;
                        absmaxrowvalue=maxrowvalue=tmp2;
                        if(absmaxrowvalue<0)
                            absmaxrowvalue=-absmaxrowvalue;
                        maxrowpos=k;
                    }
                }
            }
            basis[maxcolpos-1]=(basis[maxcolpos-1]+n+m-1)%(2*(n+m-1)); /*change the basis vector to the corresponding value*/
        }
        
        /*now the pivot element has been choosen and it is time to do the gaussian elimination*/
        tmp=1/maxrowvalue; /*multiplication is faster than division, therefore, save this value*/
        
        /*start by updating the other columns than the pivot column, first the columns before*/
        for(k=0;k<maxcolpos;k++) {
            l=k*m;
            tmp2=A[l+maxrowpos]*=tmp;
            for(j=0;j<maxrowpos;j++)
                A[l+j]-=tmp2*A[itmp+j];
            for(j=maxrowpos+1;j<m;j++)
                A[l+j]-=tmp2*A[itmp+j];
        }
        /*continue with the columns after the pivot column*/
        for(k=maxcolpos+1;k<n;k++) {
            l=k*m;
            tmp2=A[l+maxrowpos]*=tmp;
            for(j=0;j<maxrowpos;j++)
                A[l+j]-=tmp2*A[itmp+j];
            for(j=maxrowpos+1;j<m;j++)
                A[l+j]-=tmp2*A[itmp+j];
        }
        
        A[itmp+maxrowpos]=tmp; /*update the pivot element*/
        /*update the pivot column, first the elements before the pivot*/
        for(j=0;j<maxrowpos;j++) {
            A[itmp+j]=-A[itmp+j]*tmp;
        }
        /*then the elements after*/
        for(j=maxrowpos+1;j<m;j++) {
            A[itmp+j]=-A[itmp+j]*tmp;
        }
        /*update the basis vector, move the pivot vector to the basis*/
        itmp=basis[maxcolpos-1];
        basis[maxcolpos-1]=basis[n+maxrowpos-1];
        basis[n+maxrowpos-1]=itmp;
    }
    /*solution found, create the answer, first the elements in the basis, which has z or -z as value*/
    for(k=n-1;k<n+m-1;k++) {
        itmp=basis[k]%(n+m-1);
        if(itmp>=n)
            ret[itmp-n]=(basis[k]<(n+m)?-z:z)+0.5;
    }
    /*then the elements outside the basis whose values depends on their marginal costs*/
    for(k=0;k<n-1;k++) {
        itmp=basis[k]%(n+m-1);
        if(itmp>=n) {
            ret[itmp-n]=(basis[k]<(n+m-1)?margcost[2*k]-z:z-margcost[2*k])+0.5;
        }
    }
    if(fabs(z)<0.5) {
        iret=1;
    }
    else {
        iret=0;
    }
    /*solution failed due to maximum nr of iterations*/
    if(margcostmax<-1e-9)
        iret=0;
    mxFree(basis);
    mxFree(margcost);
    return iret;
}
/*------------------------------------------------------------------------*/
void printmatrix(double *A,int m,int n)
{
    int j,k;
    for(j=0;j<m;j++) {
        for(k=0;k<n;k++) {
            mexPrintf("%.16f ",A[k*m+j]);
        }
        mexPrintf("\n");
    }
}
/*------------------------------------------------------------------------*/
void gramschmidttoriginal(double *f,double *A,int m,int n)
{
    int row,j,k;
/*     char text[1000];*/
    double sum;
    for(row=0;row<m;row++) {
        for(j=0;j<row;j++) {
            sum=0;
            for(k=0;k<n;k++) 
                sum+=A[j*n+k]*A[row*n+k];
            for(k=0;k<n;k++)
                A[row*n+k]-=sum*A[j*n+k];
            f[row]-=sum*f[j];
            /*mexPrintf("sum=%f,row=%d j=%d\n",sum,row,j);*/
        }
        sum=0;
        for(k=0;k<n;k++)
            sum+=A[row*n+k]*A[row*n+k];
        if(sum<1e-6) 
            mexWarnMsgTxt("bad conditioned matrix");
        sum=1/sqrt(sum);
        for(k=0;k<n;k++)
            A[row*n+k]*=sum;
        f[row]*=sum;
    }
/*      sprintf(text,"m=%d n=%d",m,n);*/
/*      mexWarnMsgTxt(text);*/
}
/*------------------------------------------------------------------------*/
/* void getnewpos(const double *xcoord,const double* ycoord,const double xpos,const double ypos, const double *newxpos, const double *newypos, double *newx,double* newy,int ncoord, int npos)*/
void getnewpos(struct inputdata *input, double *newx,double* newy)
{
    double xpos,ypos,*xcoord,*ycoord,*newxpos,*newypos;
    int ncoord,npos;
    double *disttmp,tmp,varx,vary;
    int j,k;
    xpos=input->xpos;
    ypos=input->ypos;
    xcoord=input->xvortices;
    ycoord=input->yvortices;
    newxpos=input->newxpos;
    newypos=input->newypos;
    ncoord=input->nrvortices;
    npos=input->newcount;
    /*mexPrintf("getting new position, ncoord=%d,npos=%d\n",ncoord,npos);*/
    /*mexPrintf("newx=%x, newy=%x newxpos=%x newypos=%x\n",newx,newy,newxpos,newypos);*/
    if(ncoord==0) {
        /*mexPrintf("ncoord=0, newx=%x, newy=%x newxpos=%x newypos=%x\n",newx,newy,newxpos,newypos);*/
        *newx=newxpos[0]+xpos;
        /*mexPrintf("first value\n");*/
        *newy=newypos[0]+ypos;
        /*mexPrintf("returning first value\n");*/
        return;
    }
    /*mexPrintf("allocating disttmp\n");*/
    disttmp=mxMalloc(npos*sizeof(double));
    for(j=0;j<npos;j++) {
        disttmp[j]=1e99;
        for(k=0;k<ncoord;k++){
            varx=xcoord[k]-xpos-newxpos[j];
            vary=ycoord[k]-ypos-newypos[j];
            tmp=varx*varx+vary*vary;
            if(tmp<disttmp[j])
                disttmp[j]=tmp;
        }
    }
    k=0;
    tmp=disttmp[0];
    /*mexPrintf("Finding largest distance\n");*/
    for(j=1;j<npos;j++) {
        if(disttmp[j]>tmp) {
            tmp=disttmp[j];
            k=j;
        }
    }
   /* mexPrintf("writing result\n");*/
    *newx=newxpos[k]+xpos;
    *newy=newypos[k]+ypos;
    /*mexPrintf("max distance=%f newposindex=%d newpos =%fx%f new value=%fx%f\n",sqrt(tmp),k,newxpos[k],newypos[k],*newx,*newy);*/
    mxFree(disttmp);
}
/*------------------------------------------------------------------------*/
/* #define DEBUG_getnewposa*/
void getnewposa(struct inputdata *input, double *newx,double* newy)
{
    double xpos,ypos,*xcoord,*ycoord,*newxpos,*newypos,newposmaxdist;
    int ncoord,npos;
    double *disttmp,tmp,varx,vary,*xneighbors,*yneighbors,ytmp,ncutoff,x1,y1,k,xn,yn,
        currentxpos,currentypos,tmp2,maxdist,minneighborxpos,minneighborypos,xm,ym;
    int j,i,neighborcount=0,minind=0,minneighborind=-1,bothdir=0;
    xpos=input->xpos;
    ypos=input->ypos;
    xcoord=input->xvortices;
    ycoord=input->yvortices;
    newxpos=input->newxpos;
    newypos=input->newypos;
    ncoord=input->nrvortices;
    npos=input->newcount;
    newposmaxdist=input->newposmaxdist;
    #ifdef DEBUG_getnewposa
    mexPrintf("\ngetting new position, ncoord=%d,npos=%d xpos=%f ypos=%f\nMaximu allowed distance=%f\n",ncoord,npos,xpos,ypos,newposmaxdist);
    mexPrintf("newx=%x, newy=%x newxpos=%x newypos=%x\n",newx,newy,newxpos,newypos);
    #endif
    if(ncoord==0) {
        #ifdef DEBUG_getnewposa
        mexPrintf("ncoord=0, newx=%x, newy=%x newxpos=%x newypos=%x\n",newx,newy,newxpos,newypos);
        #endif
        *newx=newxpos[0]+xpos;
        #ifdef DEBUG_getnewposa
        mexPrintf("first value\n");
        #endif
        *newy=newypos[0]+ypos;
        #ifdef DEBUG_getnewposa
        mexPrintf("returning first value\n");
        #endif
        return;
    }
    #ifdef DEBUG_getnewposa
    mexPrintf("allocating disttmp\n");
    #endif
    disttmp=mxMalloc(npos*sizeof(double));
    for(j=0;j<npos;j++) {
        disttmp[j]=1e99;
        for(i=0;i<ncoord;i++){
            varx=xcoord[i]-xpos-newxpos[j];
            vary=ycoord[i]-ypos-newypos[j];
            tmp=varx*varx+vary*vary;
            if(tmp<disttmp[j])
                disttmp[j]=tmp;
        }
    }
    i=0;
    tmp=disttmp[0];
    #ifdef DEBUG_getnewposa
    mexPrintf("Finding largest distance\n");
    #endif
    for(j=1;j<npos;j++) {
        if(disttmp[j]>tmp) {
            tmp=disttmp[j];
            i=j;
        }
    }
    #ifdef DEBUG_getnewposa
    mexPrintf("largest distance=%f, ncoord=%d\n",tmp,ncoord);
    mexPrintf("writing result\n");
    #endif
    tmp=newxpos[i]*newxpos[i]+newypos[i]*newypos[i];
    ncutoff=4*newposmaxdist*newposmaxdist-4*sqrt(tmp)*newposmaxdist+tmp;
    #ifdef DEBUG_getnewposa
    mexPrintf("ncutoff=%f\n",ncutoff);
    #endif
    if(tmp<disttmp[i]||ncoord==0){ /*if the vortex has no neighbors closer than the center, accept this position as it is*/
        *newx=newxpos[i]+xpos;
        *newy=newypos[i]+ypos;
/*         mexPrintf("max distance=%f newposindex=%d newpos =%fx%f new value=%fx%f\n",sqrt(tmp),k,newxpos[k],newypos[k],*newx,*newy);*/
        mxFree(disttmp);
        return;
    }
    else {
        currentxpos=newxpos[i]+xpos;
        currentypos=newypos[i]+ypos;
        #ifdef DEBUG_getnewposa
        mexPrintf("currentxpos=%f currentypos=%f xpos=%f ypos=%f\n",currentxpos,currentypos,xpos,ypos);
        #endif
        xneighbors=mxMalloc((ncoord+1)*sizeof(double));
        yneighbors=mxMalloc((ncoord+1)*sizeof(double));
        if(input->includeself) {
            xneighbors[0]=xpos;
            yneighbors[0]=ypos;
            neighborcount=1;
            npos++; /*include the last element in newx/ypos, which is (0,0)*/
        }
        /*create list of neighbors close enough to possibly be important*/
        for(i=0;i<ncoord;i++){
            tmp=currentxpos-xcoord[i];
            ytmp=currentypos-ycoord[i];
            if(tmp*tmp+ytmp*ytmp<ncutoff){
                xneighbors[neighborcount]=xcoord[i];
                yneighbors[neighborcount]=ycoord[i];
                neighborcount++;
            }
        }
        #ifdef DEBUG_getnewposa
        mexPrintf("list of close neighbors contains %d elements\n",neighborcount);
        #endif
        if(neighborcount==0) {
            *newx=currentxpos;
            *newy=currentypos;
            mxFree(disttmp);
            return;
        }
        varx=xneighbors[0]-currentxpos;
        vary=yneighbors[0]-currentypos;
        tmp=varx*varx+vary*vary;
        for(i=1;i<neighborcount;i++) {
            varx=xneighbors[i]-currentxpos;
            vary=yneighbors[i]-currentypos;
            if(tmp>varx*varx+vary*vary){
                tmp=varx*varx+vary*vary;
                minind=i;
            }
        }
        x1=xneighbors[minind];
        y1=yneighbors[minind];
        #ifdef DEBUG_getnewposa
        mexPrintf("minimum position is %f x %f at position %d, distance=%f\n",x1,y1,minind,tmp);
        #endif
        /*xy1sqr=x1*x1+y1*y1;*/
        ytmp=(currentypos-y1)/(currentxpos-x1);
        if(ytmp>1||ytmp<-1){ /*x=k*y k is always supposed to be smaller than 1*/
            k=(currentxpos-x1)/(currentypos-y1); /*in case ytmp=inf*/
            #ifdef DEBUG_getnewposa
            mexPrintf("k = %f ytmp=%f\n",k,ytmp);
            #endif
            /*calculate the maximum allowed distance in this direction*/
            ytmp=(k*(xpos-x1)+ypos-y1)/(k*k+1);
            tmp2=sqrt(ytmp*ytmp-((x1-xpos)*(x1-xpos)+(y1-ypos)*(y1-ypos)-newposmaxdist*newposmaxdist)/(k*k+1));
            if((y1+tmp2+ytmp-currentypos)/(y1-currentypos)<0) /*eventuellt fel h�r!!!!!!*/
                maxdist=(tmp2+ytmp)*(tmp2+ytmp)*(k*k+1);
            else
                maxdist=(tmp2-ytmp)*(tmp2-ytmp)*(k*k+1);
            #ifdef DEBUG_getnewposa
            mexPrintf("maxdist=%f ytmp=%f tmp2=%f (tmp2-ytmp+y1)=%f (currentypos-y1)=%f\n",maxdist,ytmp,tmp2,(tmp2-ytmp+y1),(currentypos-y1));
            #endif
            for(i=0;i<neighborcount;i++){
                if(i==minind)
                    continue;
                yn=0.5*((y1-yneighbors[i])*(y1-yneighbors[i])+(x1-xneighbors[i])*(x1-xneighbors[i]))/
                    (k*(xneighbors[i]-x1)+yneighbors[i]-y1);
                xn=k*yn;
/*                 ytmp=(yn-y1)*(yn-y1)+(xn-x1)*(xn-x1);*/
                ytmp=xn*xn+yn*yn;
                #ifdef DEBUG_getnewposa
                mexPrintf("i=%d pos %f x %f yn=%f y1=%f xn=%f x1=%f ytmp=%f\n",i,xneighbors[i],yneighbors[i],yn,y1,xn,x1,ytmp);
                #endif
                if(ytmp>tmp&&ytmp<maxdist&&(y1-currentypos)/yn<0) {/*if the distance is larger than the minimum distance and the sign is correct*/
                    maxdist=ytmp;
                    minneighborxpos=xn+x1;
                    minneighborypos=yn+y1;
                    minneighborind=i;
                }
            }
            if(minneighborind==-1) {
                ytmp=sqrt(maxdist/(k*k+1))*(currentypos-y1>0?1:-1);
                minneighborxpos=x1+k*ytmp;
                minneighborypos=y1+ytmp;
            }
            #ifdef DEBUG_getnewposa
            mexPrintf("maxdist=%f dist=%f\n",maxdist,tmp);
            mexPrintf("best value: %f x %f, ind=%d\n",minneighborxpos,minneighborypos,minneighborind);
            #endif
        }
        else {
            k=ytmp;
            #ifdef DEBUG_getnewposa
            mexPrintf("k = %f ytmp=%f\n",k,ytmp);
            #endif
            /*calculate the maximum allowed distance in this direction*/
            ytmp=(xpos-x1+k*(ypos-y1))/(k*k+1);
            tmp2=sqrt(ytmp*ytmp-((x1-xpos)*(x1-xpos)+(y1-ypos)*(y1-ypos)-newposmaxdist*newposmaxdist)/(k*k+1));
            if((x1+tmp2+ytmp-currentxpos)/(x1-currentxpos)<0)
                maxdist=(tmp2+ytmp)*(tmp2+ytmp)*(k*k+1);
            else
                maxdist=(tmp2-ytmp)*(tmp2-ytmp)*(k*k+1);
            #ifdef DEBUG_getnewposa
            mexPrintf("maxdist=%f ytmp=%f tmp2=%f (tmp2-ytmp+y1)=%f (currentxpos-x1)=%f\n",maxdist,ytmp,tmp2,(tmp2-ytmp+y1),(currentxpos-x1));
            #endif
            for(i=0;i<neighborcount;i++){
                if(i==minind)
                    continue;
                xn=0.5*((y1-yneighbors[i])*(y1-yneighbors[i])+(x1-xneighbors[i])*(x1-xneighbors[i]))/
                    (xneighbors[i]-x1+k*(yneighbors[i]-y1));
                yn=k*xn;
/*                 ytmp=(yn-y1)*(yn-y1)+(xn-x1)*(xn-x1);*/
                ytmp=xn*xn+yn*yn;
                #ifdef DEBUG_getnewposa
                mexPrintf("i=%d pos %f x %f yn=%f y1=%f xn=%f x1=%f ytmp=%f\n",i,xneighbors[i],yneighbors[i],yn,y1,xn,x1,ytmp);
                #endif
                if(ytmp>tmp&&ytmp<maxdist&&(x1-currentxpos)/xn<0) {/*if the distance is larger than the minimum distance and the sign is correct*/
                    maxdist=ytmp;
                    minneighborxpos=xn+x1;
                    minneighborypos=yn+y1;
                    minneighborind=i;
                }
            }
            if(minneighborind==-1) {
                ytmp=sqrt(maxdist/(k*k+1))*(currentxpos-x1>0?1:-1);
                minneighborxpos=x1+ytmp;
                minneighborypos=y1+k*ytmp;
            }
            #ifdef DEBUG_getnewposa
            mexPrintf("maxdist=%f dist=%f\n",maxdist,tmp);
            mexPrintf("best value: %f x %f, ind=%d\n",minneighborxpos,minneighborypos,minneighborind);
            #endif
        }
        if(minneighborind!=-1) { /*make a second step*/
            xm=(x1+xneighbors[minneighborind])/2;
            ym=(y1+yneighbors[minneighborind])/2; /*calculate the mean point between the two points closest to the current position*/
            currentxpos=minneighborxpos;
            currentypos=minneighborypos;
            ytmp=-(xneighbors[minneighborind]-x1)/(yneighbors[minneighborind]-y1);
            #ifdef DEBUG_getnewposa
            mexPrintf("ytmp=%f xm=%f ym=%f\n",ytmp,xm,ym);
            #endif
            if(xm==currentxpos&&ym==currentypos)
                bothdir=1;
            if(ytmp>1||ytmp<-1){ /*x=k*y k is always supposed to be smaller than 1*/
                k=-(yneighbors[minneighborind]-y1)/(xneighbors[minneighborind]-x1); /*in case ytmp=inf*/
                #ifdef DEBUG_getnewposa
                mexPrintf("k = %f ytmp=%f\n",k,ytmp);
                #endif
                /*calculate the maximum allowed distance in this direction*/
                ytmp=(k*(xpos-xm)+ypos-ym)/(k*k+1);
                tmp2=sqrt(ytmp*ytmp-((xm-xpos)*(xm-xpos)+(ym-ypos)*(ym-ypos)-newposmaxdist*newposmaxdist)/(k*k+1));
                if(bothdir) { /*in this rare case, use only the smallest of the two (not correct, but this should basically not happen in any real case)*/
                    if(ytmp<0)
                        maxdist=(tmp2+ytmp)*(tmp2+ytmp)*(k*k+1);
                    else
                        maxdist=(tmp2+ytmp)*(tmp2+ytmp)*(k*k+1);
                }
                else {
                    if((ym+tmp2+ytmp-currentypos)/(ym-currentypos)<0)
                        maxdist=(tmp2+ytmp)*(tmp2+ytmp)*(k*k+1);
                    else
                        maxdist=(tmp2-ytmp)*(tmp2-ytmp)*(k*k+1);
                }
                #ifdef DEBUG_getnewposa
                mexPrintf("maxdist=%f ytmp=%f tmp2=%f (tmp2-ytmp+y1)=%f (currentypos-y1)=%f\n",maxdist,ytmp,tmp2,(tmp2-ytmp+y1),(currentypos-y1));
                #endif
                minneighborind=-1;
                tmp=(currentxpos-xm)*(currentxpos-xm)+(currentypos-ym)*(currentypos-ym);
                for(i=0;i<neighborcount;i++){
                    if(i==minind) /*perhaps not necessary this time*/
                        continue;
                    yn=0.5*((ym-yneighbors[i])*(ym-yneighbors[i])-(y1-ym)*(y1-ym)+(xm-xneighbors[i])*(xm-xneighbors[i])-(x1-xm)*(x1-xm))/
                    (k*(xneighbors[i]-x1)+yneighbors[i]-y1);
                    xn=k*yn;
                    /*                 ytmp=(yn-y1)*(yn-y1)+(xn-x1)*(xn-x1);*/
                    ytmp=xn*xn+yn*yn;
                    #ifdef DEBUG_getnewposa
                    mexPrintf("i=%d pos %f x %f yn=%f y1=%f xn=%f x1=%f ytmp=%f\n",i,xneighbors[i],yneighbors[i],yn,y1,xn,x1,ytmp);
                    #endif
                    if(ytmp>tmp&&ytmp<maxdist&&(ym-currentypos)/yn<=0) {/*if the distance is larger than the minimum distance and the sign is correct*/
                        maxdist=ytmp;
                        minneighborxpos=xn+xm;
                        minneighborypos=yn+ym;
                        minneighborind=i;
                    }
                }
                if(minneighborind==-1) {
                    ytmp=sqrt(maxdist/(k*k+1))*(currentypos-ym>0?1:-1);
                    minneighborxpos=xm+k*ytmp;
                    minneighborypos=ym+ytmp;
                }
                #ifdef DEBUG_getnewposa
                mexPrintf("maxdist=%f dist=%f\n",maxdist,tmp);
                mexPrintf("best value: %f x %f, ind=%d\n",minneighborxpos,minneighborypos,minneighborind);
                #endif
            }
            else {
                k=ytmp; /*in case ytmp=inf*/
                #ifdef DEBUG_getnewposa
                mexPrintf("k = %f ytmp=%f\n",k,ytmp);
                #endif
                /*calculate the maximum allowed distance in this direction*/
                ytmp=(xpos-xm+k*(ypos-ym))/(k*k+1);
                tmp2=sqrt(ytmp*ytmp-((xm-xpos)*(xm-xpos)+(ym-ypos)*(ym-ypos)-newposmaxdist*newposmaxdist)/(k*k+1));
                if(bothdir) { /*in this rare case, use only the smallest of the two (not correct, but this should basically not happen in any real case)*/
                    if(ytmp<0)
                        maxdist=(tmp2+ytmp)*(tmp2+ytmp)*(k*k+1);
                    else
                        maxdist=(tmp2+ytmp)*(tmp2+ytmp)*(k*k+1);
                }
                else {
                    if((xm+tmp2+ytmp-currentxpos)/(xm-currentxpos)<0)
                        maxdist=(tmp2+ytmp)*(tmp2+ytmp)*(k*k+1);
                    else
                        maxdist=(tmp2-ytmp)*(tmp2-ytmp)*(k*k+1);
                }
                #ifdef DEBUG_getnewposa
                mexPrintf("maxdist=%f ytmp=%f tmp2=%f (tmp2-ytmp+x1)=%f (currentypos-x1)=%f\n",maxdist,ytmp,tmp2,(tmp2-ytmp+x1),(currentxpos-x1));
                #endif
                minneighborind=-1;
                tmp=(currentxpos-xm)*(currentxpos-xm)+(currentypos-ym)*(currentypos-ym);
                for(i=0;i<neighborcount;i++){
                    if(i==minind) /*perhaps not necessary this time*/
                        continue;
                    xn=0.5*((ym-yneighbors[i])*(ym-yneighbors[i])-(y1-ym)*(y1-ym)+(xm-xneighbors[i])*(xm-xneighbors[i])-(x1-xm)*(x1-xm))/
                    (xneighbors[i]-x1+k*(yneighbors[i]-y1));
                    yn=k*xn;
                    /*                 ytmp=(yn-y1)*(yn-y1)+(xn-x1)*(xn-x1);*/
                    ytmp=xn*xn+yn*yn;
                    #ifdef DEBUG_getnewposa
                    mexPrintf("i=%d pos %f x %f yn=%f y1=%f xn=%f x1=%f ytmp=%f\n",i,xneighbors[i],yneighbors[i],yn,y1,xn,x1,ytmp);
                    #endif
                    if(ytmp>tmp&&ytmp<maxdist&&(xm-currentxpos)/xn<=0) {/*if the distance is larger than the minimum distance and the sign is correct*/
                        maxdist=ytmp;
                        minneighborxpos=xn+xm;
                        minneighborypos=yn+ym;
                        minneighborind=i;
                    }
                }
                if(minneighborind==-1) {
                    ytmp=sqrt(maxdist/(k*k+1))*(currentxpos-xm>0?1:-1);
                    minneighborxpos=xm+ytmp;
                    minneighborypos=ym+k*ytmp;
                }
                #ifdef DEBUG_getnewposa
                mexPrintf("maxdist=%f dist=%f\n",maxdist,tmp);
                mexPrintf("best value: %f x %f, ind=%d\n",minneighborxpos,minneighborypos,minneighborind);
                #endif
            }
        }
    }
    *newx=minneighborxpos;
    *newy=minneighborypos;
    mxFree(disttmp);
}

/*------------------------------------------------------------------------*/
void addcomplexelement(mxArray *array,double xval,double yval)
{
    double *arr;
    int nrelements=mxGetN(array);
    arr=mxGetPr(array);
    arr=mxRealloc(arr,(nrelements+1)*sizeof(double));
    arr[nrelements]=xval;
    mxSetPr(array,arr);
    arr=mxGetPi(array);
    arr=mxRealloc(arr,(nrelements+1)*sizeof(double));
    arr[nrelements]=yval;
    mxSetPi(array,arr);
    mxSetN(array,nrelements+1);
}
/*------------------------------------------------------------------------*/
void resizerealarray(mxArray *array,int size)
{
    double *arr=mxGetPr(array);
    arr=arr=mxRealloc(arr,size*sizeof(double));
    mxSetPr(array,arr);
    mxSetN(array,size);
}
int solvediffusioneq(struct inputdata *input,double *fractions,int *addednr)
{    
    /*input.xpos,input.ypos - position of current vortex (in)*/
    /*input.xvortices,input.yvortices - list of neighboring vortices (in/out)*/
    /*input.alloccount - length of xvortices,yvortices (in)*/
    /*input.nrvortices - number of vorticies in xvorticies (in)*/
    /*input.hnu - h_nu valeu (in)*/
    /*input.order - order of VRM (in)*/
    /*input.newxpos,input.newypos - list of positions for new vortices (in)*/
    /*input.newcount - number of elements in newxpos (in)*/
    /*fractions - how to distribute the vorticity (out)*/
    /*addednr - number of new vortices added (can be found in xvortices,yvortices) (out)*/
  int n,m,order=input->order,success,j,nrvortices=input->nrvortices,i;
  double* f,*A,tmp,rxtmp,rytmp;/*,newxval,newyval;*/
  n=(order+1)*(order+2)/2;
  m=nrvortices+1+input->includeself;
  *addednr=0;
/*   if(order<1||order>4)*/
/*        mexErrMsgTxt("Unsupported order of diffusion equations");*/
  
  /* inputs C and X */
/*   if (!ISDOUBLEMATRIX(prhs[0]) || !ISDOUBLEMATRIX(prhs[1]))*/
/*     mexErrMsgTxt("First two arguments must be double matrices.");*/


  /* create output */
/*   plhs[1] = mxDuplicateArray(prhs[1]);*/
/*   plhs[2] = mxCreateDoubleMatrix(1,n,mxREAL);*/
/*   plhs[3] = mxCreateDoubleMatrix(m,n,mxREAL);*/
/*   f=mxGetPr(plhs[2]);*/
/*   A=mxGetPr(plhs[3]);*/
  
   /*if the number of equations are not enough*/
   /*mexPrintf("m=%d n=%d\n",m,n);*/
/*    mexPrintf("vortex %f x %f\n",input->xpos,input->ypos);*/
/*    for(i=0;i<nrvortices;i++){*/
/*        mexPrintf("%f x %f\n",input->xvortices[i],input->yvortices[i]);*/
/*    }*/
   while(m<n+1) {
       if(input->alloccount==input->nrvortices) {
           return 0;
       }
       if(input->noaddelete&&input->includeself==0) {
           return 0;
       }
       /*mexPrintf("getting new pos, nrvorticies=%d, xvorticies=%x,yvortices=%x\n",nrvortices,input->xvortices,input->yvortices);*/
/*        getnewpos(input->xvortices, input->yvortices,input->xpos,*/
/*         input->ypos,input->newxpos,input->newypos, */
/*         input->xvortices+nrvortices,input->yvortices+nrvortices,input->nrvortices,input->newcount);*/
      input->getnewpos(input, input->xvortices+input->nrvortices,input->yvortices+input->nrvortices);
      m++;
/*       mexPrintf("Added element %f+%fi m=%d\n",*(input->xvortices+input->nrvortices),*(input->yvortices+input->nrvortices),m);*/
      
      if(input->includeboundary) {
          /*if the added point is a mirror point, the original point has to be available to diffuse properly. If the added point is a real point, check the distance to the mirror point*/
          if(getreflectionpoint(input->boundary,input->xvortices[input->nrvortices],input->yvortices[input->nrvortices],&rxtmp,&rytmp)!=input->side){
              input->nrvortices++;
              m++;
              input->xvortices[input->nrvortices]=rxtmp;
              input->yvortices[input->nrvortices]=rytmp;
/*               mexPrintf("Added original %f+%fi\n",input->xvortices[input->nrvortices],input->yvortices[input->nrvortices]);*/
          }
          else if(rxtmp*rxtmp+rytmp*rytmp<input->sqrcutoff){ /*store the mirror first, then the original*/
              input->xvortices[input->nrvortices+1]=input->xvortices[input->nrvortices];
              input->yvortices[input->nrvortices+1]=input->yvortices[input->nrvortices];
              input->xvortices[input->nrvortices]=rxtmp;
              input->yvortices[input->nrvortices]=rytmp;
/*               mexPrintf("Added mirror %f+%fi\n",input->xvortices[input->nrvortices],input->yvortices[input->nrvortices]);*/
              m++;
              input->nrvortices++;
          }
      }
      input->nrvortices++;
/*       mexPrintf("Added element %f+%fi m=%d\n",newxval,newyval,m);*/
     /* mexPrintf("2: m=%d n=%d\n",m,n);*/
   }
   /*plhs[0] = mxCreateDoubleMatrix(1,m-1,mxREAL);*/
   f=mxCalloc(n,sizeof(double));
   A=mxCalloc(m*n,sizeof(double));
  /*plhs[0]=mxCreateDoubleMatrix(1,n,mxREAL);*/
  /*plhs[1]=mxCreateDoubleMatrix(m,n,mxREAL);*/

  /* evaluate the result */
  /* mexPrintf("Before getdiffusioneq 1 m=%d n=%d includeself=%d\n",m,n,input->includeself);*/
  getdiffusioneq(f, A,input->xpos,
  input->ypos,input->xvortices,input->yvortices,
  input->hnu,m,order,input->includeself);
 /*  mexPrintf("After getdiffusioneq 1\n");*/
/*   if(input->debug==1) {*/
/*       mexPrintf("Matrix A equals\n");*/
/*       printmatrix(A,m,n);*/
/*       mexPrintf("f equals: ");*/
/*       printmatrix(f,n,1);*/
/*   }*/
  if(gramschmidtt(f, A,n,m)) {
/*       if(input->debug==1) {*/
/*           mexPrintf("Matrix A equals\n");*/
/*           printmatrix(A,m,n);*/
/*           mexPrintf("f equals: ");*/
/*           printmatrix(f,n,1);*/
/*       }*/
      success=minlinf(fractions,f,A,m,n);
      if(success) {
          tmp=0;
          for(i=0;i<m-1;i++)
              tmp+=fractions[i];
          /*mexPrintf("tmp=%f\n",tmp);*/
          if(tmp>(1+CIRCTOL)||tmp<(1-CIRCTOL)){
              /*mexPrintf("Tolerance error\n");*/
              success=0;
          }
      }
  }
  else {
/*       if(input->debug==1) {*/
/*           mexPrintf("Gram schmidth failed\nMatrix A equals\n");*/
/*           printmatrix(A,m,n);*/
/*           mexPrintf("f equals: ");*/
/*           printmatrix(f,n,1);*/
/*       }*/
      success=0;
  }
  while(success==0) {
      if(input->alloccount==input->nrvortices||input->nrvortices-nrvortices>=input->newcount) {
          mxFree(f);
          mxFree(A);
          return 0;
      }
      if(input->noaddelete&&input->includeself==0){
          mxFree(f);
          mxFree(A);
          return 0;
      }
/*       getnewpos(input->xvortices, input->yvortices,input->xpos,*/
/*         input->ypos,input->newxpos,input->newypos, */
/*         input->xvortices+nrvortices,input->yvortices+nrvortices,input->nrvortices,input->newcount);*/
      input->getnewpos(input, input->xvortices+input->nrvortices,input->yvortices+input->nrvortices);
      if(input->includeboundary) {
          /*if the added point is a mirror point, the original point has to be available to diffuse properly. If the added point is a real point, check the distance to the mirror point*/
          if(getreflectionpoint(input->boundary,input->xvortices[input->nrvortices],input->yvortices[input->nrvortices],&rxtmp,&rytmp)!=input->side){
              input->nrvortices++;
              m++;
              input->xvortices[input->nrvortices]=rxtmp;
              input->yvortices[input->nrvortices]=rytmp;
          }
          else if(rxtmp*rxtmp+rytmp*rytmp<input->sqrcutoff){ /*store the mirror first, then the original*/
              input->xvortices[input->nrvortices+1]=input->xvortices[input->nrvortices];
              input->yvortices[input->nrvortices+1]=input->yvortices[input->nrvortices];
              input->xvortices[input->nrvortices]=rxtmp;
              input->yvortices[input->nrvortices]=rytmp;
              m++;
              input->nrvortices++;
          }
      }
      input->nrvortices++;
      /*addcomplexelement(plhs[1],newxval,newyval);*/
      /*resizerealarray(plhs[0],m);*/
      m++;
/*       mexPrintf("Added element %f+%fi m=%d\n",*(input->xvortices+input->nrvortices),*(input->yvortices+input->nrvortices),m);*/
      mxFree(A);
      A=mxCalloc(m*n,sizeof(double));
      for(j=0;j<n;j++)
          f[j]=0;
     /*  mexPrintf("Before getdiffusioneq 2\n");*/
      getdiffusioneq(f, A,input->xpos,
        input->ypos,input->xvortices,input->yvortices, 
        input->hnu,m,order,input->includeself);
     /* mexPrintf("After getdiffusioneq 2\n");*/
/*       if(input->debug==1) {*/
/*           mexPrintf("Matrix A equals\n");*/
/*           printmatrix(A,m,n);*/
/*           mexPrintf("f equals: ");*/
/*           printmatrix(f,n,1);*/
/*       }*/
      if(gramschmidtt(f, A,n,m)) {
          /* mexPrintf("After gramschmidtt 2\n");*/
/*           if(input->debug==1) {*/
/*               mexPrintf("Matrix A equals\n");*/
/*               printmatrix(A,m,n);*/
/*               mexPrintf("f equals: ");*/
          /*               printmatrix(f,n,1);*/
          /*           }*/
          success=minlinf(fractions,f,A,m,n);
          if(success) {
              tmp=0;
              for(i=0;i<m-1;i++)
                  tmp+=fractions[i];
/*               mexPrintf("tmp=%f\n",tmp);*/
              if(tmp>(1+CIRCTOL)||tmp<(1-CIRCTOL)){
                  mexPrintf("Tolerance error\n");
                  success=0;
              }
          }
          /* mexPrintf("After minlinf 2\n");*/
      }
      else {
/*           if(input->debug==1) {*/
/*               mexPrintf("Gram schmidth failed\nMatrix A equals\n");*/
/*               printmatrix(A,m,n);*/
/*               mexPrintf("f equals: ");*/
/*               printmatrix(f,n,1);*/
/*           }*/
          success=0;
      }
  }
  *addednr=input->nrvortices-nrvortices;
  mxFree(f);
  mxFree(A);
  return 1;
}

struct boundarydata mindistfromvector(double xpos,double ypos,double* xvect,double *yvect, double cutoff,int vectcount)
{
    struct boundarydata ret;
    int i;
    double a1,a2,b1,b2;
    double scalar,adist,tmp1,tmp2;
    if(vectcount<1) {
        memset(&ret,0,sizeof(struct boundarydata));
        return ret;
    }
    ret.distance=cutoff;
    for(i=0;i<vectcount*2;i+=2){
        a1=xvect[i]-xvect[i+1];/*first row starting points, second row, end points*/
        a2=yvect[i]-yvect[i+1];
        b1=xvect[i]-xpos;
        b2=yvect[i]-ypos;
        adist=a1*a1+a2*a2;
        scalar=a1*b1+a2*b2;
        if(scalar<0) { /*if the first endpoint is the closest point*/
            tmp1=b1*b1+b2*b2;
            if(tmp1<ret.distance) {
                ret.distance=tmp1;
                ret.basexvector=xvect[i];
                ret.baseyvector=yvect[i];
                ret.xvector=b2;
                ret.yvector=-b1;
            }
        }
        else if(scalar>adist) { /*if the second endpoint is the closest point*/
            b1=xvect[i+1]-xpos;
            b2=yvect[i+1]-ypos;
            tmp1=b1*b1+b2*b2;
            if(tmp1<ret.distance){
                ret.basexvector=xvect[i+1];
                ret.baseyvector=yvect[i+1];
                ret.xvector=b2;
                ret.yvector=-b1;
                ret.distance=tmp1;
            }
        }
        else { /*check for points on the vector*/
            tmp1=a2*b1-a1*b2;
            tmp2=tmp1*tmp1/adist;
            if(tmp2<ret.distance) {
                ret.distance=tmp2;
                ret.basexvector=xvect[i];
                ret.baseyvector=yvect[i];
                ret.xvector=a1;
                ret.yvector=a2;
            }
        }
    }
    tmp1=sqrt(ret.xvector*ret.xvector+ret.yvector*ret.yvector); /*normalise the vector to simplify computations later*/
    ret.xvector/=tmp1;
    ret.yvector/=tmp1;
    ret.distance=sqrt(ret.distance);
    if(fabs(ret.xvector)>fabs(ret.yvector))
        ret.majordir=1;
    else
        ret.majordir=0;
    return ret;
}

int getreflectionpoint(struct boundarydata *boundary,double xpos,double ypos,double *rxpos,double *rypos) 
{
    double b1,b2,scalar,tmp;
    b1=boundary->basexvector-xpos;
    b2=boundary->baseyvector-ypos;
    scalar=b1*boundary->xvector+b2*boundary->yvector;
    if(boundary->majordir==1) {
        tmp=b2-scalar*boundary->yvector;
        *rxpos=2*(b1-scalar*boundary->xvector)+xpos;
        *rypos=2*tmp+ypos;
        if(tmp>0)
            return 1;
        return 0;
    }
    else {
        tmp=b1-scalar*boundary->xvector;
        *rxpos=2*tmp+xpos;
        *rypos=2*(b2-scalar*boundary->yvector)+ypos;
        if(tmp>0)
            return 1;
        return 0;
    }     
}
