'cylinderflow_cpugpu'


plotinterval = 2;
saveinterval = inf;%250;

startnlevels = 5;
starttheta = 0.4;

% defaults
atfunc  = @at3b;
ifAT = 1;
filesuffix = 'at3b';

numruns = 6;
for run = 4:numruns

if run <= numruns/2
'GPU'
  fmm = @fmm2d_8threads;
else
'CPU'
  fmm = @fmm2d_8threads_cpu;
end



nlevels = startnlevels;
theta = starttheta;
at = initat(nlevels, theta);


starttime = clock;

filesuffix = strcat('cpugpu',int2str(run));


Re=10000; %3000;          %number of vortices released each time step. Increase to more quickly fill the domain with vortices
deltat=0.05;      %time step
timesteps=300;   %number of time steps
vasympt=1;        %asymptotic flow velocity
TSR=0.5;          %Rotational speed of the cylinder
vortexplot=true; %plot vortices
boundarytol=0;    %tolerance for calculating boundary velocity

nlevs = zeros(timesteps,1);
thets = zeros(timesteps,1);

time = zeros(timesteps, 9);
fmmtime = inf;


VRMorder=2;       %order of diffusion algorithm. Should be alteast 2, can be 3 and 4 as well
nodiffuselimit=2; %diffusion turned of outside this region
rkorder=4;        %order of runge kutta algorithm for convection. Can be 1, 2 and 4

nu=(2*vasympt)/Re;      %Kinematic viscosity
dv=sqrt(0.5*nu*deltat); %vortex release distance. Forces are quite insensitive to this value
xopt=sqrt(nu*deltat);   %smoother value for vortex interaction
Nreleasepoints=2*pi/dv; %Number of vortices released


releasepointangle=linspace(0,2*pi,Nreleasepoints+1);
releasepointangle(end)=[];
controlpoints=exp(releasepointangle*1i); %points where velocity should be zero
releasepoints=(1+dv)*controlpoints;      %points where vortices are released

%for the boundary condition, create matrix to solve to satisfy the no slip
%condition
A=zeros(length(controlpoints));
for m=1:length(releasepoints)
  A(:,m)=real(1i/(2*pi).*(conj(1./(controlpoints-releasepoints(m)))-1./(conj(controlpoints)-1./releasepoints(m))).*conj(1i.*controlpoints));
end
invA=inv(A); %From experience, this is usually faster than lu factorization

vortices=[];
gammas=[];
Forcesum=zeros(1,timesteps+1);

%VRM boundary
% clear VRMboundary
% VRMboundary(1,:)=controlpoints;
% VRMboundary(2,1:size(controlpoints,2)-1)=controlpoints(1,2:size(controlpoints,2));
% VRMboundary(2,size(controlpoints,2))=controlpoints(1,1);
VRMboundary=[];


for k=1:timesteps
k
  nlevs(k) = nlevels;
  thets(k) = theta;  
    
  %for the boundary condition
  delvortices=abs(vortices)<1+dv*2; %remove all vortices within this region of the flow
  vortices(delvortices)=[];
  gammas(delvortices)=[];
  tic;
  [f ti] = fmm([vortices 1./conj(vortices)],[gammas -gammas],controlpoints,'pot',1,'tol',boundarytol,'cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
  controlpointvelocities=real((vasympt-vasympt./conj(controlpoints).^2-conj(f)*1i/2/pi).*conj(1i.*controlpoints)); %the velocity at the control points
  boundarytime=toc;
  releasestrength=invA*transpose(-controlpointvelocities+TSR*vasympt); %strength of vortices to release to satisfy the boundary condition
  vortices=[vortices releasepoints]; %add vortices
  gammas=[gammas transpose(releasestrength)];

 
  
  tic;
  gammasold=gammas;
  if(mod(k,10)==0)
    [vortices gammas]=runVRM(vortices,gammas,nu,deltat,VRMorder,VRMboundary,ones(size(vortices))==1,abs(vortices)>nodiffuselimit);
   % disp(['Deletion step (' num2str(k) ')']);
  else
    [vortices gammas]=runVRM(vortices,gammas,nu,deltat,VRMorder,VRMboundary,abs(vortices)<1+dv*3,abs(vortices)>nodiffuselimit);
  end
  
  %random walk option, not used
%     vortices=vortices+2*nu*deltat*(randn(size(vortices))+1i*randn(size(vortices)));
  difftime=toc;
  
  
  %convection part
   %fmm call 1
  tic;
  [f ti] = fmm([vortices(1:length(gammasold)) 1./conj(vortices(1:length(gammasold)))],[gammasold -gammasold],vortices,'tol',1e-6,'pot',1,'smooth','oseen','cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
  vortexvelocities=vasympt-vasympt./conj(vortices).^2-conj(f)*1i/(2*pi);
  fmmtime=toc;
  
  if rkorder <=2
    %delete removed vortices from VRM
    delvortices=gammas==0;
    vortices(delvortices)=[];
    gammas(delvortices)=[];
    vortexvelocities(delvortices)=[];
  end
  
  if rkorder==1 %euler forward
    vortices=vortices+deltat*vortexvelocities;
  elseif rkorder==2 %heuns method
    %euler forward, the temporary step in heuns method
    vorticesnew=vortices+deltat*vortexvelocities;
    
    %second fulfillment of bc at temporary step
    delvortices=abs(vorticesnew)<1+dv; %remove all vortices within this region of the flow
    vorticesnew(delvortices)=[];
    gammas(delvortices)=[];
    vortices(delvortices)=[]; %also remove them from the original starting position
    vortexvelocities(delvortices)=[]; %also remove them from the original starting position
    tic;
    [f ti] = fmm([vortices 1./conj(vortices)],[gammas -gammas],controlpoints,'pot',1,'tol',boundarytol,'cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
    controlpointvelocities=real((vasympt-vasympt./conj(controlpoints).^2-conj(f)*1i/2/pi).*conj(1i.*controlpoints)); %the velocity at the control points
    boundarytime=boundarytime+toc;
    releasestrength=invA*transpose(-controlpointvelocities+TSR*vasympt); %strength of vortices to release to satisfy the boundary condition
    
    %fmm call 2
    tic;
    %add the temporary vortices here as well
    [f ti] = fmm([vorticesnew releasepoints 1./conj(vorticesnew) 1./conj(releasepoints)],[gammas transpose(releasestrength) -gammas -transpose(releasestrength)],vorticesnew,'tol',1e-6,'pot',1,'smooth','oseen','cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
    vortexvelocitiesnew=vasympt-vasympt./conj(vorticesnew).^2-conj(f)*1i/(2*pi);
    fmmtime=fmmtime+toc;
    
    %heuns method
    vortices=vortices+deltat*0.5*(vortexvelocities+vortexvelocitiesnew);
  elseif rkorder==4 %rk4
    gammasold=[gammasold zeros(1,length(gammas)-length(gammasold))];
    gammasmiddle=0.5*(gammas+gammasold);
    vorticesnew=vortices+0.5*deltat*vortexvelocities;
    
    %second fulfillment of bc at temporary step
    delvortices=abs(vorticesnew)<1+dv; %remove all vortices within this region of the flow
    vorticesnew(delvortices)=[];
    gammas(delvortices)=[];
    vortices(delvortices)=[]; %also remove them from the original starting position
    vortexvelocities(delvortices)=[]; %also remove them from the original starting position
    gammasmiddle(delvortices)=[];
    tic
    [f ti] = fmm([vortices 1./conj(vortices)],[gammas -gammas],controlpoints,'pot',1,'tol',boundarytol,'cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
    controlpointvelocities=real((vasympt-vasympt./conj(controlpoints).^2-conj(f)*1i/2/pi).*conj(1i.*controlpoints)); %the velocity at the control points
    boundarytime=boundarytime+toc;
    releasestrength=invA*transpose(-controlpointvelocities+TSR*vasympt); %strength of vortices to release to satisfy the boundary condition
    
    %fmm call 2
    tic;
    %add the temporary vortices here as well
    [f ti] = fmm([vorticesnew releasepoints 1./conj(vorticesnew) 1./conj(releasepoints)],[gammasmiddle transpose(releasestrength) -gammasmiddle -transpose(releasestrength)],vorticesnew,'tol',1e-6,'pot',1,'smooth','oseen','cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
    k2=vasympt-vasympt./conj(vorticesnew).^2-conj(f)*1i/(2*pi);
    fmmtime=fmmtime+toc;
    
    vorticesnew=vortices+0.5*deltat*k2;
    
    %third fulfillment of bc at temporary step
    delvortices=abs(vorticesnew)<1+dv; %remove all vortices within this region of the flow
    vorticesnew(delvortices)=[];
    gammas(delvortices)=[];
    vortices(delvortices)=[]; %also remove them from the original starting position
    vortexvelocities(delvortices)=[]; %also remove them from the original starting position
    gammasmiddle(delvortices)=[];
    k2(delvortices)=[];
    tic
    [f ti] = fmm([vortices 1./conj(vortices)],[gammas -gammas],controlpoints,'pot',1,'tol',boundarytol,'cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
    controlpointvelocities=real((vasympt-vasympt./conj(controlpoints).^2-conj(f)*1i/2/pi).*conj(1i.*controlpoints)); %the velocity at the control points
    boundarytime=boundarytime+toc;
    releasestrength=invA*transpose(-controlpointvelocities+TSR*vasympt); %strength of vortices to release to satisfy the boundary condition
    
    %fmm call 3
    tic;
    %add the temporary vortices here as well
    [f ti] = fmm([vorticesnew releasepoints 1./conj(vorticesnew) 1./conj(releasepoints)],[gammasmiddle transpose(releasestrength) -gammasmiddle -transpose(releasestrength)],vorticesnew,'tol',1e-6,'pot',1,'smooth','oseen','cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
    k3=vasympt-vasympt./conj(vorticesnew).^2-conj(f)*1i/(2*pi);
    fmmtime=fmmtime+toc;
    
    %delete removed vortices from VRM
    delvortices=gammas==0;
    vortices(delvortices)=[];
    gammas(delvortices)=[];
    vortexvelocities(delvortices)=[];
    k2(delvortices)=[];
    k3(delvortices)=[];
    
    vorticesnew=vortices+deltat*k3;

    %fourth fulfillment of bc at temporary step
    delvortices=abs(vorticesnew)<1+dv; %remove all vortices within this region of the flow
    vorticesnew(delvortices)=[];
    gammas(delvortices)=[];
    vortices(delvortices)=[]; %also remove them from the original starting position
    vortexvelocities(delvortices)=[]; %also remove them from the original starting position
    k2(delvortices)=[];
    k3(delvortices)=[];
    tic
    [f ti] = fmm([vortices 1./conj(vortices)],[gammas -gammas],controlpoints,'pot',1,'tol',boundarytol,'cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
    controlpointvelocities=real((vasympt-vasympt./conj(controlpoints).^2-conj(f)*1i/2/pi).*conj(1i.*controlpoints)); %the velocity at the control points
    boundarytime=boundarytime+toc;
    releasestrength=invA*transpose(-controlpointvelocities+TSR*vasympt); %strength of vortices to release to satisfy the boundary condition
    
    %fmm call 4
    tic;
    %add the temporary vortices here as well
    [f ti] = fmm([vorticesnew releasepoints 1./conj(vorticesnew) 1./conj(releasepoints)],[gammas transpose(releasestrength) -gammas -transpose(releasestrength)],vorticesnew,'tol',1e-6,'pot',1,'smooth','oseen','cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','times');
    k4=vasympt-vasympt./conj(vorticesnew).^2-conj(f)*1i/(2*pi);
    fmmtime=fmmtime+toc;

    vortices=vortices+deltat*1/6*(vortexvelocities+2*k2+2*k3+k4);
  end

  if(ifAT)
  [at nlevels theta] = atfunc (at, fmmtime, k, ti(1));
  end
  
  %force calculation
  Forcesum(k+1)=1i*sum(gammas.*(vortices-1./conj(vortices)));

  ti(9) = fmmtime;
  time(k,:) = ti; 
               
  
  %plot
  if mod(k,plotinterval)==0 && vortexplot
    figure(1)
    hold off
    plot(real(vortices(gammas<0)),imag(vortices(gammas<0)),'bo','MarkerSize',2);
    hold on
    plot(real(vortices(gammas>0)),imag(vortices(gammas>0)),'ro','MarkerSize',2);
    axis equal
    drawnow;
    figure(2)
    subplot(2,1,1)
    hold off
    plotyy([1:timesteps],nlevs,[1:timesteps],thets)      
    legend('nlevs','theta')
    subplot(2,1,2)
    plot([1:timesteps],time(:,9),'b',[1:timesteps],time(:,1),'g')
    legend('runtime','loadbalance');
  %  subplot(3,1,3)
  %  hold off;
  %  plot([1:k],partition,'r:');
  %  hold on;
  %  plot([1:k],connectivity,'b:');
  %  plot([1:k],P2M,'g');
  %  plot([1:k],M2M,'g:');
  %  plot([1:k],ML2L,'r');
  %  plot([1:k],L2P,'b:');
  %  plot([1:k],P2P,'m');
  %  plot([1:k],time,'k');
  %  legend('Partition','Connect','P2M','M2M','ML2L','L2P','P2P','Total'); 
    drawnow
  end
  if mod(k,saveinterval)==1
    save(strcat('examples/data/cylinderflow_save',filesuffix,int2str(k-1),'.mat'))
  end
 % disp(['Step ' num2str(k) ' number of vortices: ' num2str(length(vortices)) ' convection time: ' num2str(fmmtime) ' diffusion time: ' num2str(difftime) ' boundary velocity time: ' num2str(boundarytime)]);
end

Forces=diff(Forcesum)/deltat;


  clockelapsed = clock - starttime;
  totaltime = 3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6)
outfile = fopen(strcat('examples/data/cylinderflow_',filesuffix,'.out'),'w');
fprintf(outfile,'totaltime %f\n',totaltime);
fclose(outfile);

%En kalkyl av antalet"interaktioner" pÂ GPU vs CPU. Np2p = antalet direkt evaluations;
%  Nfmm = N*N - Np2p
Np2p = 0;
[M C] = fmm(vortices,ones(size(vortices)),'tol',1e-6,'pot',1,'smooth','oseen','cutoff',xopt*3,'xopt',xopt,'nlevels',nlevels,'theta',theta,'out','mesh');
% for each box in M, Ngpu += number of close particles
for l = 1:length(M)
  Np2p = Np2p + sum(sum(C{l}(:,:)==1));
end
Nfmm = length(vortices)*length(vortices) - Np2p;
Np2p 
Nfmm


save(strcat('examples/data/cylinderflow_',filesuffix,'.mat'))

end

