% Experiment: run galaxy collision simulation with varying starting conditions
'run_galaxy_startconds'

fmm = @fmm2d_8threads;
fmm_log = @fmm2d_log_8threads;



% common
Nstarsgal = 300000;    % number of stars per galaxy
Nstarsplot = 2000; % number of stars to plot
plotinterval = inf;%5;
saveinterval = inf;%500;

% initial data
R = 1; % radius

% final time and time-step
T1 = 0.12; % time for single-galaxy run
NR = 20; % Bigger NR shrinks timestep dt.

% defaults
atfunc  = @at3b;
ifAT = 1;

numreps = 3;
for rep = 1:numreps
for levrun = 1:5
startnlevels = 3 + levrun;
for thetrun = 1:5
filesuffix = strcat('startcond',int2str(levrun),int2str(thetrun),'rep',int2str(rep));
starttheta = 0.25 + thetrun*0.1;
nlevels = startnlevels;
theta = starttheta;
at = initat(nlevels, theta);

nlevs = 0;
thets = 0;

t = 0;



starttime = clock;

Nstars = Nstarsgal;

% uniformly inside disc of radius R
z = zeros(0,1);
while size(z,1) < Nstars
  zz = complex(R*(2*rand(Nstars-size(z,1),1)-1), ...
               R*(2*rand(Nstars-size(z,1),1)-1));
  zz(real(zz).^2+imag(zz).^2 > R^2) = [];
  z = [z; zz];
end
m = ones(Nstars,1); % mass
m = m/sum(m);
z0 = (m'*z)/sum(m); % initial center of mass

% Determine the initial velocity by enforcing Kepler orbits:

% Temporary initial xopt and cutoff values
xopt = 1e-7;
cutoff = 2*xopt;

% (1) gravitational force at initial positions
tol = R/Nstars/100;
F = conj(fmm(z,m,'pot',1,'smooth','scully','xopt',xopt,'cutoff',cutoff, ...
               'tol',tol,'nlevels',nlevels,'theta',theta));

% (2) magnitude towards the center of mass z0 (times |z-z0|)
Fc = max(real(z0-z).*real(F)+imag(z0-z).*imag(F),0);
%Fc = abs(F); % *** ?

% (3) interpreting this force as a centripetal force yields the speed
% (a multiplication with |z-z0| has been avoided here)
v = sqrt(Fc./m);

% (4) the speed gives the velocity when understood as a rotation about
% the center of mass z0
v = v.*complex(sin(angle(z-z0)),-cos(angle(z-z0)));

dt = 1/NR*R/mean(abs(v)); % on the average, each star travels a
                          % distance R in about NR time-steps

cutoff = R/NR;  % Cutoff on order of dt*v
xopt = cutoff/2;

numsteps = (T1)/dt; 

time = zeros(floor(numsteps), 9);
runtime = -1;

% evolve system
k = 0;
while t < T1
  k = k+1;
  % velocity Stˆrmer-Verlet method
  z = z+dt*v+dt*dt/2*F./m;
  Fold = F;
  nlevs(k) = nlevels;
  thets(k) = theta;  
  if mod(k,saveinterval) == 1 && rep == numreps
    [t k]
    ! rm *.log
    [p l] = fmm_log(z,m,'pot',1,'smooth','scully','xopt',xopt,'cutoff',cutoff, ...
               'tol',tol,'nlevels',nlevels,'theta',theta,'out','times');
    unix(strcat('mv *.log examples/data/cgalaxy_',int2str(k),filesuffix,'.vis'));
  else
    clockstart = clock;
    [p l] = fmm(z,m,'pot',1,'smooth','scully','xopt',xopt,'cutoff',cutoff, ...
               'tol',tol,'nlevels',nlevels,'theta',theta,'out','times');
    clockelapsed = clock - clockstart;
    runtime = 3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6);
  end
  F = conj(p);
  
  [at nlevels theta] = atfunc (at, runtime, k, l(1));
  
  l(9) = runtime;
  time(k,:) = l; 
               
  v = v+dt/2*(F+Fold)./m;
  t = t+dt;
  if mod(k,saveinterval) == 1
    save(strcat('examples/data/cgalaxy_',int2str(k),filesuffix,'.mat'));
  end
  
  if mod(k,plotinterval) == 0
    %M = fmm(z,m,'out','mesh','ndirect',24);
    figure(1)
    plot(z(1:Nstarsplot),'k.');
    title(strcat('one galaxy, t=',num2str(t)));
    %hold on, 
    %meshplot(M{end},'b:');
    %hold off,
    axis(4*[-R R -R R]),
    figure(2)
    subplot(2,1,1)
    plotyy([1:k],nlevs,[1:k],thets)
    legend('nlevels','theta')
    subplot(2,1,2)
    plot([1:k],time(1:k,9),'b',[1:k],time([1:k],1),'g')
    legend('runtime','loadbalance')
  %  subplot(3,1,3)
  %  hold off;
  %  plot([1:k],partition,'r:');
  %  hold on;
  %  plot([1:k],connectivity,'b:');
  %  plot([1:k],P2M,'g');
  %  plot([1:k],M2M,'g:');
  %  plot([1:k],ML2L,'r');
  %  plot([1:k],L2P,'b:');
  %  plot([1:k],P2P,'m');
  %  plot([1:k],time,'k');
  %  legend('Partition','Connect','P2M','M2M','ML2L','L2P','P2P','Total');
    drawnow;
  end

end

  clockelapsed = clock - starttime;
  totaltime = 3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6)
  outfile = fopen(strcat('examples/data/cgalaxy_',filesuffix,'.out'),'w');
fprintf(outfile,'totaltime %f\n',totaltime);  

  fclose(outfile);


save(strcat('examples/data/cgalaxy_',filesuffix,'.mat'))

end
end
end

