% Experiment: run K-H simulation with varying levels of autotuning
% A larger problem than in run_khinstability_small.m
'run_khinstability_cpugpu'

% common
timesteps=250;

plotinterval=1;
saveinterval = inf;%500;
deltat=0.001;


startnlevels = 5;
starttheta = 0.5;

% defaults
atfunc  = @at3b;
ifAT = 1;

numruns = 2;
for run = 1:numruns
pause
if run > numruns/2
    
'GPU'
  fmm = @fmm2d_8threads;
else
    
'CPU'
  fmm = @fmm2d_8threads_cpu;
end

filesuffix = int2str(run);

nlevels = startnlevels;
theta = starttheta;
at = initat(nlevels, theta);

nlevs = zeros(timesteps,1);
thets = zeros(timesteps,1);

time = zeros(timesteps, 9);
runtime = inf;


starttime = clock;


[xgrid ygrid]=meshgrid(0.005:0.005:199.995,-0.095:0.005:0.095);
%[xgrid ygrid]=meshgrid(0.005:0.05:199.995,-0.095:0.05:0.095);

vortices=xgrid+1i*ygrid;
vortices=transpose(vortices(:));
gammas=0.05*ones(size(vortices));
 gammas(imag(vortices)<0)=-0.05; %optional, changes a lot in how the simulation looks



for k=1:timesteps
k
  nlevs(k) = nlevels;
  thets(k) = theta;  
  clockstart = clock;
  [p l] = fmm(vortices,gammas,'tol',1e-6,'pot',1,'smooth','oseen','cutoff',0.05,'xopt',0.02,'nlevels',nlevels,'theta',theta,'out','times');
  clockelapsed = clock - clockstart;
  runtime = 3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6)
 max(max(abs(vortices)))
  vortexvelocities=-conj(p)*1i/(2*pi);

  if ifAT
    [at nlevels theta] = atfunc (at, runtime, k, l(1));
  end

  l(9) = runtime;
  time(k,:) = l; 
               
  vortices=vortices+deltat*vortexvelocities;
  if mod(k,plotinterval)==0
    figure(1)
    hold off
    plot(real(vortices(gammas<0)),imag(vortices(gammas<0)),'bx','MarkerSize',2);
    hold on
    plot(real(vortices(gammas>0)),imag(vortices(gammas>0)),'r+','MarkerSize',2);
    figure(2)
    hold off
    subplot(2,1,1)
    plotyy([1:timesteps],nlevs,[1:timesteps],thets)   
    legend('nlevs','theta')
    
    subplot(2,1,2)
    plot([1:k],time(1:k,9),'b',[1:k],time([1:k],1),'g')
    legend('runtime','loadbalance');
  
    drawnow;
  end
  if mod(k,saveinterval)==0
    save(strcat('examples/data/kh_cpugpu_save',int2str(k),filesuffix,'.mat'))
  end
end

  clockelapsed = clock - starttime;
  totaltime = 3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6)

save(strcat('examples/data/kh_cpugpu',filesuffix,'.mat'))

if(run == 1)
  constnlevels = nlevels;
  consttheta = theta;
end
end

