function create_plots_cputhreads(errorplot)

if nargin < 1
  errorplot = 0;
end

load('examples/data/cputhreads.mat');


cputime = mean(cputimereps,2);
 
for i = 1:length(cputhreads)
  su_cputime(i,:) = cputime(1,:)./cputime(i,:);
end

%  loadbalance = time(1);  
%  partition = time(2);
%  connectivity = time(3);
%  P2M = time(4);
%  M2M = time(5);
%  ML2L = time(6);
%  L2P = time(7);
%  P2P = time(8);
%  runtime = time(9); 

line1='-k'; % 'k-o'
line2='-.k'; % 'b:o'
line3='-.r'; % 'b-o'
line4='-g'; % 'r:o'
line5='.-k'; % 'r-o'
line6='--m'; % 'm-o'
line7='.-b'; % 'g-o'
line8='--m';

figure()
hold off
plot(cputhreads,cputime(:,9),'k-o','LineWidth',2);
hold on;
plot(cputhreads,cputime(:,2),'k-o','LineWidth',1);
plot(cputhreads,cputime(:,3),'b:o','LineWidth',1);
plot(cputhreads,cputime(:,4),'b-o','LineWidth',1);
plot(cputhreads,cputime(:,5),'r:o','LineWidth',1);
plot(cputhreads,cputime(:,6),'r-o','LineWidth',1);
plot(cputhreads,cputime(:,7),'m-o','LineWidth',1);
plot(cputhreads,cputime(:,8),'g-o','LineWidth',1);
if errorplot
   % get stddev, create box plots
   stddevs = std(cputimereps,0,2);
   h = errorbar(cputhreads,cputime(:,9),stddevs(:,9),'k.');
   errorbar_tick(h,2,'units');   
   h = errorbar(cputhreads,cputime(:,6),stddevs(:,6),'r.');
   errorbar_tick(h,2,'units');
   h = errorbar(cputhreads,cputime(:,7),stddevs(:,7),'m.');
   errorbar_tick(h,2,'units');
   h = errorbar(cputhreads,cputime(:,8),stddevs(:,8),'g.');
   errorbar_tick(h,2,'units');
end
%text(cputhreads(:),0.5*cputime(1,9)*ones(length(cputhreads),1),strcat('nlevels=',int2str(cpunlevels(:))));

%title('Runtime w/ CPU only');
h = legend('Total','Partition','Connectivity','P2M','M2M','ML2L','L2P','P2P','Location','EO');
set(h,'FontSize',10);
h = xlabel('Threads'); set(h,'FontSize',10);
h = ylabel('Time per iteration (s)'); set(h,'FontSize',10);
%axis([1e3 5e6 0 20]);
set(gca,'XTick',[1 2 4 8])
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 250]);
grid on
print -depsc examples/figs/CPUthreads_runtime.eps

figure()
hold off
plot(cputhreads,su_cputime(:,9),'k-x','LineWidth',2);
hold on;
plot(cputhreads,su_cputime(:,2),'k-o','LineWidth',1);
plot(cputhreads,su_cputime(:,3),'b:o','LineWidth',1);
plot(cputhreads,su_cputime(:,4),'b-o','LineWidth',1);
plot(cputhreads,su_cputime(:,5),'r:o','LineWidth',1);
plot(cputhreads,su_cputime(:,6),'r-o','LineWidth',1);
plot(cputhreads,su_cputime(:,7),'m-o','LineWidth',1);
plot(cputhreads,su_cputime(:,8),'g-o','LineWidth',1);
plot(cputhreads,cputhreads+1,'k--');
plot(cputhreads,su_cputime(:,9),'k-x','LineWidth',2); % repeat so it's on top
if errorplot
% get stddev, create box plots
  for i = 1:length(cputhreads)
    su_stddevs(i,:) = std(cputimereps(1,:,:)./cputimereps(i,:,:),0,2);
  end
   h = errorbar(cputhreads,su_cputime(:,9),su_stddevs(:,9),'k.');
   errorbar_tick(h,2,'units');
   h = errorbar(cputhreads,su_cputime(:,6),su_stddevs(:,6),'r.');
   errorbar_tick(h,2,'units');
   h = errorbar(cputhreads,su_cputime(:,7),su_stddevs(:,7),'m.');
   errorbar_tick(h,2,'units');
   h = errorbar(cputhreads,su_cputime(:,8),su_stddevs(:,8),'g.');
   errorbar_tick(h,2,'units');
end

%text(cputhreads(:),0.2*cputime(1,9)*ones(length(cputhreads),1),strcat('nlevels=',int2str(cpunlevels(:))));
%title('Speedup w/ CPU only')
h = legend('Total','Partition','Connectivity','P2M','M2M','ML2L','L2P','P2P','Location','EO');
set(h,'FontSize',10);
h = xlabel('Threads'); set(h,'FontSize',10);
h = ylabel('Speedup'); set(h,'FontSize',10);
%axis([1e3 5e6 0 20]);
set(gca,'XTick',[1 2 4 8])
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 250]);
grid on
print -depsc examples/figs/CPUthreads_speedup.eps

if 0
figure()
plot(cputhreads,cputime(:,9)./cputime(:,9),'k-o','LineWidth',4);
hold on;
plot(cputhreads,cputime(:,2)./cputime(:,9),'k-o','LineWidth',2);
plot(cputhreads,cputime(:,3)./cputime(:,9),'b:o','LineWidth',2);
plot(cputhreads,cputime(:,4)./cputime(:,9),'b-o','LineWidth',2);
plot(cputhreads,cputime(:,5)./cputime(:,9),'r:o','LineWidth',2);
plot(cputhreads,cputime(:,6)./cputime(:,9),'r-o','LineWidth',2);
plot(cputhreads,cputime(:,7)./cputime(:,9),'m-o','LineWidth',2);
plot(cputhreads,cputime(:,8)./cputime(:,9),'g-o','LineWidth',2);
legend('Total','Partition','Connectivity','P2M','M2M','ML2L','L2P','P2P');
title(strcat('Proportion w/ CPU only'))
xlabel('Threads')
ylabel('Proportion of runtime')
grid on
print -depsc examples/figs/CPUthreads_proportion.eps
end
