function create_plots_threads(errorplot)


if nargin < 1
  errorplot = 0;
end

load('examples/data/cputhreads.mat');
load('examples/data/gputhreads.mat');


cputime = mean(cputimereps,2);

gputime = mean(gputimereps,2);



%  loadbalance = time(1);  
%  partition = time(2);
%  connectivity = time(3);
%  P2M = time(4);
%  M2M = time(5);
%  ML2L = time(6);
%  L2P = time(7);
%  P2P = time(8);
%  runtime = time(9); 

figure()
hold off
plot(gputhreads,gputime(:,9),'r-o','LineWidth',1);
hold on;
plot(cputhreads,cputime(:,9),'b-.o','LineWidth',1);
if errorplot
   % get stddev, create box plots
   cpustddevs = std(cputimereps,0,2);
   gpustddevs = std(gputimereps,0,2);   
   h = errorbar(cputhreads,cputime(:,9),cpustddevs(:,9),'b.');
   errorbar_tick(h,1,'units');   
   h = errorbar(gputhreads,gputime(:,9),gpustddevs(:,6),'r.');
   errorbar_tick(h,1,'units');   
end
h = legend('Hybrid','CPU only')
set(h,'FontSize',10);
h = xlabel('Threads'); set(h,'FontSize',10);
h = ylabel('Time per iteration (s)'); set(h,'FontSize',10);
%axis([1e3 5e6 0 20]);
set(gca,'XTick',[1 2 4 8])
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 250]);
grid on
print -depsc examples/figs/threads_runtime.eps

figure()
hold off
plot(gputhreads,cputime(:,9)./gputime(:,9),'k-o','LineWidth',2);
if errorplot
   hold on
   % get stddev, create box plots
   ratiostddevs = std(cputimereps(:,:,9)./cputimereps(:,:,9),0,2); 
   h = errorbar(cputhreads,cputime(:,9)./gputime(:,9),ratiostddevs,'r.');
   errorbar_tick(h,1,'units');   
end
%title('GPU/CPU performance ratio');
h = xlabel('Threads'); set(h,'FontSize',10);
h = ylabel('Performance Ratio'); set(h,'FontSize',10);
%axis([1e3 5e6 0 20]);
set(gca,'XTick',[1 2 4 8])
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 250]);
grid on
print -depsc examples/figs/threads_gpucpuratio.eps

figure()
hold off
plot(gputhreads,cputime(1,9)./gputime(:,9),'k-o','LineWidth',2);
hold on
plot(gputhreads,cputime(1,9)./cputime(:,9),'b--o','LineWidth',2);
if errorplot
   % get stddev, create box plots
   for i = 1:length(cputhreads)
     cpu_su_stddevs(i) = std(cputimereps(1,:,9)./cputimereps(i,:,9),0,2); 
     gpu_su_stddevs(i) = std(cputimereps(1,:,9)./gputimereps(i,:,9),0,2);      
   end
   h = errorbar(cputhreads,cputime(1,9)./gputime(:,9),gpu_su_stddevs(:),'k.');
   errorbar_tick(h,1,'units');      
   h = errorbar(cputhreads,cputime(1,9)./cputime(:,9),cpu_su_stddevs(:),'b.');   
   errorbar_tick(h,1,'units');   
end
h = legend('Hybrid','CPU only','Location','NW')
set(h,'FontSize',10);
%title('CPU Thread and GPU speedup');
h = xlabel('Threads'); set(h,'FontSize',10);
h = ylabel('Speedup'); set(h,'FontSize',10);
%axis([1e3 5e6 0 20]);
set(gca,'XTick',[1 2 4 8])
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 250]);
grid on
print -depsc examples/figs/threads_gpucpuspeedup.eps

clear cpuQ
clear gpuQ

% Total runtime on GPU can be seen as the function T = max(ML2L, P2P) + Q. Or so we think. 
% On CPU, we should have T = ML2L + P2P + Q. Let's see:
cpuQ = cputime(:,2) + cputime(:,3) + cputime(:,4) + cputime(:,5) + cputime(:,7);
gpuQ = gputime(:,2) + gputime(:,3) + gputime(:,4) + gputime(:,5) + gputime(:,7);

figure()
hold off
plot(gputhreads,cputime(:,9)./gputime(:,9),'k','LineWidth',2);
hold on
plot(gputhreads,cputime(:,6)./gputime(:,6),'r-o','LineWidth',2);
plot(gputhreads,cputime(:,8)./gputime(:,8),'g-o','LineWidth',2);
plot(gputhreads,cpuQ./gpuQ,'b-o','LineWidth',2);
legend('Total','ML2L','P2P','Q');
%title('GPU/CPU performance ratio')
xlabel('Threads')
ylabel('Ratio')
grid on
print -depsc examples/figs/threads_phases_gpucpuratio.eps



figure()
hold off
plot(gputhreads,cputime(1,9)./gputime(:,9),'k-o','LineWidth',4);
hold on;
plot(gputhreads,cputime(1,2)./gputime(:,2),'k-o','LineWidth',2);
plot(gputhreads,cputime(1,3)./gputime(:,3),'b:o','LineWidth',2);
plot(gputhreads,cputime(1,4)./gputime(:,4),'b-o','LineWidth',2);
plot(gputhreads,cputime(1,5)./gputime(:,5),'r:o','LineWidth',2);
plot(gputhreads,cputime(1,6)./gputime(:,6),'r-o','LineWidth',2);
plot(gputhreads,cputime(1,7)./gputime(:,7),'m-o','LineWidth',2);
plot(gputhreads,cputime(1,8)./gputime(:,8),'g-o','LineWidth',2);
legend('Total','Partition','Connectivity','P2M','M2M','ML2L','L2P','P2P','Linear');
%title('Speedup Hybrid vs. single-core CPU')
xlabel('Threads')
ylabel('Speedup')
grid on
print -depsc examples/figs/GPUthreads_speedup_detailed.eps
