function create_plots_varytheta(errorplot)

if nargin < 1
  errorplot = 1;
end

for i=1:4
if(i == 1)
  filename = 'cpu_varytheta_uniform.mat';
  dist = 'uniform';
  arch = 1;
  runstr = ' CPU-only, uniform distribution,';
  saveprefix = 'CPU';
elseif(i == 2)
  filename = 'gpu_varytheta_uniform.mat';
  dist = 'uniform';
  arch = 2;
  runstr = ' w/ GPU, uniform distribution,';  
  saveprefix = 'GPU';  
elseif(i == 3)
  filename = 'cpu_varytheta_linear.mat';
  dist = 'linear';
  arch = 1;
  runstr = ' CPU-only, linear distribution,'; 
  saveprefix = 'CPU';  
elseif(i == 4)
  filename = 'gpu_varytheta_linear.mat';
  dist = 'linear';
  arch = 2;
  runstr = ' w/ GPU, linear distribution,'; 
  saveprefix = 'GPU';  
end

load(strcat('examples/data/',filename));
 
nthets = (ones(numreps,1)*thets)';

clear Q

% Total runtime on GPU can be seen as the function T = max(ML2L, P2P) + Q. Or so we think. 
% On CPU, we should have T = ML2L + P2P + Q. Let's see:
Q(:,:) = time(:,:,2) + time(:,:,3) + time(:,:,4) + time(:,:,5) + time(:,:,7);

if arch == 1 % CPU
   clear T
   T(:,:) = time(:,:,6) + time(:,:,8) + Q(:,:);
else % GPU
   clear T
   T(:,:) = max(time(:,:,6),time(:,:,8)) + Q(:,:);
end

minruntime = min(time(:,:,9),[],2);
minQ = min(Q,[],2);
minML2L = min(time(:,:,6),[],2);
minP2P = min(time(:,:,8),[],2);

medianruntime = median(time(:,:,9),2);
medianQ = median(Q,2);
medianML2L = median(time(:,:,6),2);
medianP2P = median(time(:,:,8),2);

stdruntime = std(time(:,:,9),0,2);
stdQ = std(Q,0,2);
stdML2L = std(time(:,:,6),0,2);
stdP2P = std(time(:,:,8),0,2);

figure()
hold off
plot(thets,medianruntime,'k','LineWidth',2);
hold on
plot(thets,medianML2L,'r--','LineWidth',2);
plot(thets,medianP2P,'b-.','LineWidth',2);
plot(thets,medianQ,'k:','LineWidth',2);
%plot(thets(1:10:length(thets)),medianML2L(1:10:length(thets)),'rs','LineWidth',2);
%plot(thets(1:10:length(thets)),medianP2P(1:10:length(thets)),'bv','LineWidth',2);
%plot(thets(1:10:length(thets)),medianQ(1:10:length(thets)),'k^','LineWidth',2);
if errorplot
   h = errorbar(thets,medianruntime,stdruntime,'k.');
  % errorbar_tick(h,.01,'units');   
   h = errorbar(thets,medianML2L,stdML2L,'r.');
   %errorbar_tick(h,.01,'units');   
   h = errorbar(thets,medianP2P,stdP2P,'c.');
   %errorbar_tick(h,.01,'units');   
   h = errorbar(thets,medianQ,stdQ,'b.');
   %errorbar_tick(h,.01,'units');   
end
xlim([0.25 0.8])

h = legend('Runtime','ML2L','P2P','Q','Location','EO');
set(h,'FontSize',10);
% title(strcat('Runtime ',runstr,' Nlevels=',int2str(nlevels)))
h = xlabel('\theta'); set(h,'FontSize',10);
h = ylabel('Time (s)'); set(h,'FontSize',10);
set(gcf,'PaperPositionMode','auto');
set(gcf,'Position',[100 100 400 250]);
grid on
print('-depsc', strcat('examples/figs/',saveprefix,'varytheta_',dist,'_runtime2.eps'));

if 0
figure()
hold off
plot(nthets,time(:,:,9),'ko','LineWidth',4);
hold on;
plot(nthets,time(:,:,2),'ko','LineWidth',2);
plot(nthets,time(:,:,3),'b.','LineWidth',2);
plot(nthets,time(:,:,4),'bo','LineWidth',2);
plot(nthets,time(:,:,5),'r.','LineWidth',2);
plot(nthets,time(:,:,6),'ro','LineWidth',2);
plot(nthets,time(:,:,7),'mo','LineWidth',2);
plot(nthets,time(:,:,8),'go','LineWidth',2);
legend('Total','Partition','Connectivity','P2M','M2M','ML2L','L2P','P2P');
title(strcat('Runtime ',runstr,' Nlevels=',int2str(nlevels)))
xlabel('theta')
ylabel('Time (s)')
grid on
print('-depsc', strcat('examples/figs/',saveprefix,'varytheta_',dist,'_runtime.eps'));
end

if 0
figure()
hold off;
plot(thets,time(:,:,9)./time(:,:,9),'ko','LineWidth',4);
hold on;
plot(nthets,time(:,:,2)./time(:,:,9),'ko','LineWidth',2);
plot(nthets,time(:,:,3)./time(:,:,9),'b.','LineWidth',2);
plot(nthets,time(:,:,4)./time(:,:,9),'bo','LineWidth',2);
plot(nthets,time(:,:,5)./time(:,:,9),'r.','LineWidth',2);
plot(nthets,time(:,:,6)./time(:,:,9),'ro','LineWidth',2);
plot(nthets,time(:,:,7)./time(:,:,9),'mo','LineWidth',2);
plot(nthets,time(:,:,8)./time(:,:,9),'go','LineWidth',2);
legend('Total','Partition','Connectivity','P2M','M2M','ML2L','L2P','P2P');
title(strcat('Proportion ',runstr,' Nlevels=',int2str(nlevels)))
xlabel('theta')
ylabel('Proportion of runtime')
grid on
print('-depsc', strcat('examples/figs/',saveprefix,'varytheta_',dist,'_proportion.eps'));
end

end
