% run_cpugpulinear.m -- cpu/hybrid comparison on linear particle distribution

N = 1e6 
z = complex(rand(N,1),rand(N,1)*0.01+0.495); 
m = rand(N,1);

tuningsteps = 200;
startnlevels = 8;
starttheta = 0.5;

for arch = 1:2
clear mex;
if(arch == 1)
  fmm = @fmm2d_8threads;
else
  fmm = @fmm2d_8threads_cpu;
end

nlevels = startnlevels;
theta = starttheta;

at = initat(nlevels, theta);

atfunc = @at3b;

nlevs = 0;
thets = 0;
time = 0;

for k = 1:tuningsteps
  k
  nlevs(k) = nlevels;
  thets(k) = theta;  

  clockstart=clock; 
  [p l] = fmm(z,m,'tol',1e-7,'nlevels',nlevels,'theta',theta,'out','times'); 
  clockelapsed=clock-clockstart;
  runtime=3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6);
  
  time(k) = runtime; 
  [at nlevels theta] = atfunc(at, runtime, k, l(1));
end

[mintime mindex] = min(time);

nlevels = nlevs(mindex);
theta = thets(mindex);
['time=',num2str(mintime),', theta=',num2str(theta),', nlevels=',int2str(nlevels)]

clear mex;
[p l] = fmm_log(z,m,'tol',1e-7,'nlevels',nlevels,'theta',theta,'out','times'); 
clear mex;
if(arch == 1)
  unix(strcat('mv *.log examples/data/cpulinear.vis'));
  save('examples/data/cpulinear.mat')
else
  unix(strcat('mv *.log examples/data/gpulinear.vis'));  
  save('examples/data/gpulinear.mat')
end

end
