% run_cpugpuuniform.m -- cpu/hybrid comparison on square particle distribution

N = 1e6
z = complex(rand(N,1),rand(N,1)); 
m = rand(N,1);

tuningsteps = 200;
startnlevels = 8;
starttheta = 0.5;

for arch = 1:2
clear mex;
if(arch == 1)
  'CPU, 8 threads'
  ! make clean  
  make8threads_log cpu
  ! cp fmm2d.mexa64 fmm2d_log.mexa64
  ! make clean  
  make8threads cpu
else
  'GPU, 8 threads'
  ! make clean  
  make8threads_log gpu
  ! cp fmm2d.mexa64 fmm2d_log.mexa64
  ! make clean  
  make8threads gpu
end

nlevels = startnlevels;
theta = starttheta;

at = initat(nlevels, theta);

atfunc = @at3b;

nlevs = 0;
thets = 0;
time = 0;

for k = 1:tuningsteps
  k
  nlevs(k) = nlevels;
  thets(k) = theta;  

  clockstart=clock; 
  [p l] = fmm(z,m,'tol',1e-7,'nlevels',nlevels,'theta',theta,'out','times'); 
  clockelapsed=clock-clockstart;
  runtime=3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6);
  
  [at nlevels theta] = atfunc(at, runtime, k, l(1));

  time(k) = runtime; 
end % done tuning

[mintime mindex] = min(time);

nlevels = nlevs(mindex);
theta = thets(mindex);
['time=',num2str(mintime),', theta=',num2str(theta),', nlevels=',int2str(nlevels)]

clear mex;
[p l] = fmm_log(z,m,'tol',1e-7,'nlevels',nlevels,'theta',theta,'out','times'); 
clear mex;
if(arch == 1)
  unix(strcat('mv *.log examples/data/cpuuniform.vis'));
  save('examples/data/cpuuniform.mat');
else
  unix(strcat('mv *.log examples/data/gpuuniform.vis'));  
  save('examples/data/gpuuniform.mat');  
end

end % end arch
