% run_gputhreads.m -- strong scaling of hybrid code

clear all;
'GPU Threads experiment'

N = 1e6 
z = complex(rand(N,1),rand(N,1)*0.01+0.495); 
m = rand(N,1);

tuningsteps = 1;
startnlevels = 7;
starttheta = 0.5;

arch = 2; % GPU

reps = 5;
gputimereps = zeros(4,reps,9);

for run = [1:4]

clear mex;

if(run == 1)
  fmm = @fmm2d_1threads;  
  numthreads = 1;
  '1 thread'
elseif(run == 2)
  fmm = @fmm2d_2threads;  
  fmm_log = @fmm2d_log_2threads;  
  numthreads = 2;
  '2 threads'
elseif(run == 3)
  fmm = @fmm2d_4threads;  
  fmm_log = @fmm2d_log_4threads;  
  numthreads = 4;
  '4 threads'
elseif(run == 4)
  fmm = @fmm2d_8threads;    
  fmm_log = @fmm2d_log_8threads;  
  numthreads = 8;
  '8 threads'
end


gputhreads(run) = numthreads;

nlevels = startnlevels;
theta = starttheta;

atfunc = @at3b;

at = initat(nlevels, theta);

nlevs = 0;
thets = 0;

%  loadbalance = time(1);  
%  partition = time(2);
%  connectivity = time(3);
%  P2M = time(4);
%  M2M = time(5);
%  ML2L = time(6);
%  L2P = time(7);
%  P2P = time(8);
%  runtime = time(9); 
time = zeros(tuningsteps,9);


for k = 1:tuningsteps
  nlevs(k) = nlevels;
  thets(k) = theta; 

  clockstart=clock; 
  [p l] = fmm(z,m,'tol',1e-7,'pot',1,'nlevels',nlevels,'theta',theta,'out','times'); 
  clockelapsed=clock-clockstart;
  runtime=3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6)
  
  [at nlevels theta] = atfunc(at, runtime, k, l(1));

  l(9) = runtime;
  time(k,:) = l;
  
end

[mintime mindex] = min(time(:,9))

nlevels = nlevs(mindex);
theta = thets(mindex);
['time=',num2str(mintime),', theta=',num2str(theta),', nlevels=',int2str(nlevels)]


for k = 1:reps
  clockstart=clock;
  [p l] = fmm(z,m,'tol',1e-7,'pot',1,'nlevels',nlevels,'theta',theta,'out','times');
  clockelapsed=clock-clockstart;
  runtime=3600*clockelapsed(4)+60*clockelapsed(5)+clockelapsed(6)
  l(9) = runtime;
  gputimereps(run,k,:) = l;
end


%gputime(run,:) = time(mindex,:);
gpunlevels(run) = nlevels;
gputheta(run) = theta;

%gpuruntime(run) = mintime; 
  
%gpupart(run) = partition(mindex);
%gpuconn(run) = connectivity(mindex);
%gpuP2M(run) = P2M(mindex);
%gpuM2M(run) = M2M(mindex);
%gpuML2L(run) = ML2L(mindex);
%gpuL2P(run) = L2P(mindex);
%gpuP2P(run) = P2P(mindex);
%gputhreads(run) = numthreads;
%gpunlevels(run) = nlevels;
%gputheta(run) = theta;

if numthreads > 1
  clear mex
  unix('rm *.log');
  [p l] = fmm_log(z,m,'tol',1e-7,'pot',1,'nlevels',nlevels,'theta',theta,'out','times'); 
  if(arch == 1)
    unix(strcat('mv *.log examples/data/cpu',int2str(numthreads),'threadsuniform.vis'));
  else
    unix(strcat('mv *.log examples/data/gpu',int2str(numthreads),'threadsuniform.vis'));  
  end
  clear mex
end

end

'DONE.'

save('examples/data/gputhreads.mat');



