/* cudaevalkernels.h is included into cudaeval.h */

#ifndef __cudaevalkernels_h
#define __cudaevalkernels_h

#include "cuda.h"
#include "expint.h"

//kernels for direct evaluation. These will be used several times, therefore use define to avoid repeating code
#define DIRACSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if(sqrdist!=0.0) {\
  sqrdist=circ*(1.0/(sqrdist));\
  xvelocity+=sqrdist*xdist;\
  yvelocity-=sqrdist*ydist;\
}
#define IDIRACSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if(sqrdist!=0.0) {\
  sqrdist=1/(sqrdist);\
  xvelocity+=sqrdist*(circ*xdist+icirc*ydist);\
  yvelocity-=sqrdist*(circ*ydist-icirc*xdist);\
}
#define RANKINESYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=1/(xdist*xdist+ydist*ydist);\
if (sqrdist<shape){/*since both contains the 1/X, the comparison has to change sign*/\
  xvelocity+=circ*sqrdist*xdist;\
  yvelocity-=circ*sqrdist*ydist;\
}\
else if(sqrdist!=0){\
  xvelocity+=circ*shape*xdist;\
  yvelocity-=circ*shape*ydist;\
}
#define IRANKINESYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=1/(xdist*xdist+ydist*ydist);\
if (sqrdist<shape){/*since both contains the 1/X, the comparison has to change sign*/\
  xvelocity+=sqrdist*(circ*xdist+icirc*ydist);\
  yvelocity-=sqrdist*(circ*ydist-icirc*xdist);\
}\
else if(sqrdist!=0){\
  xvelocity+=shape*(circ*xdist+icirc*ydist);\
  yvelocity-=shape*(circ*ydist-icirc*xdist);\
}
#define SCULLYSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist>cutoffsqr) {/*since both contains the 1/X, the comparison has to change sign**/\
  sqrdist=circ/sqrdist;\
  xvelocity+=sqrdist*xdist;\
  yvelocity-=sqrdist*ydist;\
}\
else if(sqrdist!=0) {\
  sqrdist=circ/(sqrdist+shape)*scale;\
  xvelocity+=sqrdist*xdist;\
  yvelocity-=sqrdist*ydist;\
}
#define ISCULLYSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist>cutoffsqr) {/*since both contains the 1/X, the comparison has to change sign**/\
  sqrdist=1/sqrdist;\
  xvelocity+=sqrdist*(circ*xdist+icirc*ydist);\
  yvelocity-=sqrdist*(circ*ydist-icirc*xdist);\
}\
else if(sqrdist!=0) {\
  sqrdist=1/(sqrdist+shape)*scale;\
  xvelocity+=sqrdist*(circ*xdist+icirc*ydist);\
  yvelocity-=sqrdist*(circ*ydist-icirc*xdist);\
}
#define OSEENSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist>cutoffsqr) {/*since both contains the 1/X, the comparison has to change sign*/\
  sqrdist=circ/sqrdist;\
  xvelocity+=sqrdist*xdist;\
  yvelocity-=sqrdist*ydist;\
}\
else if(sqrdist!=0) {  /*can still fail if cutoffsqr==0*/\
  sqrdist=-circ/sqrdist*scale*expm1(-shape*sqrdist);\
  xvelocity+=sqrdist*xdist;\
  yvelocity-=sqrdist*ydist;\
}//since two version should exist, one that takes data from evaluation points and one from potential points, make a prototype
#define IOSEENSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist>cutoffsqr) {/*since both contains the 1/X, the comparison has to change sign*/\
  sqrdist=1/sqrdist;\
  xvelocity+=sqrdist*(circ*xdist+icirc*ydist);\
  yvelocity-=sqrdist*(circ*ydist-icirc*xdist);\
}\
else if(sqrdist!=0) {  /*can still fail if cutoffsqr==0*/\
  sqrdist=-1/sqrdist*scale*expm1(-shape*sqrdist);\
  xvelocity+=sqrdist*(circ*xdist+icirc*ydist);\
  yvelocity-=sqrdist*(circ*ydist-icirc*xdist);\
}
#define DIRACLOGSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if(sqrdist!=0.0) {\
  xvelocity-=0.5*circ*log(sqrdist);\
}
#define IDIRACLOGSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if(sqrdist!=0.0) {\
  sqrdist=0.5*log(sqrdist);\
  xvelocity-=circ*sqrdist;\
  yvelocity-=icirc*sqrdist;\
}
#define RANKINELOGSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist >= cutoffsqr)\
  sqrdist = 0.5*log(sqrdist);\
else\
  sqrdist = 0.5*(sqrdist/cutoffsqr)+shape;\
xvelocity-=circ*sqrdist;
#define IRANKINELOGSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist >= cutoffsqr)\
  sqrdist = 0.5*log(sqrdist);\
else\
  sqrdist = 0.5*(sqrdist/cutoffsqr)+shape;\
xvelocity-=circ*sqrdist;\
yvelocity-=icirc*sqrdist;
#define SCULLYLOGSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist >= cutoffsqr)\
  sqrdist = 0.5*log(sqrdist);\
else\
  sqrdist = 0.5*log(shape+sqrdist)*scale;\
xvelocity-=circ*sqrdist;
#define ISCULLYLOGSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist >= cutoffsqr)\
  sqrdist = 0.5*log(sqrdist);\
else\
  sqrdist = 0.5*log(shape+sqrdist)*scale;\
xvelocity-=circ*sqrdist;\
yvelocity-=icirc*sqrdist;
#define OSEENLOGSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist >= cutoffsqr)\
  sqrdist = 0.5*log(sqrdist);\
else if(sqrdist!=0.0)\
  sqrdist = (0.5*log(sqrdist)+0.5*expint(shape*sqrdist))*scale;\
else\
  sqrdist = -0.5*(log(shape)+gamma_)*scale;\
xvelocity-=circ*sqrdist;
#define IOSEENLOGSYNCKERNEL(xvelocity,yvelocity,circ,icirc) sqrdist=xdist*xdist+ydist*ydist;\
if (sqrdist >= cutoffsqr)\
  sqrdist = 0.5*log(sqrdist);\
else if(sqrdist!=0.0)\
  sqrdist = (0.5*log(sqrdist)+0.5*expint(shape*sqrdist))*scale;\
else\
  sqrdist = -0.5*(log(shape)+gamma_)*scale;\
xvelocity-=circ*sqrdist;\
yvelocity-=icirc*sqrdist;

//depending on complex mass, complex answer etc, some changes has to be made to the kernel code. These strings are the differences
#define DUMMYKERNELSTRING
#define IKERNELSTRING1 __shared__ double igammav[MAXTHREADSEVALUATION];
#define IKERNELSTRING1DIRECT __shared__ double igammav[DIRECTMAXTHREADS];
#define IKERNELSTRING2 TEXTUREFETCH(igammav[threadIdx.x],tmi,k+threadIdx.x);
#define COMPLEXANSWER1 double yvelocity;
#define COMPLEXANSWER2 yvelocity=0;
#define COMPLEXANSWER3 yvelocities[i]+=yvelocity;
#define COMPLEXANSWER3DIRECT yvelocities[i]=yvelocity;

#define IKERNELSTRING2_2 TEXTUREFETCH(igammav[threadIdx.x+addstart],tmi,k+threadIdx.x);
#define COMPLEXANSWER1_2 __shared__ double yvelocity[MAXTHREADSEVALUATION],yvelocitytmp[MAXTHREADSEVALUATION];
#define COMPLEXANSWER2_2 yvelocity[threadIdx.x]=0;
#define COMPLEXANSWER3_2 yvelocities[i]+=yvelocity[threadIdx.x];
#define COMPLEXANSWER4_2 yvelocitytmp[threadIdx.x]=0;
#define COMPLEXANSWER5_2 yvelocity[threadIdx.x]+=yvelocitytmp[threadIdx.x<<1]+yvelocitytmp[(threadIdx.x<<1)+1];
#define COMPLEXANSWER6_2 yvelocity[threadIdx.x+ilocal]+=yvelocitytmp[threadIdx.x<<2]+yvelocitytmp[(threadIdx.x<<2)+1]+yvelocitytmp[(threadIdx.x<<2)+2]+yvelocitytmp[(threadIdx.x<<2)+3];
//kernel code prototype
#define SYNCHRONIZEDKERNEL(kernel,tevalr,tevali,txptr,ISTRING1,ISTRING2,CANSWER1,CANSWER2,CANSWER3,CANSWER4,CANSWER5,CANSWER6) int i,j,k,boxnr,m;\
  double sqrdist,xdist,ydist,xvelocity;\
  CANSWER1\
  __shared__ double vxpositions[MAXTHREADSEVALUATION];\
  __shared__ double vypositions[MAXTHREADSEVALUATION];\
  __shared__ double gammav[MAXTHREADSEVALUATION];\
  ISTRING1\
  __shared__ double xpositions[MAXTHREADSEVALUATION];/*makes code approx 1% slower for heavy load, but for some reason, doubles has to be fetched into an array*/\
  __shared__ double ypositions[MAXTHREADSEVALUATION];\
  __shared__ int count,loopmax,vcount,begin,end,loopstart,mstart,mend;\
  for(boxnr=blockIdx.x;boxnr<Nf;boxnr+=gridDim.x) {/*first loop is over the boxes, one block take one box*/\
    if(threadIdx.x==0) {\
      count=tex1Dfetch(/*tevalend,m*/txptr,boxnr+1); /*right now, it is possible that twi boxes has same evalbegin if one box has 0 elements*/\
      loopstart=tex1Dfetch(/*tevalbegin*/txptr,boxnr);\
      loopmax=count-(count+blockDim.x-1-(loopstart%blockDim.x))%blockDim.x-1+blockDim.x;/*This one makes sure all threads loop the same number of times, necessary for __syncthreads()*/\
      mstart=jcptr[boxnr];/*tex1Dfetch(tboxstart, boxnr)*/\
      mend=j2cptr[boxnr];/*tex1Dfetch(tboxstart, boxnr+1)*/\
    }\
    __syncthreads();\
    for(i=threadIdx.x+loopstart;i<loopmax;i+=blockDim.x) { /*if more points than threads*/\
      if(i<count) {/*Always check if the point belongs to the box*/\
        xvelocity=0; /*use local variables as much as possible to help the optimizer understand that they are constant*/\
        /*yvelocity=0;*/CANSWER2\
        TEXTUREFETCH(xpositions[threadIdx.x],tevalr,i);\
        TEXTUREFETCH(ypositions[threadIdx.x],tevali,i);\
      }\
      for(m=mstart;m<mend;m++) {/*Loop over all other boxes*/\
        if(threadIdx.x==0) {/*Set up the loop limits*/\
          begin=tex1Dfetch(tixptr,/*tex1Dfetch(tpotbegin, m)*/ir[m]);\
          end=tex1Dfetch(tixptr,/*tex1Dfetch(tpotbegin, m)*/ir[m]+1);\
          vcount=(end-begin)%blockDim.x;\
        }\
        __syncthreads();\
        for(k=begin;k+blockDim.x<=end;k+=blockDim.x) {/*Loop over the elements in the external box, this step is for when there are more than blockDim.x elements left*/\
          TEXTUREFETCH(vxpositions[threadIdx.x],tzr,k+threadIdx.x);\
          TEXTUREFETCH(vypositions[threadIdx.x],tzi,k+threadIdx.x);\
          TEXTUREFETCH(gammav[threadIdx.x],tmr,k+threadIdx.x);\
          ISTRING2\
          __syncthreads();\
          if(i<count) {\
            for(j=0;j<blockDim.x;j++) {/*the interaction loop*/\
              xdist=vxpositions[j]-xpositions[threadIdx.x];\
              ydist=vypositions[j]-ypositions[threadIdx.x];\
              kernel(xvelocity,yvelocity,circ,icirc);\
            }\
          }\
          __syncthreads();\
        }\
        if(k+threadIdx.x<end) {/*The remaining points*/\
          TEXTUREFETCH(vxpositions[threadIdx.x],tzr,k+threadIdx.x);\
          TEXTUREFETCH(vypositions[threadIdx.x],tzi,k+threadIdx.x);\
          TEXTUREFETCH(gammav[threadIdx.x],tmr,k+threadIdx.x);\
          ISTRING2\
        }\
        __syncthreads();\
        if(i<count) {\
          for(j=0;j<vcount;j++) {/*the interaction loop*/\
            xdist=vxpositions[j]-xpositions[threadIdx.x];\
            ydist=vypositions[j]-ypositions[threadIdx.x];\
            kernel(xvelocity,yvelocity,circ,icirc);\
          }\
        }\
        __syncthreads();\
      }\
      if(i<count) {/*write results*/\
        xvelocities[i]+=xvelocity;\
        /*yvelocities[i]+=yvelocity*/CANSWER3\
      }\
    }\
    __syncthreads();\
  }

//#define BOXLOOPSIZE 32 //cache this many boxes indices. Too high number results in more memory use. Do not use higher number than number of threads
#define SOURCEMULTIPLIER 1 //most likely more source points than evaluation points, cache more source points to increase efficiency by reducing loop overhead

/*This code caches a full line of points before any kernel evaluation,
 *and if number of evaluation points is smaller than number of threads
 *the code will use more threads for each point. At the moment, a maximum of
 *4 threads for each point is used, since it was observed that the evaluation
 *of high values of ndirect slightly increased when increasing the number of splits*/
#define SYNCHRONIZEDKERNEL2(kernel,tevalr,tevali,txptr,ISTRING1,ISTRING2,CANSWER1,CANSWER2,CANSWER3,CANSWER4,CANSWER5,CANSWER6) int i,j,k,boxnr,m,addstart,ilocal;\
  double sqrdist,xdist,ydist;\
  __shared__ double xvelocity[MAXTHREADSEVALUATION];\
  CANSWER1\
  __shared__ double xvelocitytmp[MAXTHREADSEVALUATION];\
  __shared__ double vxpositions[MAXTHREADSEVALUATION*SOURCEMULTIPLIER];\
  __shared__ double vypositions[MAXTHREADSEVALUATION*SOURCEMULTIPLIER];\
  __shared__ double gammav[MAXTHREADSEVALUATION*SOURCEMULTIPLIER];\
  ISTRING1\
  __shared__ double xpositions[MAXTHREADSEVALUATION];/*makes code approx 1% slower for heavy load, but for some reason, doubles has to be fetched into an array*/\
  __shared__ double ypositions[MAXTHREADSEVALUATION];\
  __shared__ int count,loopmax,begin,end,loopstart,mstart,mend;\
  for(boxnr=blockIdx.x;boxnr<Nf;boxnr+=gridDim.x) {\
    if(threadIdx.x==0) {\
      count=tex1Dfetch(txptr,boxnr+1); /*right now, it is possible that two boxes has same evalbegin if one box has 0 elements*/\
      loopstart=tex1Dfetch(txptr,boxnr);\
      loopmax=count-(count+blockDim.x-1-(loopstart%blockDim.x))%blockDim.x-1+blockDim.x;\
      mstart=jcptr[boxnr];\
      mend=j2cptr[boxnr];\
    }\
    __syncthreads();\
    for(i=threadIdx.x+loopstart;i<loopmax;i+=blockDim.x) { /*loop for each point. if more points than threads*/\
      if(i<count) {\
        xvelocity[threadIdx.x]=0; /*use local variables as much as possible to help the optimizer understand that they are constant*/\
        CANSWER2\
        TEXTUREFETCH(xpositions[threadIdx.x],tevalr,i);\
        TEXTUREFETCH(ypositions[threadIdx.x],tevali,i);\
      }\
      addstart=0;\
/*xvelocity[threadIdx.x] = (mend-mstart)*1000000;*/\
      for(m=mstart;m<mend;m++) {/*Loop through all interaction boxes*/\
        __syncthreads();\
        if(threadIdx.x==0) {/*cache start and end position*/\
/*printf("%d, %d reading %p, m=%d\n",threadIdx.x,blockIdx.x,&ir[m],m);*/\
          k=ir[m];\
          begin=tex1Dfetch(tixptr,k);\
          end=tex1Dfetch(tixptr,k+1);\
        }\
        __syncthreads();\
        \
        for(k=begin;k<=end/*&&addstart<MAXTHREADSEVALUATION*SOURCEMULTIPLIER*/;k+=blockDim.x) {/*Start and fill the array. End condition is when box is done. Not that code has to enter the loop even if no points, because it may be the last loop, and summation is inside loop*/\
          if(end-k>=MAXTHREADSEVALUATION*SOURCEMULTIPLIER-addstart&&MAXTHREADSEVALUATION*SOURCEMULTIPLIER-addstart<=blockDim.x) {/*full array*/\
            if(addstart+threadIdx.x<MAXTHREADSEVALUATION*SOURCEMULTIPLIER) {/*If it will fill the whole array, fill it and perform the interaction*/\
              TEXTUREFETCH(vxpositions[addstart+threadIdx.x],tzr,k+threadIdx.x);\
              TEXTUREFETCH(vypositions[addstart+threadIdx.x],tzi,k+threadIdx.x);\
              TEXTUREFETCH(gammav[addstart+threadIdx.x],tmr,k+threadIdx.x);\
              ISTRING2\
            }\
            __syncthreads();\
            ilocal=0;/*remembers how many interactions that has been doen*/\
            if(count-(i-threadIdx.x)>=blockDim.x) {/*if more calculation points than threads, one thread per evaluation point*/\
              for(j=0;j<MAXTHREADSEVALUATION*SOURCEMULTIPLIER;j++) {\
                xdist=vxpositions[j]-xpositions[threadIdx.x];\
                ydist=vypositions[j]-ypositions[threadIdx.x];\
                kernel(xvelocity[threadIdx.x],yvelocity[threadIdx.x],gammav[j],igammav[j]);\
              }\
            }\
            else { /*else, use several threads for each point to get better balance*/\
              if(count-(i-threadIdx.x)>=(blockDim.x>>1)){ /*more than half of the points active?*/\
                xvelocitytmp[threadIdx.x]=0;\
                CANSWER4/*yvelocitytmp[threadIdx.x]=0*/;\
                for(j=0;j<MAXTHREADSEVALUATION*SOURCEMULTIPLIER;j+=2) {/*calculate temporary points*/\
                  xdist=vxpositions[j+(threadIdx.x&1)]-xpositions[(threadIdx.x>>1)];\
                  ydist=vypositions[j+(threadIdx.x&1)]-ypositions[(threadIdx.x>>1)];\
                  kernel(xvelocitytmp[threadIdx.x],yvelocitytmp[threadIdx.x],gammav[j+(threadIdx.x&1)],igammav[j+(threadIdx.x&1)]);\
                }\
                __syncthreads();\
                if(threadIdx.x<(blockDim.x>>1)) {/*sum, note that ilocal only can be zero here*/\
                  xvelocity[threadIdx.x]+=xvelocitytmp[threadIdx.x<<1]+xvelocitytmp[(threadIdx.x<<1)+1];\
                  CANSWER5/*yvelocity[threadIdx.x]+=yvelocitytmp[threadIdx.x<<1]+yvelocitytmp[(threadIdx.x<<1)+1];*/\
                }\
                __syncthreads();\
                ilocal+=(blockDim.x>>1);\
              }\
              if(count-(i-threadIdx.x)-ilocal>=(blockDim.x>>2)){/*if remaining points are more than 1/4 of the number of threads*/\
                xvelocitytmp[threadIdx.x]=0;\
                CANSWER4/*yvelocitytmp[threadIdx.x]=0*/;\
                for(j=0;j<MAXTHREADSEVALUATION*SOURCEMULTIPLIER;j+=4) {/*calculate temporary points*/\
                  xdist=vxpositions[j+(threadIdx.x&3)]-xpositions[(threadIdx.x>>2)+ilocal];\
                  ydist=vypositions[j+(threadIdx.x&3)]-ypositions[(threadIdx.x>>2)+ilocal];\
                  kernel(xvelocitytmp[threadIdx.x],yvelocitytmp[threadIdx.x],gammav[j+(threadIdx.x&3)],igammav[j+(threadIdx.x&3)]);\
                }\
                __syncthreads();\
                if(threadIdx.x<(blockDim.x>>2)) {/*sum*/\
                  xvelocity[threadIdx.x+ilocal]+=xvelocitytmp[threadIdx.x<<2]+xvelocitytmp[(threadIdx.x<<2)+1]+xvelocitytmp[(threadIdx.x<<2)+2]+xvelocitytmp[(threadIdx.x<<2)+3];\
                  CANSWER6/*yvelocity[threadIdx.x+ilocal]+=yvelocitytmp[threadIdx.x<<2]+yvelocitytmp[(threadIdx.x<<2)+1]+yvelocitytmp[(threadIdx.x<<2)+2]+yvelocitytmp[(threadIdx.x<<2)+3];*/\
                }\
                __syncthreads();\
                ilocal+=(blockDim.x>>2);\
              }\
              if((threadIdx.x>>2)+ilocal+(i-threadIdx.x)<count) {/*take care of the remaining points. More additional steps could be performed, but for high ndirect, adding more steps seems to decrease performance slightly*/\
                xvelocitytmp[threadIdx.x]=0;\
                CANSWER4/*yvelocitytmp[threadIdx.x]=0*/;\
                for(j=0;j<MAXTHREADSEVALUATION*SOURCEMULTIPLIER;j+=4) {\
                  xdist=vxpositions[j+(threadIdx.x&3)]-xpositions[(threadIdx.x>>2)+ilocal];\
                  ydist=vypositions[j+(threadIdx.x&3)]-ypositions[(threadIdx.x>>2)+ilocal];\
                  kernel(xvelocitytmp[threadIdx.x],yvelocitytmp[threadIdx.x],gammav[j+(threadIdx.x&3)],igammav[j+(threadIdx.x&3)]);\
                }\
              }\
              __syncthreads();\
              if(threadIdx.x+ilocal+(i-threadIdx.x)<count) {/*sum*/\
                xvelocity[threadIdx.x+ilocal]+=xvelocitytmp[threadIdx.x<<2]+xvelocitytmp[(threadIdx.x<<2)+1]+xvelocitytmp[(threadIdx.x<<2)+2]+xvelocitytmp[(threadIdx.x<<2)+3];\
                CANSWER6/*yvelocity[threadIdx.x+ilocal]+=yvelocitytmp[threadIdx.x<<2]+yvelocitytmp[(threadIdx.x<<2)+1]+yvelocitytmp[(threadIdx.x<<2)+2]+yvelocitytmp[(threadIdx.x<<2)+3];*/\
              }\
            }\
            __syncthreads();\
            k-=blockDim.x-(MAXTHREADSEVALUATION*SOURCEMULTIPLIER-addstart);/*All blockDim.x points was not read, correct the value for the next loop*/\
            addstart=0;\
            __syncthreads();\
            continue;\
          }\
          else if(end-k<=blockDim.x&&/*tex1Dfetch(tboxstart, boxnr+1)*/mend==m+1) {/*time to compute array due to no more available interaction points. Here, the list will not be full*/\
            if(k+threadIdx.x<end) {\
              TEXTUREFETCH(vxpositions[addstart+threadIdx.x],tzr,k+threadIdx.x);\
              TEXTUREFETCH(vypositions[addstart+threadIdx.x],tzi,k+threadIdx.x);\
              TEXTUREFETCH(gammav[addstart+threadIdx.x],tmr,k+threadIdx.x);\
              ISTRING2 /*Fix this one*/\
            }\
            addstart+=end-k;\
            __syncthreads();/*The remaining part is the same as above, but with the loops only running to addstart*/\
            if(count-(i-threadIdx.x)>=blockDim.x) {/*if more calculation points than threads*/\
              for(j=0;j<addstart;j++) {\
                xdist=vxpositions[j]-xpositions[threadIdx.x];\
                ydist=vypositions[j]-ypositions[threadIdx.x];\
                kernel(xvelocity[threadIdx.x],yvelocity[threadIdx.x],gammav[j],igammav[j]);\
              }\
            }\
            else { /*else, use several threads for each point to get better balance*/\
              ilocal=0;\
              if(count-(i-threadIdx.x)>=(blockDim.x>>1)){ /*more than half of the points active?*/\
                xvelocitytmp[threadIdx.x]=0;\
                CANSWER4/*yvelocitytmp[threadIdx.x]=0*/;\
                for(j=0;j+(threadIdx.x&1)<addstart;j+=2) {/*calculate temporary points*/\
                  xdist=vxpositions[j+(threadIdx.x&1)]-xpositions[(threadIdx.x>>1)];\
                  ydist=vypositions[j+(threadIdx.x&1)]-ypositions[(threadIdx.x>>1)];\
                  kernel(xvelocitytmp[threadIdx.x],yvelocitytmp[threadIdx.x],gammav[j+(threadIdx.x&1)],igammav[j+(threadIdx.x&1)]);\
                }\
                __syncthreads();\
                if(threadIdx.x<(blockDim.x>>1)) {/*sum*/\
                  xvelocity[threadIdx.x]+=xvelocitytmp[threadIdx.x<<1]+xvelocitytmp[(threadIdx.x<<1)+1];\
                  CANSWER5/*yvelocity[threadIdx.x]+=yvelocitytmp[threadIdx.x<<1]+yvelocitytmp[(threadIdx.x<<1)+1];*/\
                }\
                __syncthreads();\
                ilocal+=(blockDim.x>>1);\
              }\
              if(count-(i-threadIdx.x)-ilocal>=(blockDim.x>>2)){/*if remaining points are more than 1/4 of the number of threads*/\
                xvelocitytmp[threadIdx.x]=0;\
                CANSWER4/*yvelocitytmp[threadIdx.x]=0*/;\
                for(j=0;j+(threadIdx.x&3)<addstart;j+=4) {\
                  xdist=vxpositions[j+(threadIdx.x&3)]-xpositions[(threadIdx.x>>2)+ilocal];\
                  ydist=vypositions[j+(threadIdx.x&3)]-ypositions[(threadIdx.x>>2)+ilocal];\
                  kernel(xvelocitytmp[threadIdx.x],yvelocitytmp[threadIdx.x],gammav[j+(threadIdx.x&3)],igammav[j+(threadIdx.x&3)]);\
                }\
                __syncthreads();\
                if(threadIdx.x<(blockDim.x>>2)) {/*sum*/\
                  xvelocity[threadIdx.x+ilocal]+=xvelocitytmp[threadIdx.x<<2]+xvelocitytmp[(threadIdx.x<<2)+1]+xvelocitytmp[(threadIdx.x<<2)+2]+xvelocitytmp[(threadIdx.x<<2)+3];\
                  CANSWER6/*yvelocity[threadIdx.x+ilocal]+=yvelocitytmp[threadIdx.x<<2]+yvelocitytmp[(threadIdx.x<<2)+1]+yvelocitytmp[(threadIdx.x<<2)+2]+yvelocitytmp[(threadIdx.x<<2)+3];*/\
                }\
                __syncthreads();\
                ilocal+=(blockDim.x>>2);\
              }\
              if((threadIdx.x>>2)+ilocal+(i-threadIdx.x)<count) {/*take care of the remaining points. More additional steps could be performed, but for high ndirect, adding more steps seems to decrease performance slightly*/\
                xvelocitytmp[threadIdx.x]=0;\
                CANSWER4/*yvelocitytmp[threadIdx.x]=0*/;\
                for(j=0;j+(threadIdx.x&3)<addstart;j+=4) {\
                  xdist=vxpositions[j+(threadIdx.x&3)]-xpositions[(threadIdx.x>>2)+ilocal];\
                  ydist=vypositions[j+(threadIdx.x&3)]-ypositions[(threadIdx.x>>2)+ilocal];\
                  kernel(xvelocitytmp[threadIdx.x],yvelocitytmp[threadIdx.x],gammav[j+(threadIdx.x&3)],igammav[j+(threadIdx.x&3)]);\
                }\
              }\
              __syncthreads();\
              if(threadIdx.x+ilocal+(i-threadIdx.x)<count) {\
                xvelocity[threadIdx.x+ilocal]+=xvelocitytmp[threadIdx.x<<2]+xvelocitytmp[(threadIdx.x<<2)+1]+xvelocitytmp[(threadIdx.x<<2)+2]+xvelocitytmp[(threadIdx.x<<2)+3];\
                CANSWER6/*yvelocity[threadIdx.x+ilocal]+=yvelocitytmp[threadIdx.x<<2]+yvelocitytmp[(threadIdx.x<<2)+1]+yvelocitytmp[(threadIdx.x<<2)+2]+yvelocitytmp[(threadIdx.x<<2)+3];*/\
              }\
            }\
            addstart=0;\
            __syncthreads();\
            continue;\
          }\
          if(k+threadIdx.x<end) {/*If the list is not full, add the points and continue to next loop*/\
            TEXTUREFETCH(vxpositions[addstart+threadIdx.x],tzr,k+threadIdx.x);\
            TEXTUREFETCH(vypositions[addstart+threadIdx.x],tzi,k+threadIdx.x);\
            TEXTUREFETCH(gammav[addstart+threadIdx.x],tmr,k+threadIdx.x);\
            ISTRING2 /*Fix this one*/\
          }\
          if(k+blockDim.x<end)/*update addstart with the number of points added this loop*/\
            addstart+=blockDim.x;/*for sourcemultiplier==1, this is really unnecessary (but ifdefs cannot be used inside defines)*/\
          else\
            addstart+=end-k;\
        } /* end loop for k=begin:blockdim:end */\
      } /* end loop for all interaction boxes */\
      if(i<count) {\
        xvelocities[i]+=xvelocity[threadIdx.x];/*add the velocity*/\
        CANSWER3\
      }\
    } /* end loop for all points */\ 
    __syncthreads();\
  } /* for each box in the thread block */
    //for direct summation (tol==0) some simplifications can be made, since there are no mesh
#define SYNCHRONIZEDKERNELDIRECTSUM(kernel,tevalr,tevali,evalcount,ISTRING1,ISTRING2,CANSWER1,CANSWER2,CANSWER3) int i,j,k;\
  double sqrdist,xdist,ydist,xvelocity;\
  CANSWER1\
  __shared__ double vxpositions[DIRECTMAXTHREADS];\
  __shared__ double vypositions[DIRECTMAXTHREADS];\
  __shared__ double gammav[DIRECTMAXTHREADS];\
  ISTRING1\
  __shared__ double xpositions[DIRECTMAXTHREADS];/*makes code approx 1% slower for heavy load, but for some reason, doubles has to be fetched into an array*/\
  __shared__ double ypositions[DIRECTMAXTHREADS];\
  __shared__ int loopmax,vcount;\
  if(threadIdx.x==0) {\
    loopmax=evalcount+blockDim.x-(evalcount+blockDim.x-1)%blockDim.x-1;\
    vcount=count%blockDim.x;\
  }\
  __syncthreads();\
  for(i=threadIdx.x+blockIdx.x*blockDim.x;i<loopmax;i+=blockDim.x*gridDim.x) { /*if more points than threads*/\
    if(i<evalcount) {\
      xvelocity=0; /*use local variables as much as possible to help the optimizer understand that they are constant*/\
      /*yvelocity=0;*/CANSWER2\
      TEXTUREFETCH(xpositions[threadIdx.x],tevalr,i);\
      TEXTUREFETCH(ypositions[threadIdx.x],tevali,i);\
    }\
    for(k=0;k+blockDim.x<=count;k+=blockDim.x) {\
      TEXTUREFETCH(vxpositions[threadIdx.x],tzr,k+threadIdx.x);\
      TEXTUREFETCH(vypositions[threadIdx.x],tzi,k+threadIdx.x);\
      TEXTUREFETCH(gammav[threadIdx.x],tmr,k+threadIdx.x);\
      ISTRING2\
      __syncthreads();\
      if(i<evalcount) {\
        for(j=0;j<blockDim.x;j++) {\
          xdist=vxpositions[j]-xpositions[threadIdx.x];\
          ydist=vypositions[j]-ypositions[threadIdx.x];\
          kernel(xvelocity,yvelocity,circ,icirc);\
        }\
      }\
      __syncthreads();\
    }\
    if(k+threadIdx.x<count) {\
      TEXTUREFETCH(vxpositions[threadIdx.x],tzr,k+threadIdx.x);\
      TEXTUREFETCH(vypositions[threadIdx.x],tzi,k+threadIdx.x);\
      TEXTUREFETCH(gammav[threadIdx.x],tmr,k+threadIdx.x);\
      ISTRING2\
    }\
    __syncthreads();\
    if(i<evalcount) {\
      for(j=0;j<vcount;j++) {\
        xdist=vxpositions[j]-xpositions[threadIdx.x];\
        ydist=vypositions[j]-ypositions[threadIdx.x];\
        kernel(xvelocity,yvelocity,circ,icirc);\
      }\
      xvelocities[i]=xvelocity;\
      CANSWER3\
    }\
    __syncthreads();\
  }
#define circ gammav[j]
#define icirc igammav[j]

//choose between the old and new SYNCRONIZEDKERNEL by changing the defines
#ifdef SYNCKERNELOLD
#define IKERNELSTR2 IKERNELSTRING2
#define CANS1 COMPLEXANSWER1
#define CANS2 COMPLEXANSWER2
#define CANS3 COMPLEXANSWER3
#define CANS4
#define CANS5
#define CANS6
#define SYNCKERNEL SYNCHRONIZEDKERNEL

#else
#define IKERNELSTR2 IKERNELSTRING2_2
#define CANS1 COMPLEXANSWER1_2
#define CANS2 COMPLEXANSWER2_2
#define CANS3 COMPLEXANSWER3_2
#define CANS4 COMPLEXANSWER4_2
#define CANS5 COMPLEXANSWER5_2
#define CANS6 COMPLEXANSWER6_2
#define SYNCKERNEL SYNCHRONIZEDKERNEL2
#endif
//create all the functions
//there are one version that reads data from evaluation points, and one that reads from potential point.
__global__ void pot1_nomi_dirac_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir )
{
  SYNCKERNEL(DIRACSYNCKERNEL,ter,tei,tjxptr,DUMMYKERNELSTRING,DUMMYKERNELSTRING,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot1_mi_dirac_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir )
{
  SYNCKERNEL(IDIRACSYNCKERNEL,ter,tei,tjxptr,IKERNELSTRING1,IKERNELSTR2,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot1_nomi_rankine_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape )
{
  SYNCKERNEL(RANKINESYNCKERNEL,ter,tei,tjxptr,DUMMYKERNELSTRING,DUMMYKERNELSTRING,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot1_mi_rankine_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape )
{
  SYNCKERNEL(IRANKINESYNCKERNEL,ter,tei,tjxptr,IKERNELSTRING1,IKERNELSTR2,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot1_nomi_scully_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double scale,double cutoffsqr )
{
  SYNCKERNEL(SCULLYSYNCKERNEL,ter,tei,tjxptr,DUMMYKERNELSTRING,DUMMYKERNELSTRING,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot1_mi_scully_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double scale,double cutoffsqr )
{
  SYNCKERNEL(ISCULLYSYNCKERNEL,ter,tei,tjxptr,IKERNELSTRING1,IKERNELSTR2,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot1_nomi_oseen_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double scale,double cutoffsqr )
{
  SYNCKERNEL(OSEENSYNCKERNEL,ter,tei,tjxptr,DUMMYKERNELSTRING,DUMMYKERNELSTRING,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot1_mi_oseen_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double scale,double cutoffsqr )
{
  SYNCKERNEL(IOSEENSYNCKERNEL,ter,tei,tjxptr,IKERNELSTRING1,IKERNELSTR2,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}

//log kernels
__global__ void pot0_nomi_dirac_synchronized(double* xvelocities,int Nf,int* jcptr,int* j2cptr,int* ir )
{
// this is the one to look at.
  SYNCKERNEL(DIRACLOGSYNCKERNEL,ter,tei,tjxptr,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING);
}
__global__ void pot0_mi_dirac_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir )
{
  SYNCKERNEL(IDIRACLOGSYNCKERNEL,ter,tei,tjxptr,IKERNELSTRING1,IKERNELSTR2,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot0_nomi_rankine_synchronized(double* xvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double cutoffsqr )
{
  SYNCKERNEL(RANKINELOGSYNCKERNEL,ter,tei,tjxptr,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING);
}
__global__ void pot0_mi_rankine_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double cutoffsqr )
{
  SYNCKERNEL(IRANKINELOGSYNCKERNEL,ter,tei,tjxptr,IKERNELSTRING1,IKERNELSTR2,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot0_nomi_scully_synchronized(double* xvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double scale,double cutoffsqr )
{
  SYNCKERNEL(SCULLYLOGSYNCKERNEL,ter,tei,tjxptr,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING);
}
__global__ void pot0_mi_scully_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double scale,double cutoffsqr )
{
  SYNCKERNEL(ISCULLYLOGSYNCKERNEL,ter,tei,tjxptr,IKERNELSTRING1,IKERNELSTR2,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}
__global__ void pot0_nomi_oseen_synchronized(double* xvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double scale,double cutoffsqr )
{
  SYNCKERNEL(OSEENLOGSYNCKERNEL,ter,tei,tjxptr,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING);
}
__global__ void pot0_mi_oseen_synchronized(double* xvelocities,double* yvelocities,int Nf,int* jcptr,int* j2cptr,int* ir,double shape,double scale,double cutoffsqr )
{
  SYNCKERNEL(IOSEENLOGSYNCKERNEL,ter,tei,tjxptr,IKERNELSTRING1,IKERNELSTR2,CANS1,CANS2,CANS3,CANS4,CANS5,CANS6);
}

//direct eval kernels
__global__ void direct_pot1_nomi_dirac_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount)
{
  SYNCHRONIZEDKERNELDIRECTSUM(DIRACSYNCKERNEL,ter,tei,evalcount,DUMMYKERNELSTRING,DUMMYKERNELSTRING,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot1_mi_dirac_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount)
{
  SYNCHRONIZEDKERNELDIRECTSUM(IDIRACSYNCKERNEL,ter,tei,evalcount,IKERNELSTRING1DIRECT,IKERNELSTRING2,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot1_nomi_rankine_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape)
{
  SYNCHRONIZEDKERNELDIRECTSUM(RANKINESYNCKERNEL,ter,tei,evalcount,DUMMYKERNELSTRING,DUMMYKERNELSTRING,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot1_mi_rankine_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape)
{
  SYNCHRONIZEDKERNELDIRECTSUM(IRANKINESYNCKERNEL,ter,tei,evalcount,IKERNELSTRING1DIRECT,IKERNELSTRING2,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot1_nomi_scully_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape,double scale,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(SCULLYSYNCKERNEL,ter,tei,evalcount,DUMMYKERNELSTRING,DUMMYKERNELSTRING,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot1_mi_scully_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape,double scale,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(ISCULLYSYNCKERNEL,ter,tei,evalcount,IKERNELSTRING1DIRECT,IKERNELSTRING2,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot1_nomi_oseen_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape,double scale,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(OSEENSYNCKERNEL,ter,tei,evalcount,DUMMYKERNELSTRING,DUMMYKERNELSTRING,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot1_mi_oseen_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape,double scale,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(IOSEENSYNCKERNEL,ter,tei,evalcount,IKERNELSTRING1DIRECT,IKERNELSTRING2,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
//log kernels
__global__ void direct_pot0_nomi_dirac_synchronized(double* xvelocities,int count,int evalcount)
{
  SYNCHRONIZEDKERNELDIRECTSUM(DIRACLOGSYNCKERNEL,ter,tei,evalcount,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING);
}
__global__ void direct_pot0_mi_dirac_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount)
{
  SYNCHRONIZEDKERNELDIRECTSUM(IDIRACLOGSYNCKERNEL,ter,tei,evalcount,IKERNELSTRING1DIRECT,IKERNELSTRING2,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot0_nomi_rankine_synchronized(double* xvelocities,int count,int evalcount,double shape,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(RANKINELOGSYNCKERNEL,ter,tei,evalcount,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING);
}
__global__ void direct_pot0_mi_rankine_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(IRANKINELOGSYNCKERNEL,ter,tei,evalcount,IKERNELSTRING1DIRECT,IKERNELSTRING2,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot0_nomi_scully_synchronized(double* xvelocities,int count,int evalcount,double shape,double scale,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(SCULLYLOGSYNCKERNEL,ter,tei,evalcount,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING);
}
__global__ void direct_pot0_mi_scully_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape,double scale,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(ISCULLYLOGSYNCKERNEL,ter,tei,evalcount,IKERNELSTRING1DIRECT,IKERNELSTRING2,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
__global__ void direct_pot0_nomi_oseen_synchronized(double* xvelocities,int count,int evalcount,double shape,double scale,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(OSEENLOGSYNCKERNEL,ter,tei,evalcount,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING,DUMMYKERNELSTRING);
}
__global__ void direct_pot0_mi_oseen_synchronized(double* xvelocities,double* yvelocities,int count,int evalcount,double shape,double scale,double cutoffsqr)
{
  SYNCHRONIZEDKERNELDIRECTSUM(IOSEENLOGSYNCKERNEL,ter,tei,evalcount,IKERNELSTRING1DIRECT,IKERNELSTRING2,COMPLEXANSWER1,COMPLEXANSWER2,COMPLEXANSWER3DIRECT);
}
#undef circ


#define tevalr ter
#define tevali tei

//reorder functions. Four different versions exists, which takes 1, 2, 3 and 4 vectors for reordering
__global__ void reorder_cuda_1(double* output1, double* input1, int* index, int count) {
  int localindex;
  for(int i=blockDim.x * blockIdx.x + threadIdx.x;i<count;i+=blockDim.x*gridDim.x) {
    localindex=index[i];
    output1[i]=input1[localindex];
  }
}
__global__ void reorder_cuda_2(double* output1, double* output2, double* input1, double* input2, int* index, int count) {
  int localindex;
  for(int i=blockDim.x * blockIdx.x + threadIdx.x;i<count;i+=blockDim.x*gridDim.x) {
    localindex=index[i];
    output1[i]=input1[localindex];
    output2[i]=input2[localindex];
  }
}
__global__ void reorder_cuda_3(double* output1, double* output2, double* output3, double* input1, double* input2, double* input3, int* index, int count) {
  int localindex;
  for(int i=blockDim.x * blockIdx.x + threadIdx.x;i<count;i+=blockDim.x*gridDim.x) {
    localindex=index[i];
    output1[i]=input1[localindex];
    output2[i]=input2[localindex];
    output3[i]=input3[localindex];
  }
}
__global__ void reorder_cuda_4(double* output1, double* output2, double* output3, double* output4, double* input1, double* input2, double* input3, double* input4, int* index, int count) {
  int localindex;
  for(int i=blockDim.x * blockIdx.x + threadIdx.x;i<count;i+=blockDim.x*gridDim.x) {
    localindex=index[i];
    output1[i]=input1[localindex];
    output2[i]=input2[localindex];
    output3[i]=input3[localindex];
    output4[i]=input4[localindex];
  }
}

//as above, but reorders them in the opposite direction
__global__ void reorder_cuda_1_inv(double* output1, double* input1, int* index, int count) {
  int localindex;
  for(int i=blockDim.x * blockIdx.x + threadIdx.x;i<count;i+=blockDim.x*gridDim.x) {
    localindex=index[i];
    output1[localindex]=input1[i];
  }
}
__global__ void reorder_cuda_2_inv(double* output1, double* output2, double* input1, double* input2, int* index, int count) {
  int localindex;
  for(int i=blockDim.x * blockIdx.x + threadIdx.x;i<count;i+=blockDim.x*gridDim.x) {
    localindex=index[i];
    output1[localindex]=input1[i];
    output2[localindex]=input2[i];
  }
}
__global__ void reorder_cuda_3_inv(double* output1, double* output2, double* output3, double* input1, double* input2, double* input3, int* index, int count) {
  int localindex;
  for(int i=blockDim.x * blockIdx.x + threadIdx.x;i<count;i+=blockDim.x*gridDim.x) {
    localindex=index[i];
    output1[localindex]=input1[i];
    output2[localindex]=input2[i];
    output3[localindex]=input3[i];
  }
}
__global__ void reorder_cuda_4_inv(double* output1, double* output2, double* output3, double* output4, double* input1, double* input2, double* input3, double* input4, int* index, int count) {
  int localindex;
  for(int i=blockDim.x * blockIdx.x + threadIdx.x;i<count;i+=blockDim.x*gridDim.x) {
    localindex=index[i];
    output1[localindex]=input1[i];
    output2[localindex]=input2[i];
    output3[localindex]=input3[i];
    output4[localindex]=input4[i];
  }
}

#endif // cudaevalkernels_h
