/* fmm.h -- Fast multipole method for 2-D potentials in free space */

/* S. Engblom 2009-10-08 (Revision) */
/* S. Engblom 2009-04-15 (Major revision, adaptivity) */
/* S. Engblom 2007-07-08 (Major revision, performance) */
/* S. Engblom 2007-06-19 */

#ifndef __fmm_h
#define __fmm_h

#include <math.h>
#include <complex.h>
#include <stdbool.h>

#include "tl.h"


// obsolete, but for now...
#define MALLOC mxMalloc
#define CALLOC mxCalloc
#define FREE mxFree

//typedef double complex dcmplx; // not C++ compatible

#include "common.h"

// for performance experiments:
#include <time.h>
#include <stdio.h>

static struct timeval TIME_before, TIME_after;
typedef struct times_struct
{
   double loadbalance;
   double partition, connectivity;
   double P2M, M2M, ML2L, L2P, P2P;   
} times_t;
static times_t times;
#define StartTime gettimeofday(&TIME_before, NULL)
#define StopTime gettimeofday(&TIME_after, NULL)
#define GetTime (double)(TIME_after.tv_sec-TIME_before.tv_sec)+ \
		(TIME_after.tv_usec-TIME_before.tv_usec)/1000000.0		


#include "mex.h"

/* Criterion for near/far approximation: two boxes interact through
   cluster-cluster approximations provided that the function
   mpexp_theta() below returns true. Generally, small theta (say, less
   than 0.5) means that less coefficients are needed but that the
   interaction lists get larger, and vice versa. */
/*#ifndef FMM_THETA
#define FMM_THETA 0.5
#endif
const static double theta = FMM_THETA; // 0 < theta < 1
*/

/* Criterion for dividing a box. If one side is more than meshratio
   times the other, then the longer side is split into two. Otherwise
   it is split according to an ordering which yields good cache
   effects. */
#ifndef FMM_MESHRATIO
#define FMM_MESHRATIO 2.0
#endif
const static double meshratio = FMM_MESHRATIO; // 1.0 <= meshratio



/* Arguments to tasks */
typedef struct {
  int j, nlevel, startlevel, *lptr, nb;  
  const mpSparse *connect;
  mpexp *root;
} task_down_args;

typedef struct {
  int j, nlevel, startlevel, *lptr;  
  const mpSparse *connect;
  const mpexp *root;
  int *restrict ix, *restrict jx, *ixptr, *jxptr;  
  int N, NE, Nmax, Nmax0, Ndirect, cstride, pstride;
  const double *zr, *zi, *er, *ei;
} task_partition_args;

typedef struct {
  int index, jbegin, jend, pot, Nf;
  MPexp *This;
  const mpSparse *C;
  mpexp *offset;
  const double *zr, *zi, *mr, *mi;
} task_init_args;

/* Evaluation by the fast multipole method to accuracy TOL of the
   potential from N pointmasses (mr,mi) at the positions (zr,zi). */
void fmm2dInteract(int N,
		   const double *zr,const double *zi,
		   const double *restrict mr,const double *restrict mi,
		   int NE,
		   const double *er,const double *ei,
		   double *restrict pr,double *restrict pi,
		   double *qr,double *qi,
		   int pot,double tol,int Ndirect,double theta_,double eta,
		   SMOOTHER smooth,double xopt,double cutoff,bool cont,double *times);

/* Returns the multipole mesh and sparse connection matrices. For
   debugging and pedagogical purposes. */
void fmm2dGetMesh(int nlhs,mxArray *plhs[],
		  int N,
		  const double *zr,const double *zi,
		  const double *mr,const double *mi,
		  int NE,
		  const double *er,const double *ei,
		  int pot,double tol,int Ndirect,double theta_,double eta);

#endif /* __fmm_h */
